--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 253 - Synthesizable DDR SDRAM Controller
--
--*******************************************************************************
--
--  File name :       ddr1_top
--
--  Description :     
--                    Main DDR SDRAM controller block. This includes the following
--                    features:
--                    - The main controller state machine that controlls the 
--                    initialization process upon power up, as well as the 
--                    read, write, and refresh commands. 
--                    - handles the data path during READ and WRITEs.
--                    - Generates control signals for other modules, including the
--                      data strobe(DQS) signal
--
--  Date - revision : 05/01/2002
--
--  Author :          Lakshmi Gopalakrishnan
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************
library ieee;
use ieee.std_logic_1164.all;
--library synplify; 
--use synplify.attributes.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--

entity ddr1_top is
port(
     SYS_CLK               : in std_logic;
     SYS_CLKb              : in std_logic;
     dip1                  : in std_logic;   	
     rst_dqs_div_in   	   : in std_logic;
     rst_dqs_div_in2       : in std_logic;
     rst_dqs_div_out  	   : out std_logic;
     rst_dqs_div_out2      : out std_logic;
     reset_in              : in std_logic;   
     user_input_data       : in std_logic_vector(63 downto 0);
     user_output_data      : out std_logic_vector(63 downto 0):=(OTHERS => 'Z');
     user_data_valid       : out std_logic;
     user_input_address    : in std_logic_vector(21 downto 0);
     user_bank_address     : in std_logic_vector(1 downto 0);
     user_config_register  : in std_logic_vector(9 downto 0);
     user_command_register : in std_logic_vector(2 downto 0);
     user_cmd_ack          : out std_logic;      
     burst_done            : in std_logic;
     read_out_data1        : in std_logic_vector(63 downto 0);
     pass1                 : in std_logic;
     init_val              : out std_logic;
     ar_done               : out std_logic;
     ddr_dqs               : inout std_logic_vector(3 downto 0);
     ddr_dq                : inout std_logic_vector(31 downto 0):= (OTHERS => 'Z');signal 
     ddr_cke               : out std_logic;
     ddr_csb               : out std_logic;
     ddr_rasb              : out std_logic;
     ddr_casb              : out std_logic;
     ddr_web               : out std_logic;
     ddr_dm                : out std_logic_vector(3 downto 0); 
     ddr_ba                : out std_logic_vector(1 downto 0);
     ddr_address           : out std_logic_vector(12 downto 0);
     ddr1_clk0             : out std_logic;
     ddr1_clk0b            : out std_logic;
     ddr1_clk1             : out std_logic;
     ddr1_clk1b            : out std_logic;
     ddr1_clk2             : out std_logic;
     ddr1_clk2b            : out std_logic;
     ddr1_clk3             : out std_logic;
     ddr1_clk3b            : out std_logic;
     ddr1_clk4             : out std_logic;
     ddr1_clk4b            : out std_logic;
     dimm_clk0             : out std_logic;
     dimm_clk0b            : out std_logic;
     dimm_clk1             : out std_logic;
     dimm_clk1b            : out std_logic;
     dimm_clk2             : out std_logic;
     dimm_clk2b            : out std_logic; 
     JP1_header            : out std_logic_vector(7 downto 0);    
     JP2_header            : out std_logic_vector(7 downto 0);  
     clk_int_val           : out std_logic;
     clk90_int_val         : out std_logic; 
     sys_rst_val           : out std_logic;
     sys_rst90_val         : out std_logic;
     sys_rst180_val        : out std_logic;
     sys_rst270_val        : out std_logic 
     );
end ddr1_top;


architecture arc_ddr1_top of ddr1_top is
component controller 
port(
    dip1             : in std_logic;
    clk              : in std_logic;
    rst0             : in std_logic;
    rst180           : in std_logic;
    address          : in std_logic_vector(21 downto 0);
    bank_address     : in std_logic_vector(1 downto 0);
    config_register  : in std_logic_vector(9 downto 0);
    command_register : in std_logic_vector(2 downto 0);
    burst_done       : in std_logic;
    ddr_rasb_cntrl   : out std_logic;
    ddr_casb_cntrl   : out std_logic;
    ddr_web_cntrl    : out std_logic;
    ddr_ba_cntrl     : out std_logic_vector(1 downto 0);
    ddr_address_cntrl: out std_logic_vector(12 downto 0);
    ddr_cke_cntrl    : out std_logic;
    ddr_csb_cntrl    : out std_logic;
    dqs_enable       : out std_logic;
    dqs_reset        : out std_logic;
    write_enable     : out std_logic;
    rst_calib        : out std_logic;
    rst_dqs_div_int  : out std_logic;
    rst_dqs_div_int1  : out std_logic;
    cmd_ack          : out std_logic;
    init             : out std_logic;
    ar_done          : out std_logic
    
);
end component;                                    

component data_path
port(
     user_input_data    : in std_logic_vector(63 downto 0);
     clk                : in std_logic;
     clk90              : in std_logic;
     reset              : in std_logic;
     reset90            : in std_logic;
     reset180           : in std_logic;
     reset270           : in std_logic;
     write_enable       : in std_logic;
     rst_dqs_div_in     : in std_logic;
     rst_dqs_div_in2    : in std_logic;
     delay_sel          : in std_logic_vector(4 downto 0);   
     dqs_int_delay_in0  : in std_logic;
     dqs_int_delay_in1  : in std_logic;
     dqs_int_delay_in2  : in std_logic;
     dqs_int_delay_in3  : in std_logic;
     dq                 : in std_logic_vector(31 downto 0);       
     u_data_val         : out std_logic;
     user_output_data   : out std_logic_vector(63 downto 0);
     write_en_val       : out std_logic;
     write_en_val1      : out std_logic;
     reset90_r_val     : out std_logic;
     data_mask_f        : out std_logic_vector(3 downto 0);
     data_mask_r        : out std_logic_vector(3 downto 0);
     write_data_falling : out std_logic_vector(31 downto 0);
     write_data_rising  : out std_logic_vector(31 downto 0)
     );
end component;


component infrastructure
port(
     reset_in           : in std_logic;
     sys_clk_ibuf       : in std_logic;
     read_out_data1     : in std_logic_vector(63 downto 0);
     pass1              : in std_logic;    
     rst_calib1         : in std_logic;
     delay_sel_val1_val     : out std_logic_vector(4 downto 0);
     sys_rst_val            : out std_logic;
     sys_rst90_val          : out std_logic;
     sys_rst180_val         : out std_logic;
     sys_rst270_val         : out std_logic;
     clk_int_val            : out std_logic;
     clk90_int_val          : out std_logic;
     JP1_header_infra   : out std_logic_vector(7 downto 0);
     JP2_header_infra   : out std_logic_vector(7 downto 0)     
    );
end component;


component iobs
port(
     SYS_CLK           : in std_logic;
     SYS_CLKb          : in std_logic;
     clk               : in std_logic;
     clk90             : in std_logic;  
     JP1_header_infra  : in std_logic_vector(7 downto 0);
     JP2_header_infra  : in std_logic_vector(7 downto 0); 
     ddr_rasb_cntrl    : in std_logic;
     ddr_casb_cntrl    : in std_logic;
     ddr_web_cntrl     : in std_logic;
     ddr_cke_cntrl     : in std_logic;
     ddr_csb_cntrl     : in std_logic;
     ddr_address_cntrl : in std_logic_vector(12 downto 0);
     ddr_ba_cntrl      : in std_logic_vector(1 downto 0);
     rst_dqs_div_int   : in std_logic;
     rst_dqs_div_int1   : in std_logic;
     dqs_reset         : in std_logic;
     dqs_enable        : in std_logic;   
     ddr_dqs           : inout std_logic_vector(3 downto 0);
     ddr_dq            : inout std_logic_vector(31 downto 0);
     write_data_falling: in std_logic_vector(31 downto 0);
     write_data_rising : in std_logic_vector(31 downto 0);
     write_en_val      : in std_logic;
     write_en_val1       : in std_logic;
     reset90_r        : in std_logic;
     data_mask_f       : in std_logic_vector(3 downto 0);
     data_mask_r       : in std_logic_vector(3 downto 0); 
     sys_clk_ibuf      : out std_logic;
     sys_clkb_ibuf     : out std_logic;
     ddr1_clk0         : out std_logic;
     ddr1_clk0b        : out std_logic;
     ddr1_clk1         : out std_logic;
     ddr1_clk1b        : out std_logic;
     ddr1_clk2         : out std_logic;
     ddr1_clk2b        : out std_logic;
     ddr1_clk3         : out std_logic;
     ddr1_clk3b        : out std_logic;
     ddr1_clk4         : out std_logic;
     ddr1_clk4b        : out std_logic;
     dimm_clk0         : out std_logic;
     dimm_clk0b        : out std_logic;
     dimm_clk1         : out std_logic;
     dimm_clk1b        : out std_logic;
     dimm_clk2         : out std_logic;
     dimm_clk2b        : out std_logic; 
     JP1_header        : out std_logic_vector(7 downto 0);
     JP2_header        : out std_logic_vector(7 downto 0);     
     ddr_rasb          : out std_logic;
     ddr_casb          : out std_logic;
     ddr_web           : out std_logic;
     ddr_ba            : out std_logic_vector(1 downto 0);
     ddr_address       : out std_logic_vector(12 downto 0);
     ddr_cke           : out std_logic;
     ddr_csb           : out std_logic;
     rst_dqs_div       : out std_logic;
     rst_dqs_div2      : out std_logic; 
     rst_dqs_div_in    : in std_logic;
     rst_dqs_div_in2   : in std_logic;
     rst_dqs_div_out   : out std_logic;
     rst_dqs_div_out2  : out std_logic;
     dqs_int_delay_in0 : out std_logic;
     dqs_int_delay_in1 : out std_logic;
     dqs_int_delay_in2 : out std_logic;
     dqs_int_delay_in3 : out std_logic;
     dq                : out std_logic_vector(31 downto 0);
     ddr_dm            : out std_logic_vector(3 downto 0)  
    );
end component;



-------
signal sys_clk_ibuf       : std_logic;
signal rst_calib          : std_logic;
signal delay_sel          : std_logic_vector(4 downto 0);
signal sys_rst            : std_logic ;
signal sys_rst90          : std_logic;
signal sys_rst180         : std_logic ;
signal sys_rst270         : std_logic ;
signal clk_int            : std_logic;
signal clk90_int          : std_logic;
signal JP1_header_infra   : std_logic_vector(7 downto 0);
signal JP2_header_infra   : std_logic_vector(7 downto 0);

-----

signal write_enable       : std_logic;
signal dqs_div_rst        : std_logic;
signal dqs_div_rst2       : std_logic;
signal dqs_enable         : std_logic;
signal dqs_reset          : std_logic;
signal dqs_int_delay_in0  : std_logic;
signal dqs_int_delay_in1  : std_logic;
signal dqs_int_delay_in2  : std_logic;
signal dqs_int_delay_in3  : std_logic;
signal dq                 : std_logic_vector(31 downto 0);
signal u_data_val         : std_logic;
--- signal user_output_data   : std_logic_vector(63 downto 0);
signal write_en_val       : std_logic;
signal write_en_val1       : std_logic;
signal reset90_r         : std_logic;
signal data_mask_f        : std_logic_vector(3 downto 0);
signal data_mask_r        : std_logic_vector(3 downto 0);
signal write_data_falling : std_logic_vector(31 downto 0);
signal write_data_rising  : std_logic_vector(31 downto 0);

-----

signal ddr_rasb_cntrl   :  std_logic;
signal ddr_casb_cntrl   :  std_logic;
signal ddr_web_cntrl    :  std_logic;
signal ddr_ba_cntrl     :  std_logic_vector(1 downto 0);
signal ddr_address_cntrl:  std_logic_vector(12 downto 0);
signal ddr_cke_cntrl    :  std_logic;
signal ddr_csb_cntrl    :  std_logic;
signal rst_dqs_div_int  :  std_logic;
signal rst_dqs_div_int1   : std_logic;


begin


sys_rst_val <= sys_rst;
sys_rst90_val <= sys_rst90;
sys_rst180_val <= sys_rst180;
sys_rst270_val <= sys_rst270;
clk_int_val <= clk_int;
clk90_int_val <= clk90_int;



controller0 : controller port map (
                                   dip1               => dip1,
                                   clk                => clk_int, 
                                   rst0               => sys_rst,
                                   rst180             => sys_rst180,
                                   address            => user_input_address,
                                   bank_address       => user_bank_address,
                                   config_register    => user_config_register,
                                   command_register   => user_command_register,
                                   burst_done         => burst_done,
                                   ddr_rasb_cntrl     => ddr_rasb_cntrl,
                                   ddr_casb_cntrl     => ddr_casb_cntrl,
                                   ddr_web_cntrl      => ddr_web_cntrl,
                                   ddr_ba_cntrl       => ddr_ba_cntrl,
                                   ddr_address_cntrl  => ddr_address_cntrl,
                                   ddr_cke_cntrl      => ddr_cke_cntrl,
                                   ddr_csb_cntrl      => ddr_csb_cntrl,
                                   dqs_enable         => dqs_enable,
                                   dqs_reset          => dqs_reset,
                                   write_enable       => write_enable,
                                   rst_calib          => rst_calib,
                                   rst_dqs_div_int    => rst_dqs_div_int,
                                   rst_dqs_div_int1    => rst_dqs_div_int1,
                                   cmd_ack            => user_cmd_ack,
                                   init               => init_val,
                                   ar_done            => ar_done
                                   
                                  );
                             
data_path0 : data_path port map ( 
                                 user_input_data     => user_input_data,
                                 clk                 => clk_int,
                                 clk90               => clk90_int,
                                 reset               => sys_rst,
                                 reset90             => sys_rst90,
                                 reset180            => sys_rst180,
                                 reset270            => sys_rst270,
                                 write_enable        => write_enable,
                                 rst_dqs_div_in      => dqs_div_rst,
                                 rst_dqs_div_in2     => dqs_div_rst2, 
                                 delay_sel           => delay_sel,
                                 dqs_int_delay_in0   => dqs_int_delay_in0,
                                 dqs_int_delay_in1   => dqs_int_delay_in1,
                                 dqs_int_delay_in2   => dqs_int_delay_in2,
                                 dqs_int_delay_in3   => dqs_int_delay_in3,
                                 dq                  => dq,      
                                 u_data_val          => user_data_valid,
                                 user_output_data    => user_output_data,
                                 write_en_val        => write_en_val,
                                 write_en_val1       => write_en_val1,
                                 reset90_r_val       => reset90_r,
                                 data_mask_f         => data_mask_f,
                                 data_mask_r         => data_mask_r,
                                 write_data_falling  => write_data_falling,
                                 write_data_rising   => write_data_rising                     
                                );                           
                          		
							
infrastructure0 : infrastructure port map
                                        (
                                         reset_in           =>   reset_in,
                                         sys_clk_ibuf       =>   sys_clk_ibuf,
                                         read_out_data1     =>   read_out_data1,
                                         pass1              =>   pass1,
                                         rst_calib1         =>   rst_calib,
                                         delay_sel_val1_val =>   delay_sel,
                                         sys_rst_val        =>   sys_rst,
                                         sys_rst90_val      =>   sys_rst90,
                                         sys_rst180_val     =>   sys_rst180,
                                         sys_rst270_val     =>   sys_rst270,
                                         clk_int_val        =>   clk_int,
                                         clk90_int_val      =>   clk90_int,
                                         JP1_header_infra   =>   JP1_header_infra,
                                         JP2_header_infra   =>   JP2_header_infra 
                                        );  
iobs0 : iobs port map 
                    (
                     SYS_CLK            =>  SYS_CLK,  
                     SYS_CLKb           =>  SYS_CLKb,
                     clk                =>  clk_int,
                     clk90              =>  clk90_int,
                     JP1_header_infra   =>  JP1_header_infra,
                     JP2_header_infra   =>  JP2_header_infra,
                     ddr_rasb_cntrl     =>  ddr_rasb_cntrl, 
                     ddr_casb_cntrl     =>  ddr_casb_cntrl,
                     ddr_web_cntrl      =>  ddr_web_cntrl,
                     ddr_cke_cntrl      =>  ddr_cke_cntrl,
                     ddr_csb_cntrl      =>  ddr_csb_cntrl,
                     ddr_address_cntrl  =>  ddr_address_cntrl,
                     ddr_ba_cntrl       =>  ddr_ba_cntrl,
                     rst_dqs_div_int    =>  rst_dqs_div_int,
                     rst_dqs_div_int1   =>  rst_dqs_div_int1,
                     dqs_reset          =>  dqs_reset,
                     dqs_enable         =>  dqs_enable,
                     ddr_dqs            =>  ddr_dqs,
                     ddr_dq             =>  ddr_dq,
                     write_data_falling =>  write_data_falling, 
                     write_data_rising  =>  write_data_rising,
                     write_en_val       =>  write_en_val,
                     write_en_val1      =>  write_en_val1,
                     reset90_r          =>  reset90_r,
                     data_mask_f        =>  data_mask_f, 
                     data_mask_r        =>  data_mask_r,
                     sys_clk_ibuf       =>  sys_clk_ibuf,
                     ddr1_clk0          =>  ddr1_clk0, 
                     ddr1_clk0b         =>  ddr1_clk0b,
                     ddr1_clk1          =>  ddr1_clk1,
                     ddr1_clk1b         =>  ddr1_clk1b,
                     ddr1_clk2          =>  ddr1_clk2,
                     ddr1_clk2b         =>  ddr1_clk2b,
                     ddr1_clk3          =>  ddr1_clk3,
                     ddr1_clk3b         =>  ddr1_clk3b,
                     ddr1_clk4          =>  ddr1_clk4,
                     ddr1_clk4b         =>  ddr1_clk4b,
                     dimm_clk0          =>  dimm_clk0,
                     dimm_clk0b         =>  dimm_clk0b,
                     dimm_clk1          =>  dimm_clk1,
                     dimm_clk1b         =>  dimm_clk1b,
                     dimm_clk2          =>  dimm_clk2,
                     dimm_clk2b         =>  dimm_clk2b,
                     JP1_header         =>  JP1_header,
                     JP2_header         =>  JP2_header,   
                     ddr_rasb           =>  ddr_rasb, 
                     ddr_casb           =>  ddr_casb,
                     ddr_web            =>  ddr_web,
                     ddr_ba             =>  ddr_ba,
                     ddr_address        =>  ddr_address,
                     ddr_cke            =>  ddr_cke,
                     ddr_csb            =>  ddr_csb,
                     rst_dqs_div        =>  dqs_div_rst, 
                     rst_dqs_div2       =>  dqs_div_rst2,                 
  		     rst_dqs_div_in	=>  rst_dqs_div_in,
                     rst_dqs_div_in2    =>  rst_dqs_div_in2, 
		     rst_dqs_div_out	=>  rst_dqs_div_out,
                     rst_dqs_div_out2   =>  rst_dqs_div_out2, 
                     dqs_int_delay_in0  =>  dqs_int_delay_in0,
                     dqs_int_delay_in1  =>  dqs_int_delay_in1,
                     dqs_int_delay_in2  =>  dqs_int_delay_in2,
                     dqs_int_delay_in3  =>  dqs_int_delay_in3, 
                     dq                 =>  dq,
                     ddr_dm             =>  ddr_dm
                    );							

end arc_ddr1_top;
                                                   
