library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--

entity addr_gen is
port(
   clk            : in std_logic;
   rst            : in std_logic;
   addr_rst       : in std_logic;
   addr_inc       : in std_logic;
   burst_type		  : in std_logic;
   addr_out       : out std_logic_vector(23 downto 0);
   column_address : out std_logic_vector(7 downto 0); 
   test_cnt_ena   : in std_logic;
   test_out       : out std_logic_vector(4 downto 0);
   burst_done     : out std_logic;
   cnt_roll       : out std_logic);
end addr_gen;


architecture arc_addr_gen of addr_gen is

attribute syn_noprune : boolean;  -- Using Syn_Keep Derictive
attribute syn_preserve : boolean;  -- Using Syn_Keep Derictive

signal addr_reg 			: std_logic_vector(22 downto 0); 
signal addr_int 			: std_logic_vector(22 downto 0);
signal test_cnt 			: std_logic_vector(4 downto 0);
signal testcnt_reg 		: std_logic_vector(3 downto 0);
signal column_counter 	: std_logic_vector(7 downto 0);
signal column_counter_cp 	: std_logic_vector(7 downto 0);
signal row_counter    	: std_logic_vector(7 downto 0);
signal ba_count       	: std_logic_vector(1 downto 0);
signal cnt            	: std_logic_vector(1 downto 0);
signal burst_done_reg   : std_logic;
signal burst_done_1_reg : std_logic;
signal cnt_roll_p     	: std_logic;
signal cnt_roll_p2    	:std_logic;

attribute syn_noprune of  column_counter: signal is true;
attribute syn_noprune of column_counter_cp : signal is true;

attribute syn_preserve of column_counter : signal is true;
attribute syn_preserve of column_counter_cp : signal is true;


begin

ba_count       <= "00"; 
column_address <= column_counter;

process(clk)
begin
  if clk'event and clk = '1' then
     if (rst = '1' or addr_rst = '1') then
        column_counter <= "00000000";
        column_counter_cp <= "00000000";
        row_counter    <= "00000000";
        cnt <= "00";
     elsif addr_inc = '1' then
       
       if(burst_type= '0') then    -- burst length =4
       		if cnt = "01" then 
            			cnt <= "00";
        		else
            			cnt <= cnt + "01";
        		end if;
        
        		if test_cnt_ena = '1' and cnt = "01" then 
        
          			 if column_counter = "11110000" then  --11110000
        
                 				column_counter <= "00000000";
                                                column_counter_cp <= "00000000";
           			else 
                				column_counter <= column_counter + "00000100";
                                                column_counter_cp <= column_counter_cp + "00000100";
           			end if;     
        		else 
            			column_counter <= column_counter;
                                column_counter_cp <= column_counter_cp;
        		end if;
        elsif (burst_type= '1') then   --burst length =8
       		if cnt = "11" then 
            			cnt <= "00";
        		else
            			cnt <= cnt + "01";
        		end if;
        
        		if test_cnt_ena = '1' and cnt = "11" then 
        
          			 if column_counter = "11110000" then  
        
                 				column_counter <= "00000000";
                                                column_counter_cp <= "00000000";
           			else 
                				column_counter <= column_counter + "00001000";
                                                column_counter_cp <= column_counter_cp + "00001000";
           			end if;     
        		else 
            			column_counter <= column_counter;
                                column_counter_cp <= column_counter_cp;
        		end if;
        end if;		
        
     end if;
  end if;
end process;


addr_out <= "00000000000100" & column_counter_cp & ba_count;

process(clk)
begin
 if clk'event and clk = '1' then
  if rst = '1' then
     burst_done_reg <= '0';
     burst_done_1_reg <= '0';
  elsif (column_counter_cp = "11101100" and burst_type= '0') then   --burst length =4   --11101100
     burst_done_reg <= '1';
  elsif (column_counter_cp = "11101000" and burst_type= '1') then   --burst length =8
     burst_done_reg <= '1';
     
  else
     burst_done_reg <= '0';
  end if;
  burst_done_1_reg <= burst_done_reg;
 end if;
end process;


burst_done <= burst_done_1_reg;

process(clk)
begin
 if clk'event and clk = '1' then
  if rst = '1' then
     cnt_roll_p <= '0';
  elsif (column_counter_cp = "11101100" and burst_type= '0') then  --burst length =4 
     cnt_roll_p <= '1';
  elsif (column_counter_cp = "11101000" and burst_type= '1') then  --burst length =8
     cnt_roll_p <= '1';
  else
     cnt_roll_p <= '0';
  end if;
 end if;
end process;

process(clk)
begin
 if clk'event and clk = '1' then
  if rst = '1' then
     cnt_roll_p2 <= '0';
     cnt_roll    <= '0';
  else
     cnt_roll_p2  <= cnt_roll_p;
     cnt_roll <= cnt_roll_p2;
  end if;
 end if;
end process;

end arc_addr_gen;
							
	

