--*********************************************************************
-- DDR 72 Bit Controller DATA PATH for LEFT RIGHT Pins

-- In the current DATA PATH logic DATA CAPTURE part was modified.
-- The below changes were made to reduce the resources in 
-- the data capture

-- in the current architecture data ( dq ) from ddr memory 
-- directly stored into the FIFO's.

-- Architectural changes :

-- Used only TWO FIFOs ( instead of FOUR FIFOs ) 
-- Used Single col ( col0 ) dqs_delayed_col signals
-- Used Gray Counters for write and read pointers of the FIFOs 

-- fbit stage is removed from ddr1_dqbit module ( in the data capture )
-- dq_clk stage was removed 
-- dqs_clk_div logic was removed
-- ddr1_transfer_done logic was removed 
-- data valid signals registering in clk90 domain was removed

-- fifo_0 and fifo_1 wr_addr was double registered in clk90 domain
-- only fifo_0 and fifo_1 empty signals were used for read_data_valid_1 logic  
-- write enable for the FIFOs derived from rst_dqs_div signal


-- Code revised by 	: Narayana Murty.
-- Date 			      : Nov 18, 2003. 

--*********************************************************************


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--library synplify; 
--use synplify.attributes.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--

entity data_read is
port(

     clk90              : in std_logic;
     reset90_r          : in std_logic;
     ddr_dq_in          : in std_logic_vector(71 downto 0);   
     read_valid_data_1  : in std_logic;
     
     fifo_00_wr_en			: in std_logic;
     fifo_10_wr_en			: in std_logic;
     fifo_20_wr_en			: in std_logic;
     fifo_30_wr_en			: in std_logic;
     fifo_40_wr_en			: in std_logic;
     fifo_50_wr_en			: in std_logic;
     fifo_60_wr_en			: in std_logic;
     fifo_70_wr_en			: in std_logic;
     fifo_80_wr_en			: in std_logic;

     fifo_01_wr_en			: in std_logic;
     fifo_11_wr_en			: in std_logic;
     fifo_21_wr_en			: in std_logic;
     fifo_31_wr_en			: in std_logic;
     fifo_41_wr_en			: in std_logic;
     fifo_51_wr_en			: in std_logic;
     fifo_61_wr_en			: in std_logic;
     fifo_71_wr_en			: in std_logic;
     fifo_81_wr_en			: in std_logic;
		
     fifo_00_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_01_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_10_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_11_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_20_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_21_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_30_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_31_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_40_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_41_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_50_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_51_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_60_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_61_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_70_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_71_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_80_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_81_wr_addr    : in std_logic_vector(3 downto 0);

     fifo_00_wr_en_1			: in std_logic;
     fifo_10_wr_en_1			: in std_logic;
     fifo_20_wr_en_1			: in std_logic;
     fifo_30_wr_en_1			: in std_logic;
     fifo_40_wr_en_1			: in std_logic;
     fifo_50_wr_en_1			: in std_logic;
     fifo_60_wr_en_1			: in std_logic;
     fifo_70_wr_en_1			: in std_logic;
     fifo_80_wr_en_1			: in std_logic;

     fifo_01_wr_en_1			: in std_logic;
     fifo_11_wr_en_1			: in std_logic;
     fifo_21_wr_en_1			: in std_logic;
     fifo_31_wr_en_1			: in std_logic;
     fifo_41_wr_en_1			: in std_logic;
     fifo_51_wr_en_1			: in std_logic;
     fifo_61_wr_en_1			: in std_logic;
     fifo_71_wr_en_1			: in std_logic;
     fifo_81_wr_en_1			: in std_logic;
		
     fifo_00_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_01_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_10_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_11_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_20_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_21_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_30_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_31_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_40_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_41_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_50_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_51_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_60_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_61_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_70_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_71_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_80_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_81_wr_addr_1    : in std_logic_vector(3 downto 0);
		
     dqs0_delayed_col1  : in std_logic;
     dqs1_delayed_col1  : in std_logic;
     dqs2_delayed_col1  : in std_logic;
     dqs3_delayed_col1  : in std_logic;
     dqs4_delayed_col1  : in std_logic;
     dqs5_delayed_col1  : in std_logic;
     dqs6_delayed_col1  : in std_logic;
     dqs7_delayed_col1  : in std_logic;
     dqs8_delayed_col1  : in std_logic;
     
     dqs0_delayed_col0  : in std_logic;
     dqs1_delayed_col0  : in std_logic;
     dqs2_delayed_col0  : in std_logic;
     dqs3_delayed_col0  : in std_logic;
     dqs4_delayed_col0  : in std_logic;
     dqs5_delayed_col0  : in std_logic;
     dqs6_delayed_col0  : in std_logic;
     dqs7_delayed_col0  : in std_logic;
     dqs8_delayed_col0  : in std_logic;
     
     
     user_output_data   : out std_logic_vector(143 downto 0);
     fifo0_rd_addr_val: out std_logic_vector(3 downto 0);
     fifo1_rd_addr_val: out std_logic_vector(3 downto 0)     

     );
end data_read;


architecture arc_data_read of data_read is

attribute syn_noprune : boolean;  -- Using syn_noprune Derictive

attribute syn_preserve : boolean;  -- Using syn_noprune Derictive
-- rd_gray_cntr is a gray counter with a SYNC reset ( reset_90r) for fifo rd_addr
component rd_gray_cntr port (
			clk 				: 	in std_logic;         
			reset				:	in std_logic;
			cnt_en			        :	in std_logic; 
                        rgc_gcnt                       :       out	std_logic_vector(3 downto 0)
                        
		  );
end component; 

component FD port(
      Q        : out STD_LOGIC;
      C        : in STD_LOGIC;
      D        : in STD_LOGIC
      );
end component;

-- 16x1 Dual Port RAM Component Instansiated 

component RAM16X1D
  port (D     : in std_logic;
        WE    : in std_logic;
        WCLK  : in std_logic;
        A0    : in std_logic;
        A1    : in std_logic;
        A2    : in std_logic;
        A3    : in std_logic;
        DPRA0 : in std_logic;
        DPRA1 : in std_logic;
        DPRA2 : in std_logic;
        DPRA3 : in std_logic;
 		  SPO   : out std_logic;
        DPO   : out std_logic);

end component;


signal read_valid_data_1_r   : std_logic;
signal read_valid_data_1_r1   : std_logic;
signal read_valid_data_1_r2   : std_logic;



signal fifo00_rd_addr        : std_logic_vector(3 downto 0);
signal fifo01_rd_addr        : std_logic_vector(3 downto 0);


signal fifo00_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo01_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo10_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo11_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo20_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo21_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo30_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo31_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo40_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo41_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo50_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo51_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo60_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo61_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo70_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo71_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo80_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifo81_rd_addr_r        : std_logic_vector(3 downto 0);
signal fifop_rd_addr_r         : std_logic_vector(3 downto 0);


attribute syn_noprune of fifo00_rd_addr_r : signal is true;
attribute syn_noprune of fifo01_rd_addr_r : signal is true;
attribute syn_noprune of fifo10_rd_addr_r : signal is true;
attribute syn_noprune of fifo11_rd_addr_r : signal is true;
attribute syn_noprune of fifo20_rd_addr_r : signal is true;
attribute syn_noprune of fifo21_rd_addr_r : signal is true;
attribute syn_noprune of fifo30_rd_addr_r : signal is true;
attribute syn_noprune of fifo31_rd_addr_r : signal is true;
attribute syn_noprune of fifo40_rd_addr_r : signal is true;
attribute syn_noprune of fifo41_rd_addr_r : signal is true;
attribute syn_noprune of fifo50_rd_addr_r : signal is true;
attribute syn_noprune of fifo51_rd_addr_r : signal is true;
attribute syn_noprune of fifo60_rd_addr_r : signal is true;
attribute syn_noprune of fifo61_rd_addr_r : signal is true;
attribute syn_noprune of fifo70_rd_addr_r : signal is true;
attribute syn_noprune of fifo71_rd_addr_r : signal is true;
attribute syn_noprune of fifo80_rd_addr_r : signal is true;
attribute syn_noprune of fifo81_rd_addr_r : signal is true;
attribute syn_noprune of fifop_rd_addr_r  : signal is true;

attribute syn_preserve  of fifo00_rd_addr_r : signal is true;
attribute syn_preserve  of fifo01_rd_addr_r : signal is true;
attribute syn_preserve  of fifo10_rd_addr_r : signal is true;
attribute syn_preserve  of fifo11_rd_addr_r : signal is true;
attribute syn_preserve  of fifo20_rd_addr_r : signal is true;
attribute syn_preserve  of fifo21_rd_addr_r : signal is true;
attribute syn_preserve  of fifo30_rd_addr_r : signal is true;
attribute syn_preserve  of fifo31_rd_addr_r : signal is true;
attribute syn_preserve  of fifo40_rd_addr_r : signal is true;
attribute syn_preserve  of fifo41_rd_addr_r : signal is true;
attribute syn_preserve  of fifo50_rd_addr_r : signal is true;
attribute syn_preserve  of fifo51_rd_addr_r : signal is true;
attribute syn_preserve  of fifo60_rd_addr_r : signal is true;
attribute syn_preserve  of fifo61_rd_addr_r : signal is true;
attribute syn_preserve  of fifo70_rd_addr_r : signal is true;
attribute syn_preserve  of fifo71_rd_addr_r : signal is true;
attribute syn_preserve  of fifo80_rd_addr_r : signal is true;
attribute syn_preserve  of fifo81_rd_addr_r : signal is true;
attribute syn_preserve  of fifop_rd_addr_r  : signal is true;



signal fifo_00_data_out       : std_logic_vector(7 downto 0);
signal fifo_01_data_out       : std_logic_vector(7 downto 0);
signal fifo_10_data_out       : std_logic_vector(7 downto 0);
signal fifo_11_data_out       : std_logic_vector(7 downto 0);
signal fifo_20_data_out       : std_logic_vector(7 downto 0);
signal fifo_21_data_out       : std_logic_vector(7 downto 0);
signal fifo_30_data_out       : std_logic_vector(7 downto 0);
signal fifo_31_data_out       : std_logic_vector(7 downto 0);
signal fifo_40_data_out       : std_logic_vector(7 downto 0);
signal fifo_41_data_out       : std_logic_vector(7 downto 0);
signal fifo_50_data_out       : std_logic_vector(7 downto 0);
signal fifo_51_data_out       : std_logic_vector(7 downto 0);
signal fifo_60_data_out       : std_logic_vector(7 downto 0);
signal fifo_61_data_out       : std_logic_vector(7 downto 0);
signal fifo_70_data_out       : std_logic_vector(7 downto 0);
signal fifo_71_data_out       : std_logic_vector(7 downto 0);
signal fifo_80_data_out       : std_logic_vector(7 downto 0);
signal fifo_81_data_out       : std_logic_vector(7 downto 0);

-- reg added for timing 
signal fifo_00_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_01_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_10_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_11_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_20_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_21_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_30_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_31_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_40_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_41_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_50_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_51_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_60_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_61_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_70_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_71_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_80_data_out_r       : std_logic_vector(7 downto 0);
signal fifo_81_data_out_r       : std_logic_vector(7 downto 0);



signal first_sdr_data         : std_logic_vector(143 downto 0);

signal dqs0_delayed_col0_n	: std_logic;
signal dqs1_delayed_col0_n      : std_logic;
signal dqs2_delayed_col0_n	: std_logic;
signal dqs3_delayed_col0_n      : std_logic;
signal dqs4_delayed_col0_n	: std_logic;
signal dqs5_delayed_col0_n      : std_logic;
signal dqs6_delayed_col0_n	: std_logic;
signal dqs7_delayed_col0_n      : std_logic;
signal dqs8_delayed_col0_n	: std_logic;

-- Directive for synthesis   
--attribute syn_noprune of dqs0_delayed_col0_n : signal is true;
--attribute syn_noprune of dqs1_delayed_col0_n : signal is true;
--attribute syn_noprune of dqs2_delayed_col0_n : signal is true;
--attribute syn_noprune of dqs3_delayed_col0_n : signal is true;
--attribute syn_noprune of dqs4_delayed_col0_n : signal is true;
--attribute syn_noprune of dqs5_delayed_col0_n : signal is true;
--attribute syn_noprune of dqs6_delayed_col0_n : signal is true;
--attribute syn_noprune of dqs7_delayed_col0_n : signal is true;
--attribute syn_noprune of dqs8_delayed_col0_n : signal is true;

signal dqs0_delayed_col1_n	: std_logic;
signal dqs1_delayed_col1_n      : std_logic;
signal dqs2_delayed_col1_n	: std_logic;
signal dqs3_delayed_col1_n      : std_logic;
signal dqs4_delayed_col1_n	: std_logic;
signal dqs5_delayed_col1_n      : std_logic;
signal dqs6_delayed_col1_n	: std_logic;
signal dqs7_delayed_col1_n      : std_logic;
signal dqs8_delayed_col1_n	: std_logic;

-- Directive for synthesis   
--attribute syn_noprune of dqs0_delayed_col1_n : signal is true;
--attribute syn_noprune of dqs1_delayed_col1_n : signal is true;
--attribute syn_noprune of dqs2_delayed_col1_n : signal is true;
--attribute syn_noprune of dqs3_delayed_col1_n : signal is true;
--attribute syn_noprune of dqs4_delayed_col1_n : signal is true;
--attribute syn_noprune of dqs5_delayed_col1_n : signal is true;
--attribute syn_noprune of dqs6_delayed_col1_n : signal is true;
--attribute syn_noprune of dqs7_delayed_col1_n : signal is true;
--attribute syn_noprune of dqs8_delayed_col1_n : signal is true;




begin

dqs0_delayed_col0_n <= not dqs0_delayed_col0;
dqs1_delayed_col0_n <= not dqs1_delayed_col0;
dqs2_delayed_col0_n <= not dqs2_delayed_col0;
dqs3_delayed_col0_n <= not dqs3_delayed_col0;
dqs4_delayed_col0_n <= not dqs4_delayed_col0;
dqs5_delayed_col0_n <= not dqs5_delayed_col0;
dqs6_delayed_col0_n <= not dqs6_delayed_col0;
dqs7_delayed_col0_n <= not dqs7_delayed_col0;
dqs8_delayed_col0_n <= not dqs8_delayed_col0;


dqs0_delayed_col1_n <= not dqs0_delayed_col1;
dqs1_delayed_col1_n <= not dqs1_delayed_col1;
dqs2_delayed_col1_n <= not dqs2_delayed_col1;
dqs3_delayed_col1_n <= not dqs3_delayed_col1;
dqs4_delayed_col1_n <= not dqs4_delayed_col1;
dqs5_delayed_col1_n <= not dqs5_delayed_col1;
dqs6_delayed_col1_n <= not dqs6_delayed_col1;
dqs7_delayed_col1_n <= not dqs7_delayed_col1;
dqs8_delayed_col1_n <= not dqs8_delayed_col1;

user_output_data    <= first_sdr_data;


fifo0_rd_addr_val   <= fifo01_rd_addr;
fifo1_rd_addr_val   <= fifo00_rd_addr;


process(clk90)
begin
if clk90'event and clk90 = '1' then
 if reset90_r = '1' then
    fifo_00_data_out_r <= "00000000";
    fifo_01_data_out_r <= "00000000";
    fifo_10_data_out_r <= "00000000";
    fifo_11_data_out_r <= "00000000";
    fifo_20_data_out_r <= "00000000";
    fifo_21_data_out_r <= "00000000";
    fifo_30_data_out_r <= "00000000";
    fifo_31_data_out_r <= "00000000";
    fifo_40_data_out_r <= "00000000";
    fifo_41_data_out_r <= "00000000";
    fifo_50_data_out_r <= "00000000";
    fifo_51_data_out_r <= "00000000";
    fifo_60_data_out_r <= "00000000";
    fifo_61_data_out_r <= "00000000";
    fifo_70_data_out_r <= "00000000";
    fifo_71_data_out_r <= "00000000";
    fifo_80_data_out_r <= "00000000";
    fifo_81_data_out_r <= "00000000";
 else
    fifo_00_data_out_r <= fifo_00_data_out;
    fifo_01_data_out_r <= fifo_01_data_out;
    fifo_10_data_out_r <= fifo_10_data_out;
    fifo_11_data_out_r <= fifo_11_data_out;
    fifo_20_data_out_r <= fifo_20_data_out;
    fifo_21_data_out_r <= fifo_21_data_out;
    fifo_30_data_out_r <= fifo_30_data_out;
    fifo_31_data_out_r <= fifo_31_data_out;
    fifo_40_data_out_r <= fifo_40_data_out;
    fifo_41_data_out_r <= fifo_41_data_out;
    fifo_50_data_out_r <= fifo_50_data_out;
    fifo_51_data_out_r <= fifo_51_data_out;
    fifo_60_data_out_r <= fifo_60_data_out;
    fifo_61_data_out_r <= fifo_61_data_out;
    fifo_70_data_out_r <= fifo_70_data_out;
    fifo_71_data_out_r <= fifo_71_data_out;
    fifo_80_data_out_r <= fifo_80_data_out;
    fifo_81_data_out_r <= fifo_81_data_out;
end if;                                                    
end if;                                                    
end process;


process(clk90)
begin
if clk90'event and clk90 = '1' then
 if reset90_r = '1' then
    fifo00_rd_addr_r <= "0000";
    fifo01_rd_addr_r <= "0000";
    fifo10_rd_addr_r <= "0000";
    fifo11_rd_addr_r <= "0000";
    fifo20_rd_addr_r <= "0000";
    fifo21_rd_addr_r <= "0000";
    fifo30_rd_addr_r <= "0000";
    fifo31_rd_addr_r <= "0000";
    fifo40_rd_addr_r <= "0000";
    fifo41_rd_addr_r <= "0000";
    fifo50_rd_addr_r <= "0000";
    fifo51_rd_addr_r <= "0000";
    fifo60_rd_addr_r <= "0000";
    fifo61_rd_addr_r <= "0000";
    fifo70_rd_addr_r <= "0000";
    fifo71_rd_addr_r <= "0000";
    fifo80_rd_addr_r <= "0000";
    fifo81_rd_addr_r <= "0000";
    fifop_rd_addr_r  <= "0000";
else
    fifo00_rd_addr_r <= fifo00_rd_addr;
    fifo01_rd_addr_r <= fifo00_rd_addr;
    fifo10_rd_addr_r <= fifo00_rd_addr;
    fifo11_rd_addr_r <= fifo00_rd_addr;
    fifo20_rd_addr_r <= fifo00_rd_addr;
    fifo21_rd_addr_r <= fifo00_rd_addr;
    fifo30_rd_addr_r <= fifo00_rd_addr;
    fifo31_rd_addr_r <= fifo00_rd_addr;
    fifo40_rd_addr_r <= fifo00_rd_addr;
    fifo41_rd_addr_r <= fifo00_rd_addr;
    fifo50_rd_addr_r <= fifo01_rd_addr;
    fifo51_rd_addr_r <= fifo01_rd_addr;
    fifo60_rd_addr_r <= fifo01_rd_addr;
    fifo61_rd_addr_r <= fifo01_rd_addr;
    fifo70_rd_addr_r <= fifo01_rd_addr;
    fifo71_rd_addr_r <= fifo01_rd_addr;
    fifo80_rd_addr_r <= fifo01_rd_addr;
    fifo81_rd_addr_r <= fifo01_rd_addr;
    fifop_rd_addr_r  <= fifo01_rd_addr; 
end if;                                                     
end if;                                                    
end process;


process(clk90)
begin
if clk90'event and clk90 = '1' then
 if reset90_r = '1' then
    first_sdr_data   <= (others => '0');  
    read_valid_data_1_r <= '0';
    read_valid_data_1_r1 <= '0';
    read_valid_data_1_r2 <= '0';
 else
             read_valid_data_1_r <= read_valid_data_1;
             read_valid_data_1_r1 <= read_valid_data_1_r;
             read_valid_data_1_r2 <= read_valid_data_1_r1;
             if (read_valid_data_1_r1 = '1') then    
               		first_sdr_data  <= fifo_00_data_out_r & fifo_01_data_out_r & 
							fifo_10_data_out_r & fifo_11_data_out_r & fifo_20_data_out_r & 
							fifo_21_data_out_r & fifo_30_data_out_r & fifo_31_data_out_r & 
							fifo_40_data_out_r & fifo_41_data_out_r & fifo_50_data_out_r & 
							fifo_51_data_out_r & fifo_60_data_out_r & fifo_61_data_out_r & 
							fifo_70_data_out_r & fifo_71_data_out_r & fifo_80_data_out_r & 
							fifo_81_data_out_r;
             else
               first_sdr_data  <= first_sdr_data;               
             end if;              
end if;                                                    
end if;                                                    
end process;     
   
--------------------------------------------------------------------

-- fifo_x0_rd_addr and fifo_x1_rd_addr counters ( gray counters )  

fifo0_rd_addr_inst :   rd_gray_cntr port map (
							clk 		=>	clk90,
							reset		=>	reset90_r,
							cnt_en	        =>	read_valid_data_1,
                                                        rgc_gcnt	=>	fifo00_rd_addr
                                                       
						); 
fifo1_rd_addr_inst :   rd_gray_cntr port map (
							clk 		=>	clk90,
							reset		=>	reset90_r,
							cnt_en	        =>	read_valid_data_1,
                                                        rgc_gcnt	=>	fifo01_rd_addr
                                                        
						);                                                                         

  
                                                                                                                                                                                                      

--*************************************************************************************************************************
-- Dual Port RAM 16x1 instantiations (fifo0 -- Positive edge, fifo1 -- Trailing edge) 
--*************************************************************************************************************************

--- Byte0 instantiation

fifo0_bit0 : RAM16X1D               
port map (DPO    => fifo_00_data_out(0),          
          A0     => fifo_00_wr_addr(0),          
          A1     => fifo_00_wr_addr(1),
          A2     => fifo_00_wr_addr(2),
          A3     => fifo_00_wr_addr(3),
          D      => ddr_dq_in(0),      
          DPRA0  => fifo00_rd_addr_r(0),
          DPRA1  => fifo00_rd_addr_r(1),
          DPRA2  => fifo00_rd_addr_r(2),
          DPRA3  => fifo00_rd_addr_r(3),
          WCLK   => dqs0_delayed_col1,          
          WE     => fifo_00_wr_en );                       

fifo1_bit0 : RAM16X1D               
port map (DPO    => fifo_01_data_out(0),          
          A0     => fifo_01_wr_addr_1(0),          
          A1     => fifo_01_wr_addr_1(1),
          A2     => fifo_01_wr_addr_1(2),
          A3     => fifo_01_wr_addr_1(3),
          D      => ddr_dq_in(0),      
          DPRA0  => fifo00_rd_addr_r(0),
          DPRA1  => fifo00_rd_addr_r(1),
          DPRA2  => fifo00_rd_addr_r(2),
          DPRA3  => fifo00_rd_addr_r(3),
          WCLK   =>   dqs0_delayed_col1_n,          
          WE     => fifo_01_wr_en_1 ); 
                      
fifo0_bit1 : RAM16X1D               
port map (DPO    => fifo_00_data_out(1),          
          A0     => fifo_00_wr_addr_1(0),          
          A1     => fifo_00_wr_addr_1(1),
          A2     => fifo_00_wr_addr_1(2),
          A3     => fifo_00_wr_addr_1(3),
          D      => ddr_dq_in(1),      
          DPRA0  => fifo01_rd_addr_r(0),
          DPRA1  => fifo01_rd_addr_r(1),
          DPRA2  => fifo01_rd_addr_r(2),
          DPRA3  => fifo01_rd_addr_r(3),
          WCLK   => dqs0_delayed_col0,          
          WE     => fifo_00_wr_en_1 );                       

fifo1_bit1 : RAM16X1D               
port map (DPO    => fifo_01_data_out(1),          
          A0     => fifo_01_wr_addr(0),          
          A1     => fifo_01_wr_addr(1),
          A2     => fifo_01_wr_addr(2),
          A3     => fifo_01_wr_addr(3),
          D      => ddr_dq_in(1),      
          DPRA0  => fifo01_rd_addr_r(0),
          DPRA1  => fifo01_rd_addr_r(1),
          DPRA2  => fifo01_rd_addr_r(2),
          DPRA3  => fifo01_rd_addr_r(3),
          WCLK   => dqs0_delayed_col0_n,          
          WE     => fifo_01_wr_en );                       



fifo0_bit2 : RAM16X1D               
port map (DPO    => fifo_00_data_out(2),          
          A0     => fifo_00_wr_addr(0),          
          A1     => fifo_00_wr_addr(1),
          A2     => fifo_00_wr_addr(2),
          A3     => fifo_00_wr_addr(3),
          D      => ddr_dq_in(2),      
          DPRA0  => fifo00_rd_addr_r(0),
          DPRA1  => fifo00_rd_addr_r(1),
          DPRA2  => fifo00_rd_addr_r(2),
          DPRA3  => fifo00_rd_addr_r(3),
          WCLK   => dqs0_delayed_col1,          
          WE     => fifo_00_wr_en );                       

fifo1_bit2 : RAM16X1D               
port map (DPO    => fifo_01_data_out(2),          
          A0     => fifo_01_wr_addr_1(0),          
          A1     => fifo_01_wr_addr_1(1),
          A2     => fifo_01_wr_addr_1(2),
          A3     => fifo_01_wr_addr_1(3),
          D      => ddr_dq_in(2),      
          DPRA0  => fifo00_rd_addr_r(0),
          DPRA1  => fifo00_rd_addr_r(1),
          DPRA2  => fifo00_rd_addr_r(2),
          DPRA3  => fifo00_rd_addr_r(3),
          WCLK   => dqs0_delayed_col1_n,          
          WE     => fifo_01_wr_en_1 ); 
                      
fifo0_bit3 : RAM16X1D               
port map (DPO    => fifo_00_data_out(3),          
          A0     => fifo_00_wr_addr_1(0),          
          A1     => fifo_00_wr_addr_1(1),
          A2     => fifo_00_wr_addr_1(2),
          A3     => fifo_00_wr_addr_1(3),
          D      => ddr_dq_in(3),      
          DPRA0  => fifo01_rd_addr_r(0),
          DPRA1  => fifo01_rd_addr_r(1),
          DPRA2  => fifo01_rd_addr_r(2),
          DPRA3  => fifo01_rd_addr_r(3),
          WCLK   => dqs0_delayed_col0,          
          WE     => fifo_00_wr_en_1 );                       

fifo1_bit3 : RAM16X1D               
port map (DPO    => fifo_01_data_out(3),          
          A0     => fifo_01_wr_addr(0),          
          A1     => fifo_01_wr_addr(1),
          A2     => fifo_01_wr_addr(2),
          A3     => fifo_01_wr_addr(3),
          D      => ddr_dq_in(3),      
          DPRA0  => fifo01_rd_addr_r(0),
          DPRA1  => fifo01_rd_addr_r(1),
          DPRA2  => fifo01_rd_addr_r(2),
          DPRA3  => fifo01_rd_addr_r(3),
          WCLK   => dqs0_delayed_col0_n,          
          WE     => fifo_01_wr_en ); 


                      
fifo0_bit4 : RAM16X1D               
port map (DPO    => fifo_00_data_out(4),          
          A0     => fifo_00_wr_addr(0),          
          A1     => fifo_00_wr_addr(1),
          A2     => fifo_00_wr_addr(2),
          A3     => fifo_00_wr_addr(3),
          D      => ddr_dq_in(4),      
          DPRA0  => fifo00_rd_addr_r(0),
          DPRA1  => fifo00_rd_addr_r(1),
          DPRA2  => fifo00_rd_addr_r(2),
          DPRA3  => fifo00_rd_addr_r(3),
          WCLK   => dqs0_delayed_col1,          
          WE     => fifo_00_wr_en );                       

fifo1_bit4 : RAM16X1D               
port map (DPO    => fifo_01_data_out(4),          
          A0     => fifo_01_wr_addr_1(0),          
          A1     => fifo_01_wr_addr_1(1),
          A2     => fifo_01_wr_addr_1(2),
          A3     => fifo_01_wr_addr_1(3),
          D      => ddr_dq_in(4),      
          DPRA0  => fifo00_rd_addr_r(0),
          DPRA1  => fifo00_rd_addr_r(1),
          DPRA2  => fifo00_rd_addr_r(2),
          DPRA3  => fifo00_rd_addr_r(3),
          WCLK   => dqs0_delayed_col1_n,          
          WE     => fifo_01_wr_en_1 ); 
                      
fifo0_bit5 : RAM16X1D               
port map (DPO    => fifo_00_data_out(5),          
          A0     => fifo_00_wr_addr_1(0),          
          A1     => fifo_00_wr_addr_1(1),
          A2     => fifo_00_wr_addr_1(2),
          A3     => fifo_00_wr_addr_1(3),
          D      => ddr_dq_in(5),      
          DPRA0  => fifo01_rd_addr_r(0),
          DPRA1  => fifo01_rd_addr_r(1),
          DPRA2  => fifo01_rd_addr_r(2),
          DPRA3  => fifo01_rd_addr_r(3),
          WCLK   => dqs0_delayed_col0,          
          WE     => fifo_00_wr_en_1 );                       

fifo1_bit5 : RAM16X1D               
port map (DPO    => fifo_01_data_out(5),          
          A0     => fifo_01_wr_addr(0),          
          A1     => fifo_01_wr_addr(1),
          A2     => fifo_01_wr_addr(2),
          A3     => fifo_01_wr_addr(3),
          D      => ddr_dq_in(5),      
          DPRA0  => fifo01_rd_addr_r(0),
          DPRA1  => fifo01_rd_addr_r(1),
          DPRA2  => fifo01_rd_addr_r(2),
          DPRA3  => fifo01_rd_addr_r(3),
          WCLK   => dqs0_delayed_col0_n,          
          WE     => fifo_01_wr_en );                       
								  

fifo0_bit6 : RAM16X1D               
port map (DPO    => fifo_00_data_out(6),          
          A0     => fifo_00_wr_addr(0),          
          A1     => fifo_00_wr_addr(1),
          A2     => fifo_00_wr_addr(2),
          A3     => fifo_00_wr_addr(3),
          D      => ddr_dq_in(6),      
          DPRA0  => fifo00_rd_addr_r(0),
          DPRA1  => fifo00_rd_addr_r(1),
          DPRA2  => fifo00_rd_addr_r(2),
          DPRA3  => fifo00_rd_addr_r(3),
          WCLK   => dqs0_delayed_col1,          
          WE     => fifo_00_wr_en );                       

fifo1_bit6 : RAM16X1D               
port map (DPO    => fifo_01_data_out(6),          
          A0     => fifo_01_wr_addr_1(0),          
          A1     => fifo_01_wr_addr_1(1),
          A2     => fifo_01_wr_addr_1(2),
          A3     => fifo_01_wr_addr_1(3),
          D      => ddr_dq_in(6),      
          DPRA0  => fifo00_rd_addr_r(0),
          DPRA1  => fifo00_rd_addr_r(1),
          DPRA2  => fifo00_rd_addr_r(2),
          DPRA3  => fifo00_rd_addr_r(3),
          WCLK   => dqs0_delayed_col1_n,          
          WE     => fifo_01_wr_en_1 ); 
                      
fifo0_bit7 : RAM16X1D               
port map (DPO    => fifo_00_data_out(7),          
          A0     => fifo_00_wr_addr_1(0),          
          A1     => fifo_00_wr_addr_1(1),
          A2     => fifo_00_wr_addr_1(2),
          A3     => fifo_00_wr_addr_1(3),
          D      => ddr_dq_in(7),      
          DPRA0  => fifo01_rd_addr_r(0),
          DPRA1  => fifo01_rd_addr_r(1),
          DPRA2  => fifo01_rd_addr_r(2),
          DPRA3  => fifo01_rd_addr_r(3),
          WCLK   => dqs0_delayed_col0,          
          WE     => fifo_00_wr_en_1 );                       

fifo1_bit7 : RAM16X1D               
port map (DPO    => fifo_01_data_out(7),          
          A0     => fifo_01_wr_addr(0),          
          A1     => fifo_01_wr_addr(1),
          A2     => fifo_01_wr_addr(2),
          A3     => fifo_01_wr_addr(3),
          D      => ddr_dq_in(7),      
          DPRA0  => fifo01_rd_addr_r(0),
          DPRA1  => fifo01_rd_addr_r(1),
          DPRA2  => fifo01_rd_addr_r(2),
          DPRA3  => fifo01_rd_addr_r(3),
          WCLK   => dqs0_delayed_col0_n,          
          WE     => fifo_01_wr_en ); 
                                     

-- Byte1 Fifo instantiation 

fifo0_bit8 : RAM16X1D               
port map (DPO    => fifo_10_data_out(0),          
          A0     => fifo_10_wr_addr(0),          
          A1     => fifo_10_wr_addr(1),
          A2     => fifo_10_wr_addr(2),
          A3     => fifo_10_wr_addr(3),
          D      => ddr_dq_in(8),      
          DPRA0  => fifo10_rd_addr_r(0),
          DPRA1  => fifo10_rd_addr_r(1),
          DPRA2  => fifo10_rd_addr_r(2),
          DPRA3  => fifo10_rd_addr_r(3),
          WCLK   => dqs1_delayed_col1,          
          WE     => fifo_10_wr_en );                       

fifo1_bit8 : RAM16X1D               
port map (DPO    => fifo_11_data_out(0),          
          A0     => fifo_11_wr_addr_1(0),          
          A1     => fifo_11_wr_addr_1(1),
          A2     => fifo_11_wr_addr_1(2),
          A3     => fifo_11_wr_addr_1(3),
          D      => ddr_dq_in(8),      
          DPRA0  => fifo10_rd_addr_r(0),
          DPRA1  => fifo10_rd_addr_r(1),
          DPRA2  => fifo10_rd_addr_r(2),
          DPRA3  => fifo10_rd_addr_r(3),
          WCLK   =>  dqs1_delayed_col1_n,          
          WE     => fifo_11_wr_en_1 ); 
                      
fifo0_bit9 : RAM16X1D               
port map (DPO    => fifo_10_data_out(1),          
          A0     => fifo_10_wr_addr_1(0),          
          A1     => fifo_10_wr_addr_1(1),
          A2     => fifo_10_wr_addr_1(2),
          A3     => fifo_10_wr_addr_1(3),
          D      => ddr_dq_in(9),      
          DPRA0  => fifo11_rd_addr_r(0),
          DPRA1  => fifo11_rd_addr_r(1),
          DPRA2  => fifo11_rd_addr_r(2),
          DPRA3  => fifo11_rd_addr_r(3),
          WCLK   => dqs1_delayed_col0,          
          WE     => fifo_10_wr_en_1 );                      

fifo1_bit9 : RAM16X1D               
port map (DPO    => fifo_11_data_out(1),          
          A0     => fifo_11_wr_addr(0),          
          A1     => fifo_11_wr_addr(1),
          A2     => fifo_11_wr_addr(2),
          A3     => fifo_11_wr_addr(3),
          D      => ddr_dq_in(9),      
          DPRA0  => fifo11_rd_addr_r(0),
          DPRA1  => fifo11_rd_addr_r(1),
          DPRA2  => fifo11_rd_addr_r(2),
          DPRA3  => fifo11_rd_addr_r(3),
          WCLK   =>  dqs1_delayed_col0_n,          
          WE     => fifo_11_wr_en );                       


fifo0_bit10 : RAM16X1D               
port map (DPO    => fifo_10_data_out(2),          
          A0     => fifo_10_wr_addr(0),          
          A1     => fifo_10_wr_addr(1),
          A2     => fifo_10_wr_addr(2),
          A3     => fifo_10_wr_addr(3),
          D      => ddr_dq_in(10),      
          DPRA0  => fifo10_rd_addr_r(0),
          DPRA1  => fifo10_rd_addr_r(1),
          DPRA2  => fifo10_rd_addr_r(2),
          DPRA3  => fifo10_rd_addr_r(3),
          WCLK   => dqs1_delayed_col1,          
          WE     => fifo_10_wr_en );                       

fifo1_bit10 : RAM16X1D               
port map (DPO    => fifo_11_data_out(2),          
          A0     => fifo_11_wr_addr_1(0),          
          A1     => fifo_11_wr_addr_1(1),
          A2     => fifo_11_wr_addr_1(2),
          A3     => fifo_11_wr_addr_1(3),
          D      => ddr_dq_in(10),      
          DPRA0  => fifo10_rd_addr_r(0),
          DPRA1  => fifo10_rd_addr_r(1),
          DPRA2  => fifo10_rd_addr_r(2),
          DPRA3  => fifo10_rd_addr_r(3),
          WCLK   =>  dqs1_delayed_col1_n,          
          WE     => fifo_11_wr_en_1 ); 
                      
fifo0_bit11 : RAM16X1D               
port map (DPO    => fifo_10_data_out(3),          
          A0     => fifo_10_wr_addr_1(0),          
          A1     => fifo_10_wr_addr_1(1),
          A2     => fifo_10_wr_addr_1(2),
          A3     => fifo_10_wr_addr_1(3),
          D      => ddr_dq_in(11),      
          DPRA0  => fifo11_rd_addr_r(0),
          DPRA1  => fifo11_rd_addr_r(1),
          DPRA2  => fifo11_rd_addr_r(2),
          DPRA3  => fifo11_rd_addr_r(3),
          WCLK   => dqs1_delayed_col0,          
          WE     => fifo_10_wr_en_1 );                       

fifo1_bit11 : RAM16X1D               
port map (DPO    => fifo_11_data_out(3),          
          A0     => fifo_11_wr_addr(0),          
          A1     => fifo_11_wr_addr(1),
          A2     => fifo_11_wr_addr(2),
          A3     => fifo_11_wr_addr(3),
          D      => ddr_dq_in(11),      
          DPRA0  => fifo11_rd_addr_r(0),
          DPRA1  => fifo11_rd_addr_r(1),
          DPRA2  => fifo11_rd_addr_r(2),
          DPRA3  => fifo11_rd_addr_r(3),
          WCLK   =>  dqs1_delayed_col0_n,          
          WE     => fifo_11_wr_en ); 


                      
fifo0_bit12 : RAM16X1D               
port map (DPO    => fifo_10_data_out(4),          
          A0     => fifo_10_wr_addr(0),          
          A1     => fifo_10_wr_addr(1),
          A2     => fifo_10_wr_addr(2),
          A3     => fifo_10_wr_addr(3),
          D      => ddr_dq_in(12),      
          DPRA0  => fifo10_rd_addr_r(0),
          DPRA1  => fifo10_rd_addr_r(1),
          DPRA2  => fifo10_rd_addr_r(2),
          DPRA3  => fifo10_rd_addr_r(3),
          WCLK   => dqs1_delayed_col1,          
          WE     => fifo_10_wr_en );                       

fifo1_bit12 : RAM16X1D               
port map (DPO    => fifo_11_data_out(4),          
          A0     => fifo_11_wr_addr_1(0),          
          A1     => fifo_11_wr_addr_1(1),
          A2     => fifo_11_wr_addr_1(2),
          A3     => fifo_11_wr_addr_1(3),
          D      => ddr_dq_in(12),      
          DPRA0  => fifo10_rd_addr_r(0),
          DPRA1  => fifo10_rd_addr_r(1),
          DPRA2  => fifo10_rd_addr_r(2),
          DPRA3  => fifo10_rd_addr_r(3),
          WCLK   =>  dqs1_delayed_col1_n,          
          WE     => fifo_11_wr_en_1 ); 
                      
fifo0_bit13 : RAM16X1D               
port map (DPO    => fifo_10_data_out(5),          
          A0     => fifo_10_wr_addr_1(0),          
          A1     => fifo_10_wr_addr_1(1),
          A2     => fifo_10_wr_addr_1(2),
          A3     => fifo_10_wr_addr_1(3),
          D      => ddr_dq_in(13),      
          DPRA0  => fifo11_rd_addr_r(0),
          DPRA1  => fifo11_rd_addr_r(1),
          DPRA2  => fifo11_rd_addr_r(2),
          DPRA3  => fifo11_rd_addr_r(3),
          WCLK   => dqs1_delayed_col0,          
          WE     => fifo_10_wr_en_1 );                       

fifo1_bit13 : RAM16X1D               
port map (DPO    => fifo_11_data_out(5),          
          A0     => fifo_11_wr_addr(0),          
          A1     => fifo_11_wr_addr(1),
          A2     => fifo_11_wr_addr(2),
          A3     => fifo_11_wr_addr(3),
          D      => ddr_dq_in(13),      
          DPRA0  => fifo11_rd_addr_r(0),
          DPRA1  => fifo11_rd_addr_r(1),
          DPRA2  => fifo11_rd_addr_r(2),
          DPRA3  => fifo11_rd_addr_r(3),
          WCLK   =>  dqs1_delayed_col0_n,          
          WE     => fifo_11_wr_en );                       


fifo0_bit14 : RAM16X1D               
port map (DPO    => fifo_10_data_out(6),          
          A0     => fifo_10_wr_addr(0),          
          A1     => fifo_10_wr_addr(1),
          A2     => fifo_10_wr_addr(2),
          A3     => fifo_10_wr_addr(3),
          D      => ddr_dq_in(14),      
          DPRA0  => fifo10_rd_addr_r(0),
          DPRA1  => fifo10_rd_addr_r(1),
          DPRA2  => fifo10_rd_addr_r(2),
          DPRA3  => fifo10_rd_addr_r(3),
          WCLK   => dqs1_delayed_col1,          
          WE     => fifo_10_wr_en );                       

fifo1_bit14 : RAM16X1D               
port map (DPO    => fifo_11_data_out(6),          
          A0     => fifo_11_wr_addr_1(0),          
          A1     => fifo_11_wr_addr_1(1),
          A2     => fifo_11_wr_addr_1(2),
          A3     => fifo_11_wr_addr_1(3),
          D      => ddr_dq_in(14),      
          DPRA0  => fifo10_rd_addr_r(0),
          DPRA1  => fifo10_rd_addr_r(1),
          DPRA2  => fifo10_rd_addr_r(2),
          DPRA3  => fifo10_rd_addr_r(3),
          WCLK   =>  dqs1_delayed_col1_n,          
          WE     => fifo_11_wr_en_1 ); 
                      
fifo0_bit15 : RAM16X1D               
port map (DPO    => fifo_10_data_out(7),          
          A0     => fifo_10_wr_addr_1(0),          
          A1     => fifo_10_wr_addr_1(1),
          A2     => fifo_10_wr_addr_1(2),
          A3     => fifo_10_wr_addr_1(3),
          D      => ddr_dq_in(15),      
          DPRA0  => fifo11_rd_addr_r(0),
          DPRA1  => fifo11_rd_addr_r(1),
          DPRA2  => fifo11_rd_addr_r(2),
          DPRA3  => fifo11_rd_addr_r(3),
          WCLK   => dqs1_delayed_col0,          
          WE     => fifo_10_wr_en_1 );                      

fifo1_bit15 : RAM16X1D               
port map (DPO    => fifo_11_data_out(7),          
          A0     => fifo_11_wr_addr(0),          
          A1     => fifo_11_wr_addr(1),
          A2     => fifo_11_wr_addr(2),
          A3     => fifo_11_wr_addr(3),
          D      => ddr_dq_in(15),      
          DPRA0  => fifo11_rd_addr_r(0),
          DPRA1  => fifo11_rd_addr_r(1),
          DPRA2  => fifo11_rd_addr_r(2),
          DPRA3  => fifo11_rd_addr_r(3),
          WCLK   =>  dqs1_delayed_col0_n,          
          WE     => fifo_11_wr_en ); 
                                    

-- Byte2 Fifo instantiation

fifo0_bit16 : RAM16X1D               
port map (DPO    => fifo_20_data_out(0),          
          A0     => fifo_20_wr_addr(0),          
          A1     => fifo_20_wr_addr(1),
          A2     => fifo_20_wr_addr(2),
          A3     => fifo_20_wr_addr(3),
          D      => ddr_dq_in(16),      
          DPRA0  => fifo20_rd_addr_r(0),
          DPRA1  => fifo20_rd_addr_r(1),
          DPRA2  => fifo20_rd_addr_r(2),
          DPRA3  => fifo20_rd_addr_r(3),
          WCLK   => dqs2_delayed_col1,          
          WE     => fifo_20_wr_en );                       

fifo1_bit16 : RAM16X1D               
port map (DPO    => fifo_21_data_out(0),          
          A0     => fifo_21_wr_addr_1(0),          
          A1     => fifo_21_wr_addr_1(1),
          A2     => fifo_21_wr_addr_1(2),
          A3     => fifo_21_wr_addr_1(3),
          D      => ddr_dq_in(16),      
          DPRA0  => fifo20_rd_addr_r(0),
          DPRA1  => fifo20_rd_addr_r(1),
          DPRA2  => fifo20_rd_addr_r(2),
          DPRA3  => fifo20_rd_addr_r(3),
          WCLK   =>  dqs2_delayed_col1_n,          
          WE     => fifo_21_wr_en_1 ); 
                      
fifo0_bit17 : RAM16X1D               
port map (DPO    => fifo_20_data_out(1),          
          A0     => fifo_20_wr_addr_1(0),          
          A1     => fifo_20_wr_addr_1(1),
          A2     => fifo_20_wr_addr_1(2),
          A3     => fifo_20_wr_addr_1(3),
          D      => ddr_dq_in(17),      
          DPRA0  => fifo21_rd_addr_r(0),
          DPRA1  => fifo21_rd_addr_r(1),
          DPRA2  => fifo21_rd_addr_r(2),
          DPRA3  => fifo21_rd_addr_r(3),
          WCLK   => dqs2_delayed_col0,          
          WE     => fifo_20_wr_en_1 );                       

fifo1_bit17 : RAM16X1D               
port map (DPO    => fifo_21_data_out(1),          
          A0     => fifo_21_wr_addr(0),          
          A1     => fifo_21_wr_addr(1),
          A2     => fifo_21_wr_addr(2),
          A3     => fifo_21_wr_addr(3),
          D      => ddr_dq_in(17),      
          DPRA0  => fifo21_rd_addr_r(0),
          DPRA1  => fifo21_rd_addr_r(1),
          DPRA2  => fifo21_rd_addr_r(2),
          DPRA3  => fifo21_rd_addr_r(3),
          WCLK   =>  dqs2_delayed_col0_n,          
          WE     => fifo_21_wr_en );                       



fifo0_bit18 : RAM16X1D               
port map (DPO    => fifo_20_data_out(2),          
          A0     => fifo_20_wr_addr(0),          
          A1     => fifo_20_wr_addr(1),
          A2     => fifo_20_wr_addr(2),
          A3     => fifo_20_wr_addr(3),
          D      => ddr_dq_in(18),      
          DPRA0  => fifo20_rd_addr_r(0),
          DPRA1  => fifo20_rd_addr_r(1),
          DPRA2  => fifo20_rd_addr_r(2),
          DPRA3  => fifo20_rd_addr_r(3),
          WCLK   => dqs2_delayed_col1,          
          WE     => fifo_20_wr_en );                       

fifo1_bit18 : RAM16X1D               
port map (DPO    => fifo_21_data_out(2),          
          A0     => fifo_21_wr_addr_1(0),          
          A1     => fifo_21_wr_addr_1(1),
          A2     => fifo_21_wr_addr_1(2),
          A3     => fifo_21_wr_addr_1(3),
          D      => ddr_dq_in(18),      
          DPRA0  => fifo20_rd_addr_r(0),
          DPRA1  => fifo20_rd_addr_r(1),
          DPRA2  => fifo20_rd_addr_r(2),
          DPRA3  => fifo20_rd_addr_r(3),
          WCLK   =>  dqs2_delayed_col1_n,          
          WE     => fifo_21_wr_en_1 ); 
                      
fifo0_bit19 : RAM16X1D               
port map (DPO    => fifo_20_data_out(3),          
          A0     => fifo_20_wr_addr_1(0),          
          A1     => fifo_20_wr_addr_1(1),
          A2     => fifo_20_wr_addr_1(2),
          A3     => fifo_20_wr_addr_1(3),
          D      => ddr_dq_in(19),      
          DPRA0  => fifo21_rd_addr_r(0),
          DPRA1  => fifo21_rd_addr_r(1),
          DPRA2  => fifo21_rd_addr_r(2),
          DPRA3  => fifo21_rd_addr_r(3),
          WCLK   => dqs2_delayed_col0,          
          WE     => fifo_20_wr_en_1 );                       

fifo1_bit19 : RAM16X1D               
port map (DPO    => fifo_21_data_out(3),          
          A0     => fifo_21_wr_addr(0),          
          A1     => fifo_21_wr_addr(1),
          A2     => fifo_21_wr_addr(2),
          A3     => fifo_21_wr_addr(3),
          D      => ddr_dq_in(19),      
          DPRA0  => fifo21_rd_addr_r(0),
          DPRA1  => fifo21_rd_addr_r(1),
          DPRA2  => fifo21_rd_addr_r(2),
          DPRA3  => fifo21_rd_addr_r(3),
          WCLK   =>  dqs2_delayed_col0_n,          
          WE     => fifo_21_wr_en ); 


fifo0_bit20 : RAM16X1D               
port map (DPO    => fifo_20_data_out(4),          
          A0     => fifo_20_wr_addr(0),          
          A1     => fifo_20_wr_addr(1),
          A2     => fifo_20_wr_addr(2),
          A3     => fifo_20_wr_addr(3),
          D      => ddr_dq_in(20),      
          DPRA0  => fifo20_rd_addr_r(0),
          DPRA1  => fifo20_rd_addr_r(1),
          DPRA2  => fifo20_rd_addr_r(2),
          DPRA3  => fifo20_rd_addr_r(3),
          WCLK   => dqs2_delayed_col1,          
          WE     => fifo_20_wr_en );                       

fifo1_bit20 : RAM16X1D               
port map (DPO    => fifo_21_data_out(4),          
          A0     => fifo_21_wr_addr_1(0),          
          A1     => fifo_21_wr_addr_1(1),
          A2     => fifo_21_wr_addr_1(2),
          A3     => fifo_21_wr_addr_1(3),
          D      => ddr_dq_in(20),      
          DPRA0  => fifo20_rd_addr_r(0),
          DPRA1  => fifo20_rd_addr_r(1),
          DPRA2  => fifo20_rd_addr_r(2),
          DPRA3  => fifo20_rd_addr_r(3),
          WCLK   =>  dqs2_delayed_col1_n,          
          WE     => fifo_21_wr_en_1 ); 
                      
fifo0_bit21 : RAM16X1D               
port map (DPO    => fifo_20_data_out(5),          
          A0     => fifo_20_wr_addr_1(0),          
          A1     => fifo_20_wr_addr_1(1),
          A2     => fifo_20_wr_addr_1(2),
          A3     => fifo_20_wr_addr_1(3),
          D      => ddr_dq_in(21),      
          DPRA0  => fifo21_rd_addr_r(0),
          DPRA1  => fifo21_rd_addr_r(1),
          DPRA2  => fifo21_rd_addr_r(2),
          DPRA3  => fifo21_rd_addr_r(3),
          WCLK   => dqs2_delayed_col0,          
          WE     => fifo_20_wr_en_1 );                       

fifo1_bit21 : RAM16X1D               
port map (DPO    => fifo_21_data_out(5),          
          A0     => fifo_21_wr_addr(0),          
          A1     => fifo_21_wr_addr(1),
          A2     => fifo_21_wr_addr(2),
          A3     => fifo_21_wr_addr(3),
          D      => ddr_dq_in(21),      
          DPRA0  => fifo21_rd_addr_r(0),
          DPRA1  => fifo21_rd_addr_r(1),
          DPRA2  => fifo21_rd_addr_r(2),
          DPRA3  => fifo21_rd_addr_r(3),
          WCLK   =>  dqs2_delayed_col0_n,          
          WE     => fifo_21_wr_en );                       



fifo0_bit22 : RAM16X1D               
port map (DPO    => fifo_20_data_out(6),          
          A0     => fifo_20_wr_addr(0),          
          A1     => fifo_20_wr_addr(1),
          A2     => fifo_20_wr_addr(2),
          A3     => fifo_20_wr_addr(3),
          D      => ddr_dq_in(22),      
          DPRA0  => fifo20_rd_addr_r(0),
          DPRA1  => fifo20_rd_addr_r(1),
          DPRA2  => fifo20_rd_addr_r(2),
          DPRA3  => fifo20_rd_addr_r(3),
          WCLK   => dqs2_delayed_col1,          
          WE     => fifo_20_wr_en );                       

fifo1_bit22 : RAM16X1D               
port map (DPO    => fifo_21_data_out(6),          
          A0     => fifo_21_wr_addr_1(0),          
          A1     => fifo_21_wr_addr_1(1),
          A2     => fifo_21_wr_addr_1(2),
          A3     => fifo_21_wr_addr_1(3),
          D      => ddr_dq_in(22),      
          DPRA0  => fifo20_rd_addr_r(0),
          DPRA1  => fifo20_rd_addr_r(1),
          DPRA2  => fifo20_rd_addr_r(2),
          DPRA3  => fifo20_rd_addr_r(3),
          WCLK   =>  dqs2_delayed_col1_n,          
          WE     => fifo_21_wr_en_1 ); 
                      
fifo0_bit23 : RAM16X1D               
port map (DPO    => fifo_20_data_out(7),          
          A0     => fifo_20_wr_addr_1(0),          
          A1     => fifo_20_wr_addr_1(1),
          A2     => fifo_20_wr_addr_1(2),
          A3     => fifo_20_wr_addr_1(3),
          D      => ddr_dq_in(23),      
          DPRA0  => fifo21_rd_addr_r(0),
          DPRA1  => fifo21_rd_addr_r(1),
          DPRA2  => fifo21_rd_addr_r(2),
          DPRA3  => fifo21_rd_addr_r(3),
          WCLK   => dqs2_delayed_col0,          
          WE     => fifo_20_wr_en_1 );                       

fifo1_bit23 : RAM16X1D               
port map (DPO    => fifo_21_data_out(7),          
          A0     => fifo_21_wr_addr(0),          
          A1     => fifo_21_wr_addr(1),
          A2     => fifo_21_wr_addr(2),
          A3     => fifo_21_wr_addr(3),
          D      => ddr_dq_in(23),      
          DPRA0  => fifo21_rd_addr_r(0),
          DPRA1  => fifo21_rd_addr_r(1),
          DPRA2  => fifo21_rd_addr_r(2),
          DPRA3  => fifo21_rd_addr_r(3),
          WCLK   =>  dqs2_delayed_col0_n,          
          WE     => fifo_21_wr_en ); 


--Byte3 Fifo Instantiation 

fifo0_bit24 : RAM16X1D               
port map (DPO    => fifo_30_data_out(0),          
          A0     => fifo_30_wr_addr(0),          
          A1     => fifo_30_wr_addr(1),
          A2     => fifo_30_wr_addr(2),
          A3     => fifo_30_wr_addr(3),
          D      => ddr_dq_in(24),      
          DPRA0  => fifo30_rd_addr_r(0),
          DPRA1  => fifo30_rd_addr_r(1),
          DPRA2  => fifo30_rd_addr_r(2),
          DPRA3  => fifo30_rd_addr_r(3),
          WCLK   => dqs3_delayed_col1,          
          WE     => fifo_30_wr_en );                       

fifo1_bit24 : RAM16X1D               
port map (DPO    => fifo_31_data_out(0),          
          A0     => fifo_31_wr_addr_1(0),          
          A1     => fifo_31_wr_addr_1(1),
          A2     => fifo_31_wr_addr_1(2),
          A3     => fifo_31_wr_addr_1(3),
          D      => ddr_dq_in(24),      
          DPRA0  => fifo30_rd_addr_r(0),
          DPRA1  => fifo30_rd_addr_r(1),
          DPRA2  => fifo30_rd_addr_r(2),
          DPRA3  => fifo30_rd_addr_r(3),
          WCLK   =>  dqs3_delayed_col1_n,          
          WE     => fifo_31_wr_en_1 ); 
                      
fifo0_bit25 : RAM16X1D               
port map (DPO    => fifo_30_data_out(1),          
          A0     => fifo_30_wr_addr_1(0),          
          A1     => fifo_30_wr_addr_1(1),
          A2     => fifo_30_wr_addr_1(2),
          A3     => fifo_30_wr_addr_1(3),
          D      => ddr_dq_in(25),      
          DPRA0  => fifo31_rd_addr_r(0),
          DPRA1  => fifo31_rd_addr_r(1),
          DPRA2  => fifo31_rd_addr_r(2),
          DPRA3  => fifo31_rd_addr_r(3),
          WCLK   => dqs3_delayed_col0,          
          WE     => fifo_30_wr_en_1 );                       

fifo1_bit25 : RAM16X1D               
port map (DPO    => fifo_31_data_out(1),          
          A0     => fifo_31_wr_addr(0),          
          A1     => fifo_31_wr_addr(1),
          A2     => fifo_31_wr_addr(2),
          A3     => fifo_31_wr_addr(3),
          D      => ddr_dq_in(25),      
          DPRA0  => fifo31_rd_addr_r(0),
          DPRA1  => fifo31_rd_addr_r(1),
          DPRA2  => fifo31_rd_addr_r(2),
          DPRA3  => fifo31_rd_addr_r(3),
          WCLK   =>  dqs3_delayed_col0_n,          
          WE     => fifo_31_wr_en );                       



fifo0_bit26 : RAM16X1D               
port map (DPO    => fifo_30_data_out(2),          
          A0     => fifo_30_wr_addr(0),          
          A1     => fifo_30_wr_addr(1),
          A2     => fifo_30_wr_addr(2),
          A3     => fifo_30_wr_addr(3),
          D      => ddr_dq_in(26),      
          DPRA0  => fifo30_rd_addr_r(0),
          DPRA1  => fifo30_rd_addr_r(1),
          DPRA2  => fifo30_rd_addr_r(2),
          DPRA3  => fifo30_rd_addr_r(3),
          WCLK   => dqs3_delayed_col1,          
          WE     => fifo_30_wr_en );                       

fifo1_bit26 : RAM16X1D               
port map (DPO    => fifo_31_data_out(2),          
          A0     => fifo_31_wr_addr_1(0),          
          A1     => fifo_31_wr_addr_1(1),
          A2     => fifo_31_wr_addr_1(2),
          A3     => fifo_31_wr_addr_1(3),
          D      => ddr_dq_in(26),      
          DPRA0  => fifo30_rd_addr_r(0),
          DPRA1  => fifo30_rd_addr_r(1),
          DPRA2  => fifo30_rd_addr_r(2),
          DPRA3  => fifo30_rd_addr_r(3),
          WCLK   =>  dqs3_delayed_col1_n,          
          WE     => fifo_31_wr_en_1 ); 
                      
fifo0_bit27 : RAM16X1D               
port map (DPO    => fifo_30_data_out(3),          
          A0     => fifo_30_wr_addr_1(0),          
          A1     => fifo_30_wr_addr_1(1),
          A2     => fifo_30_wr_addr_1(2),
          A3     => fifo_30_wr_addr_1(3),
          D      => ddr_dq_in(27),      
          DPRA0  => fifo31_rd_addr_r(0),
          DPRA1  => fifo31_rd_addr_r(1),
          DPRA2  => fifo31_rd_addr_r(2),
          DPRA3  => fifo31_rd_addr_r(3),
          WCLK   => dqs3_delayed_col0,          
          WE     => fifo_30_wr_en_1 );                       

fifo1_bit27 : RAM16X1D               
port map (DPO    => fifo_31_data_out(3),          
          A0     => fifo_31_wr_addr(0),          
          A1     => fifo_31_wr_addr(1),
          A2     => fifo_31_wr_addr(2),
          A3     => fifo_31_wr_addr(3),
          D      => ddr_dq_in(27),      
          DPRA0  => fifo31_rd_addr_r(0),
          DPRA1  => fifo31_rd_addr_r(1),
          DPRA2  => fifo31_rd_addr_r(2),
          DPRA3  => fifo31_rd_addr_r(3),
          WCLK   =>  dqs3_delayed_col0_n,          
          WE     => fifo_31_wr_en ); 


                      
fifo0_bit28 : RAM16X1D               
port map (DPO    => fifo_30_data_out(4),          
          A0     => fifo_30_wr_addr(0),          
          A1     => fifo_30_wr_addr(1),
          A2     => fifo_30_wr_addr(2),
          A3     => fifo_30_wr_addr(3),
          D      => ddr_dq_in(28),      
          DPRA0  => fifo30_rd_addr_r(0),
          DPRA1  => fifo30_rd_addr_r(1),
          DPRA2  => fifo30_rd_addr_r(2),
          DPRA3  => fifo30_rd_addr_r(3),
          WCLK   => dqs3_delayed_col1,          
          WE     => fifo_30_wr_en );                       

fifo1_bit28 : RAM16X1D               
port map (DPO    => fifo_31_data_out(4),          
          A0     => fifo_31_wr_addr_1(0),          
          A1     => fifo_31_wr_addr_1(1),
          A2     => fifo_31_wr_addr_1(2),
          A3     => fifo_31_wr_addr_1(3),
          D      => ddr_dq_in(28),      
          DPRA0  => fifo30_rd_addr_r(0),
          DPRA1  => fifo30_rd_addr_r(1),
          DPRA2  => fifo30_rd_addr_r(2),
          DPRA3  => fifo30_rd_addr_r(3),
          WCLK   =>  dqs3_delayed_col1_n,          
          WE     => fifo_31_wr_en_1 ); 
                      
fifo0_bit29 : RAM16X1D               
port map (DPO    => fifo_30_data_out(5),          
          A0     => fifo_30_wr_addr_1(0),          
          A1     => fifo_30_wr_addr_1(1),
          A2     => fifo_30_wr_addr_1(2),
          A3     => fifo_30_wr_addr_1(3),
          D      => ddr_dq_in(29),      
          DPRA0  => fifo31_rd_addr_r(0),
          DPRA1  => fifo31_rd_addr_r(1),
          DPRA2  => fifo31_rd_addr_r(2),
          DPRA3  => fifo31_rd_addr_r(3),
          WCLK   => dqs3_delayed_col0,          
          WE     => fifo_30_wr_en_1 );                       

fifo1_bit29 : RAM16X1D               
port map (DPO    => fifo_31_data_out(5),          
          A0     => fifo_31_wr_addr(0),          
          A1     => fifo_31_wr_addr(1),
          A2     => fifo_31_wr_addr(2),
          A3     => fifo_31_wr_addr(3),
          D      => ddr_dq_in(29),      
          DPRA0  => fifo31_rd_addr_r(0),
          DPRA1  => fifo31_rd_addr_r(1),
          DPRA2  => fifo31_rd_addr_r(2),
          DPRA3  => fifo31_rd_addr_r(3),
          WCLK   =>  dqs3_delayed_col0_n,          
          WE     => fifo_31_wr_en );                       



fifo0_bit30 : RAM16X1D               
port map (DPO    => fifo_30_data_out(6),          
          A0     => fifo_30_wr_addr(0),          
          A1     => fifo_30_wr_addr(1),
          A2     => fifo_30_wr_addr(2),
          A3     => fifo_30_wr_addr(3),
          D      => ddr_dq_in(30),      
          DPRA0  => fifo30_rd_addr_r(0),
          DPRA1  => fifo30_rd_addr_r(1),
          DPRA2  => fifo30_rd_addr_r(2),
          DPRA3  => fifo30_rd_addr_r(3),
          WCLK   => dqs3_delayed_col1,          
          WE     => fifo_30_wr_en );                       

fifo1_bit30 : RAM16X1D               
port map (DPO    => fifo_31_data_out(6),          
          A0     => fifo_31_wr_addr_1(0),          
          A1     => fifo_31_wr_addr_1(1),
          A2     => fifo_31_wr_addr_1(2),
          A3     => fifo_31_wr_addr_1(3),
          D      => ddr_dq_in(30),      
          DPRA0  => fifo30_rd_addr_r(0),
          DPRA1  => fifo30_rd_addr_r(1),
          DPRA2  => fifo30_rd_addr_r(2),
          DPRA3  => fifo30_rd_addr_r(3),
          WCLK   =>  dqs3_delayed_col1_n,          
          WE     => fifo_31_wr_en_1 ); 
                      
fifo0_bit31 : RAM16X1D               
port map (DPO    => fifo_30_data_out(7),          
          A0     => fifo_30_wr_addr_1(0),          
          A1     => fifo_30_wr_addr_1(1),
          A2     => fifo_30_wr_addr_1(2),
          A3     => fifo_30_wr_addr_1(3),
          D      => ddr_dq_in(31),      
          DPRA0  => fifo31_rd_addr_r(0),
          DPRA1  => fifo31_rd_addr_r(1),
          DPRA2  => fifo31_rd_addr_r(2),
          DPRA3  => fifo31_rd_addr_r(3),
          WCLK   => dqs3_delayed_col0,          
          WE     => fifo_30_wr_en_1 );                       

fifo1_bit31 : RAM16X1D               
port map (DPO    => fifo_31_data_out(7),          
          A0     => fifo_31_wr_addr(0),          
          A1     => fifo_31_wr_addr(1),
          A2     => fifo_31_wr_addr(2),
          A3     => fifo_31_wr_addr(3),
          D      => ddr_dq_in(31),      
          DPRA0  => fifo31_rd_addr_r(0),
          DPRA1  => fifo31_rd_addr_r(1),
          DPRA2  => fifo31_rd_addr_r(2),
          DPRA3  => fifo31_rd_addr_r(3),
          WCLK   =>  dqs3_delayed_col0_n,          
          WE     => fifo_31_wr_en ); 


--- Byte4 instantiation

fifo0_bit32 : RAM16X1D               
port map (DPO    => fifo_40_data_out(0),          
          A0     => fifo_40_wr_addr(0),          
          A1     => fifo_40_wr_addr(1),
          A2     => fifo_40_wr_addr(2),
          A3     => fifo_40_wr_addr(3),
          D      => ddr_dq_in(32),      
          DPRA0  => fifo40_rd_addr_r(0),
          DPRA1  => fifo40_rd_addr_r(1),
          DPRA2  => fifo40_rd_addr_r(2),
          DPRA3  => fifo40_rd_addr_r(3),
          WCLK   => dqs4_delayed_col1,          
          WE     => fifo_40_wr_en );                       

fifo1_bit32 : RAM16X1D               
port map (DPO    => fifo_41_data_out(0),          
          A0     => fifo_41_wr_addr_1(0),          
          A1     => fifo_41_wr_addr_1(1),
          A2     => fifo_41_wr_addr_1(2),
          A3     => fifo_41_wr_addr_1(3),
          D      => ddr_dq_in(32),      
          DPRA0  => fifo40_rd_addr_r(0),
          DPRA1  => fifo40_rd_addr_r(1),
          DPRA2  => fifo40_rd_addr_r(2),
          DPRA3  => fifo40_rd_addr_r(3),
          WCLK   =>  dqs4_delayed_col1_n,          
          WE     => fifo_41_wr_en_1 ); 
                      
fifo0_bit33 : RAM16X1D               
port map (DPO    => fifo_40_data_out(1),          
          A0     => fifo_40_wr_addr_1(0),          
          A1     => fifo_40_wr_addr_1(1),
          A2     => fifo_40_wr_addr_1(2),
          A3     => fifo_40_wr_addr_1(3),
          D      => ddr_dq_in(33),      
          DPRA0  => fifo41_rd_addr_r(0),
          DPRA1  => fifo41_rd_addr_r(1),
          DPRA2  => fifo41_rd_addr_r(2),
          DPRA3  => fifo41_rd_addr_r(3),
          WCLK   => dqs4_delayed_col0,          
          WE     => fifo_40_wr_en_1 );                       

fifo1_bit33 : RAM16X1D               
port map (DPO    => fifo_41_data_out(1),          
          A0     => fifo_41_wr_addr(0),          
          A1     => fifo_41_wr_addr(1),
          A2     => fifo_41_wr_addr(2),
          A3     => fifo_41_wr_addr(3),
          D      => ddr_dq_in(33),      
          DPRA0  => fifo41_rd_addr_r(0),
          DPRA1  => fifo41_rd_addr_r(1),
          DPRA2  => fifo41_rd_addr_r(2),
          DPRA3  => fifo41_rd_addr_r(3),
          WCLK   =>  dqs4_delayed_col0_n,          
          WE     => fifo_41_wr_en );                       



fifo0_bit34 : RAM16X1D               
port map (DPO    => fifo_40_data_out(2),          
          A0     => fifo_40_wr_addr(0),          
          A1     => fifo_40_wr_addr(1),
          A2     => fifo_40_wr_addr(2),
          A3     => fifo_40_wr_addr(3),
          D      => ddr_dq_in(34),      
          DPRA0  => fifo40_rd_addr_r(0),
          DPRA1  => fifo40_rd_addr_r(1),
          DPRA2  => fifo40_rd_addr_r(2),
          DPRA3  => fifo40_rd_addr_r(3),
          WCLK   => dqs4_delayed_col1,          
          WE     => fifo_40_wr_en );                       

fifo1_bit34 : RAM16X1D               
port map (DPO    => fifo_41_data_out(2),          
          A0     => fifo_41_wr_addr_1(0),          
          A1     => fifo_41_wr_addr_1(1),
          A2     => fifo_41_wr_addr_1(2),
          A3     => fifo_41_wr_addr_1(3),
          D      => ddr_dq_in(34),      
          DPRA0  => fifo40_rd_addr_r(0),
          DPRA1  => fifo40_rd_addr_r(1),
          DPRA2  => fifo40_rd_addr_r(2),
          DPRA3  => fifo40_rd_addr_r(3),
          WCLK   =>  dqs4_delayed_col1_n,          
          WE     => fifo_41_wr_en_1 ); 
                      
fifo0_bit35 : RAM16X1D               
port map (DPO    => fifo_40_data_out(3),          
          A0     => fifo_40_wr_addr_1(0),          
          A1     => fifo_40_wr_addr_1(1),
          A2     => fifo_40_wr_addr_1(2),
          A3     => fifo_40_wr_addr_1(3),
          D      => ddr_dq_in(35),      
          DPRA0  => fifo41_rd_addr_r(0),
          DPRA1  => fifo41_rd_addr_r(1),
          DPRA2  => fifo41_rd_addr_r(2),
          DPRA3  => fifo41_rd_addr_r(3),
          WCLK   => dqs4_delayed_col0,          
          WE     => fifo_40_wr_en_1 );                       

fifo1_bit35 : RAM16X1D               
port map (DPO    => fifo_41_data_out(3),          
          A0     => fifo_41_wr_addr(0),          
          A1     => fifo_41_wr_addr(1),
          A2     => fifo_41_wr_addr(2),
          A3     => fifo_41_wr_addr(3),
          D      => ddr_dq_in(35),      
          DPRA0  => fifo41_rd_addr_r(0),
          DPRA1  => fifo41_rd_addr_r(1),
          DPRA2  => fifo41_rd_addr_r(2),
          DPRA3  => fifo41_rd_addr_r(3),
          WCLK   =>  dqs4_delayed_col0_n,          
          WE     => fifo_41_wr_en ); 


                      
fifo0_bit36 : RAM16X1D               
port map (DPO    => fifo_40_data_out(4),          
          A0     => fifo_40_wr_addr(0),          
          A1     => fifo_40_wr_addr(1),
          A2     => fifo_40_wr_addr(2),
          A3     => fifo_40_wr_addr(3),
          D      => ddr_dq_in(36),      
          DPRA0  => fifo40_rd_addr_r(0),
          DPRA1  => fifo40_rd_addr_r(1),
          DPRA2  => fifo40_rd_addr_r(2),
          DPRA3  => fifo40_rd_addr_r(3),
          WCLK   => dqs4_delayed_col1,          
          WE     => fifo_40_wr_en );                       

fifo1_bit36 : RAM16X1D               
port map (DPO    => fifo_41_data_out(4),          
          A0     => fifo_41_wr_addr_1(0),          
          A1     => fifo_41_wr_addr_1(1),
          A2     => fifo_41_wr_addr_1(2),
          A3     => fifo_41_wr_addr_1(3),
          D      => ddr_dq_in(36),      
          DPRA0  => fifo40_rd_addr_r(0),
          DPRA1  => fifo40_rd_addr_r(1),
          DPRA2  => fifo40_rd_addr_r(2),
          DPRA3  => fifo40_rd_addr_r(3),
          WCLK   =>  dqs4_delayed_col1_n,          
          WE     => fifo_41_wr_en_1 ); 
                      
fifo0_bit37 : RAM16X1D               
port map (DPO    => fifo_40_data_out(5),          
          A0     => fifo_40_wr_addr_1(0),          
          A1     => fifo_40_wr_addr_1(1),
          A2     => fifo_40_wr_addr_1(2),
          A3     => fifo_40_wr_addr_1(3),
          D      => ddr_dq_in(37),      
          DPRA0  => fifo41_rd_addr_r(0),
          DPRA1  => fifo41_rd_addr_r(1),
          DPRA2  => fifo41_rd_addr_r(2),
          DPRA3  => fifo41_rd_addr_r(3),
          WCLK   => dqs4_delayed_col0,          
          WE     => fifo_40_wr_en_1 );                       

fifo1_bit37 : RAM16X1D               
port map (DPO    => fifo_41_data_out(5),          
          A0     => fifo_41_wr_addr(0),          
          A1     => fifo_41_wr_addr(1),
          A2     => fifo_41_wr_addr(2),
          A3     => fifo_41_wr_addr(3),
          D      => ddr_dq_in(37),      
          DPRA0  => fifo41_rd_addr_r(0),
          DPRA1  => fifo41_rd_addr_r(1),
          DPRA2  => fifo41_rd_addr_r(2),
          DPRA3  => fifo41_rd_addr_r(3),
          WCLK   =>  dqs4_delayed_col0_n,          
          WE     => fifo_41_wr_en );                       



fifo0_bit38 : RAM16X1D               
port map (DPO    => fifo_40_data_out(6),          
          A0     => fifo_40_wr_addr(0),          
          A1     => fifo_40_wr_addr(1),
          A2     => fifo_40_wr_addr(2),
          A3     => fifo_40_wr_addr(3),
          D      => ddr_dq_in(38),      
          DPRA0  => fifo40_rd_addr_r(0),
          DPRA1  => fifo40_rd_addr_r(1),
          DPRA2  => fifo40_rd_addr_r(2),
          DPRA3  => fifo40_rd_addr_r(3),
          WCLK   => dqs4_delayed_col1,          
          WE     => fifo_40_wr_en );                       

fifo1_bit38 : RAM16X1D               
port map (DPO    => fifo_41_data_out(6),          
          A0     => fifo_41_wr_addr_1(0),          
          A1     => fifo_41_wr_addr_1(1),
          A2     => fifo_41_wr_addr_1(2),
          A3     => fifo_41_wr_addr_1(3),
          D      => ddr_dq_in(38),      
          DPRA0  => fifo40_rd_addr_r(0),
          DPRA1  => fifo40_rd_addr_r(1),
          DPRA2  => fifo40_rd_addr_r(2),
          DPRA3  => fifo40_rd_addr_r(3),
          WCLK   =>  dqs4_delayed_col1_n,          
          WE     => fifo_41_wr_en_1 ); 
                      
fifo0_bit39 : RAM16X1D               
port map (DPO    => fifo_40_data_out(7),          
          A0     => fifo_40_wr_addr_1(0),          
          A1     => fifo_40_wr_addr_1(1),
          A2     => fifo_40_wr_addr_1(2),
          A3     => fifo_40_wr_addr_1(3),
          D      => ddr_dq_in(39),      
          DPRA0  => fifo41_rd_addr_r(0),
          DPRA1  => fifo41_rd_addr_r(1),
          DPRA2  => fifo41_rd_addr_r(2),
          DPRA3  => fifo41_rd_addr_r(3),
          WCLK   => dqs4_delayed_col0,          
          WE     => fifo_40_wr_en_1 );                       

fifo1_bit39 : RAM16X1D               
port map (DPO    => fifo_41_data_out(7),          
          A0     => fifo_41_wr_addr(0),          
          A1     => fifo_41_wr_addr(1),
          A2     => fifo_41_wr_addr(2),
          A3     => fifo_41_wr_addr(3),
          D      => ddr_dq_in(39),      
          DPRA0  => fifo41_rd_addr_r(0),
          DPRA1  => fifo41_rd_addr_r(1),
          DPRA2  => fifo41_rd_addr_r(2),
          DPRA3  => fifo41_rd_addr_r(3),
          WCLK   =>  dqs4_delayed_col0_n,          
          WE     => fifo_41_wr_en ); 
                                     

-- Byte5 Fifo instantiation 

fifo0_bit40 : RAM16X1D               
port map (DPO    => fifo_50_data_out(0),          
          A0     => fifo_50_wr_addr(0),          
          A1     => fifo_50_wr_addr(1),
          A2     => fifo_50_wr_addr(2),
          A3     => fifo_50_wr_addr(3),
          D      => ddr_dq_in(40),      
          DPRA0  => fifo50_rd_addr_r(0),
          DPRA1  => fifo50_rd_addr_r(1),
          DPRA2  => fifo50_rd_addr_r(2),
          DPRA3  => fifo50_rd_addr_r(3),
          WCLK   => dqs5_delayed_col1,          
          WE     => fifo_50_wr_en );                       

fifo1_bit40 : RAM16X1D               
port map (DPO    => fifo_51_data_out(0),          
          A0     => fifo_51_wr_addr_1(0),          
          A1     => fifo_51_wr_addr_1(1),
          A2     => fifo_51_wr_addr_1(2),
          A3     => fifo_51_wr_addr_1(3),
          D      => ddr_dq_in(40),      
          DPRA0  => fifo50_rd_addr_r(0),
          DPRA1  => fifo50_rd_addr_r(1),
          DPRA2  => fifo50_rd_addr_r(2),
          DPRA3  => fifo50_rd_addr_r(3),
          WCLK   =>  dqs5_delayed_col1_n,          
          WE     => fifo_51_wr_en_1 ); 
                      
fifo0_bit41 : RAM16X1D               
port map (DPO    => fifo_50_data_out(1),          
          A0     => fifo_50_wr_addr_1(0),          
          A1     => fifo_50_wr_addr_1(1),
          A2     => fifo_50_wr_addr_1(2),
          A3     => fifo_50_wr_addr_1(3),
          D      => ddr_dq_in(41),      
          DPRA0  => fifo51_rd_addr_r(0),
          DPRA1  => fifo51_rd_addr_r(1),
          DPRA2  => fifo51_rd_addr_r(2),
          DPRA3  => fifo51_rd_addr_r(3),
          WCLK   => dqs5_delayed_col0,          
          WE     => fifo_50_wr_en_1 );                       

fifo1_bit41 : RAM16X1D               
port map (DPO    => fifo_51_data_out(1),          
          A0     => fifo_51_wr_addr(0),          
          A1     => fifo_51_wr_addr(1),
          A2     => fifo_51_wr_addr(2),
          A3     => fifo_51_wr_addr(3),
          D      => ddr_dq_in(41),      
          DPRA0  => fifo51_rd_addr_r(0),
          DPRA1  => fifo51_rd_addr_r(1),
          DPRA2  => fifo51_rd_addr_r(2),
          DPRA3  => fifo51_rd_addr_r(3),
          WCLK   =>  dqs5_delayed_col0_n,          
          WE     => fifo_51_wr_en );                       



fifo0_bit42 : RAM16X1D               
port map (DPO    => fifo_50_data_out(2),          
          A0     => fifo_50_wr_addr(0),          
          A1     => fifo_50_wr_addr(1),
          A2     => fifo_50_wr_addr(2),
          A3     => fifo_50_wr_addr(3),
          D      => ddr_dq_in(42),      
          DPRA0  => fifo50_rd_addr_r(0),
          DPRA1  => fifo50_rd_addr_r(1),
          DPRA2  => fifo50_rd_addr_r(2),
          DPRA3  => fifo50_rd_addr_r(3),
          WCLK   => dqs5_delayed_col1,          
          WE     => fifo_50_wr_en );                       

fifo1_bit42 : RAM16X1D               
port map (DPO    => fifo_51_data_out(2),          
          A0     => fifo_51_wr_addr_1(0),          
          A1     => fifo_51_wr_addr_1(1),
          A2     => fifo_51_wr_addr_1(2),
          A3     => fifo_51_wr_addr_1(3),
          D      => ddr_dq_in(42),      
          DPRA0  => fifo50_rd_addr_r(0),
          DPRA1  => fifo50_rd_addr_r(1),
          DPRA2  => fifo50_rd_addr_r(2),
          DPRA3  => fifo50_rd_addr_r(3),
          WCLK   =>  dqs5_delayed_col1_n,          
          WE     => fifo_51_wr_en_1 ); 
                      
fifo0_bit43 : RAM16X1D               
port map (DPO    => fifo_50_data_out(3),          
          A0     => fifo_50_wr_addr_1(0),          
          A1     => fifo_50_wr_addr_1(1),
          A2     => fifo_50_wr_addr_1(2),
          A3     => fifo_50_wr_addr_1(3),
          D      => ddr_dq_in(43),      
          DPRA0  => fifo51_rd_addr_r(0),
          DPRA1  => fifo51_rd_addr_r(1),
          DPRA2  => fifo51_rd_addr_r(2),
          DPRA3  => fifo51_rd_addr_r(3),
          WCLK   => dqs5_delayed_col0,          
          WE     => fifo_50_wr_en_1 );                       

fifo1_bit43 : RAM16X1D               
port map (DPO    => fifo_51_data_out(3),          
          A0     => fifo_51_wr_addr(0),          
          A1     => fifo_51_wr_addr(1),
          A2     => fifo_51_wr_addr(2),
          A3     => fifo_51_wr_addr(3),
          D      => ddr_dq_in(43),      
          DPRA0  => fifo51_rd_addr_r(0),
          DPRA1  => fifo51_rd_addr_r(1),
          DPRA2  => fifo51_rd_addr_r(2),
          DPRA3  => fifo51_rd_addr_r(3),
          WCLK   =>  dqs5_delayed_col0_n,          
          WE     => fifo_51_wr_en ); 


                      
fifo0_bit44 : RAM16X1D               
port map (DPO    => fifo_50_data_out(4),          
          A0     => fifo_50_wr_addr(0),          
          A1     => fifo_50_wr_addr(1),
          A2     => fifo_50_wr_addr(2),
          A3     => fifo_50_wr_addr(3),
          D      => ddr_dq_in(44),      
          DPRA0  => fifo50_rd_addr_r(0),
          DPRA1  => fifo50_rd_addr_r(1),
          DPRA2  => fifo50_rd_addr_r(2),
          DPRA3  => fifo50_rd_addr_r(3),
          WCLK   => dqs5_delayed_col1,          
          WE     => fifo_50_wr_en );                       

fifo1_bit44 : RAM16X1D               
port map (DPO    => fifo_51_data_out(4),          
          A0     => fifo_51_wr_addr_1(0),          
          A1     => fifo_51_wr_addr_1(1),
          A2     => fifo_51_wr_addr_1(2),
          A3     => fifo_51_wr_addr_1(3),
          D      => ddr_dq_in(44),      
          DPRA0  => fifo50_rd_addr_r(0),
          DPRA1  => fifo50_rd_addr_r(1),
          DPRA2  => fifo50_rd_addr_r(2),
          DPRA3  => fifo50_rd_addr_r(3),
          WCLK   =>  dqs5_delayed_col1_n,          
          WE     => fifo_51_wr_en_1 ); 
                      
fifo0_bit45 : RAM16X1D               
port map (DPO    => fifo_50_data_out(5),          
          A0     => fifo_50_wr_addr_1(0),          
          A1     => fifo_50_wr_addr_1(1),
          A2     => fifo_50_wr_addr_1(2),
          A3     => fifo_50_wr_addr_1(3),
          D      => ddr_dq_in(45),      
          DPRA0  => fifo51_rd_addr_r(0),
          DPRA1  => fifo51_rd_addr_r(1),
          DPRA2  => fifo51_rd_addr_r(2),
          DPRA3  => fifo51_rd_addr_r(3),
          WCLK   => dqs5_delayed_col0,          
          WE     => fifo_50_wr_en_1 );                       

fifo1_bit45 : RAM16X1D               
port map (DPO    => fifo_51_data_out(5),          
          A0     => fifo_51_wr_addr(0),          
          A1     => fifo_51_wr_addr(1),
          A2     => fifo_51_wr_addr(2),
          A3     => fifo_51_wr_addr(3),
          D      => ddr_dq_in(45),      
          DPRA0  => fifo51_rd_addr_r(0),
          DPRA1  => fifo51_rd_addr_r(1),
          DPRA2  => fifo51_rd_addr_r(2),
          DPRA3  => fifo51_rd_addr_r(3),
          WCLK   =>  dqs5_delayed_col0_n,          
          WE     => fifo_51_wr_en );                       



fifo0_bit46 : RAM16X1D               
port map (DPO    => fifo_50_data_out(6),          
          A0     => fifo_50_wr_addr(0),          
          A1     => fifo_50_wr_addr(1),
          A2     => fifo_50_wr_addr(2),
          A3     => fifo_50_wr_addr(3),
          D      => ddr_dq_in(46),      
          DPRA0  => fifo50_rd_addr_r(0),
          DPRA1  => fifo50_rd_addr_r(1),
          DPRA2  => fifo50_rd_addr_r(2),
          DPRA3  => fifo50_rd_addr_r(3),
          WCLK   => dqs5_delayed_col1,          
          WE     => fifo_50_wr_en );                       

fifo1_bit46 : RAM16X1D               
port map (DPO    => fifo_51_data_out(6),          
          A0     => fifo_51_wr_addr_1(0),          
          A1     => fifo_51_wr_addr_1(1),
          A2     => fifo_51_wr_addr_1(2),
          A3     => fifo_51_wr_addr_1(3),
          D      => ddr_dq_in(46),      
          DPRA0  => fifo50_rd_addr_r(0),
          DPRA1  => fifo50_rd_addr_r(1),
          DPRA2  => fifo50_rd_addr_r(2),
          DPRA3  => fifo50_rd_addr_r(3),
          WCLK   =>  dqs5_delayed_col1_n,          
          WE     => fifo_51_wr_en_1 ); 
                      
fifo0_bit47 : RAM16X1D               
port map (DPO    => fifo_50_data_out(7),          
          A0     => fifo_50_wr_addr_1(0),          
          A1     => fifo_50_wr_addr_1(1),
          A2     => fifo_50_wr_addr_1(2),
          A3     => fifo_50_wr_addr_1(3),
          D      => ddr_dq_in(47),      
          DPRA0  => fifo51_rd_addr_r(0),
          DPRA1  => fifo51_rd_addr_r(1),
          DPRA2  => fifo51_rd_addr_r(2),
          DPRA3  => fifo51_rd_addr_r(3),
          WCLK   => dqs5_delayed_col0,          
          WE     => fifo_50_wr_en_1 );                       

fifo1_bit47 : RAM16X1D               
port map (DPO    => fifo_51_data_out(7),          
          A0     => fifo_51_wr_addr(0),          
          A1     => fifo_51_wr_addr(1),
          A2     => fifo_51_wr_addr(2),
          A3     => fifo_51_wr_addr(3),
          D      => ddr_dq_in(47),      
          DPRA0  => fifo51_rd_addr_r(0),
          DPRA1  => fifo51_rd_addr_r(1),
          DPRA2  => fifo51_rd_addr_r(2),
          DPRA3  => fifo51_rd_addr_r(3),
          WCLK   =>  dqs5_delayed_col0_n,          
          WE     => fifo_51_wr_en ); 
                                    

-- Byte6 Fifo instantiation

fifo0_bit48 : RAM16X1D               
port map (DPO    => fifo_60_data_out(0),          
          A0     => fifo_60_wr_addr(0),          
          A1     => fifo_60_wr_addr(1),
          A2     => fifo_60_wr_addr(2),
          A3     => fifo_60_wr_addr(3),
          D      => ddr_dq_in(48),      
          DPRA0  => fifo60_rd_addr_r(0),
          DPRA1  => fifo60_rd_addr_r(1),
          DPRA2  => fifo60_rd_addr_r(2),
          DPRA3  => fifo60_rd_addr_r(3),
          WCLK   => dqs6_delayed_col1,          
          WE     => fifo_60_wr_en );                       

fifo1_bit48 : RAM16X1D               
port map (DPO    => fifo_61_data_out(0),          
          A0     => fifo_61_wr_addr_1(0),          
          A1     => fifo_61_wr_addr_1(1),
          A2     => fifo_61_wr_addr_1(2),
          A3     => fifo_61_wr_addr_1(3),
          D      => ddr_dq_in(48),      
          DPRA0  => fifo60_rd_addr_r(0),
          DPRA1  => fifo60_rd_addr_r(1),
          DPRA2  => fifo60_rd_addr_r(2),
          DPRA3  => fifo60_rd_addr_r(3),
          WCLK   =>  dqs6_delayed_col1_n,          
          WE     => fifo_61_wr_en_1 ); 
                      
fifo0_bit49 : RAM16X1D               
port map (DPO    => fifo_60_data_out(1),          
          A0     => fifo_60_wr_addr_1(0),          
          A1     => fifo_60_wr_addr_1(1),
          A2     => fifo_60_wr_addr_1(2),
          A3     => fifo_60_wr_addr_1(3),
          D      => ddr_dq_in(49),      
          DPRA0  => fifo61_rd_addr_r(0),
          DPRA1  => fifo61_rd_addr_r(1),
          DPRA2  => fifo61_rd_addr_r(2),
          DPRA3  => fifo61_rd_addr_r(3),
          WCLK   => dqs6_delayed_col0,          
          WE     => fifo_60_wr_en_1 );                       

fifo1_bit49 : RAM16X1D               
port map (DPO    => fifo_61_data_out(1),          
          A0     => fifo_61_wr_addr(0),          
          A1     => fifo_61_wr_addr(1),
          A2     => fifo_61_wr_addr(2),
          A3     => fifo_61_wr_addr(3),
          D      => ddr_dq_in(49),      
          DPRA0  => fifo61_rd_addr_r(0),
          DPRA1  => fifo61_rd_addr_r(1),
          DPRA2  => fifo61_rd_addr_r(2),
          DPRA3  => fifo61_rd_addr_r(3),
          WCLK   =>  dqs6_delayed_col0_n,          
          WE     => fifo_61_wr_en );                       



fifo0_bit50 : RAM16X1D               
port map (DPO    => fifo_60_data_out(2),          
          A0     => fifo_60_wr_addr(0),          
          A1     => fifo_60_wr_addr(1),
          A2     => fifo_60_wr_addr(2),
          A3     => fifo_60_wr_addr(3),
          D      => ddr_dq_in(50),      
          DPRA0  => fifo60_rd_addr_r(0),
          DPRA1  => fifo60_rd_addr_r(1),
          DPRA2  => fifo60_rd_addr_r(2),
          DPRA3  => fifo60_rd_addr_r(3),
          WCLK   => dqs6_delayed_col1,          
          WE     => fifo_60_wr_en );                       

fifo1_bit50 : RAM16X1D               
port map (DPO    => fifo_61_data_out(2),          
          A0     => fifo_61_wr_addr_1(0),          
          A1     => fifo_61_wr_addr_1(1),
          A2     => fifo_61_wr_addr_1(2),
          A3     => fifo_61_wr_addr_1(3),
          D      => ddr_dq_in(50),      
          DPRA0  => fifo60_rd_addr_r(0),
          DPRA1  => fifo60_rd_addr_r(1),
          DPRA2  => fifo60_rd_addr_r(2),
          DPRA3  => fifo60_rd_addr_r(3),
          WCLK   =>  dqs6_delayed_col1_n,          
          WE     => fifo_61_wr_en_1 ); 
                      
fifo0_bit51 : RAM16X1D               
port map (DPO    => fifo_60_data_out(3),          
          A0     => fifo_60_wr_addr_1(0),          
          A1     => fifo_60_wr_addr_1(1),
          A2     => fifo_60_wr_addr_1(2),
          A3     => fifo_60_wr_addr_1(3),
          D      => ddr_dq_in(51),      
          DPRA0  => fifo61_rd_addr_r(0),
          DPRA1  => fifo61_rd_addr_r(1),
          DPRA2  => fifo61_rd_addr_r(2),
          DPRA3  => fifo61_rd_addr_r(3),
          WCLK   => dqs6_delayed_col0,          
          WE     => fifo_60_wr_en_1 );                       

fifo1_bit51 : RAM16X1D               
port map (DPO    => fifo_61_data_out(3),          
          A0     => fifo_61_wr_addr(0),          
          A1     => fifo_61_wr_addr(1),
          A2     => fifo_61_wr_addr(2),
          A3     => fifo_61_wr_addr(3),
          D      => ddr_dq_in(51),      
          DPRA0  => fifo61_rd_addr_r(0),
          DPRA1  => fifo61_rd_addr_r(1),
          DPRA2  => fifo61_rd_addr_r(2),
          DPRA3  => fifo61_rd_addr_r(3),
          WCLK   =>  dqs6_delayed_col0_n,          
          WE     => fifo_61_wr_en ); 


                      
fifo0_bit52 : RAM16X1D               
port map (DPO    => fifo_60_data_out(4),          
          A0     => fifo_60_wr_addr(0),          
          A1     => fifo_60_wr_addr(1),
          A2     => fifo_60_wr_addr(2),
          A3     => fifo_60_wr_addr(3),
          D      => ddr_dq_in(52),      
          DPRA0  => fifo60_rd_addr_r(0),
          DPRA1  => fifo60_rd_addr_r(1),
          DPRA2  => fifo60_rd_addr_r(2),
          DPRA3  => fifo60_rd_addr_r(3),
          WCLK   => dqs6_delayed_col1,          
          WE     => fifo_60_wr_en );                       

fifo1_bit52 : RAM16X1D               
port map (DPO    => fifo_61_data_out(4),          
          A0     => fifo_61_wr_addr_1(0),          
          A1     => fifo_61_wr_addr_1(1),
          A2     => fifo_61_wr_addr_1(2),
          A3     => fifo_61_wr_addr_1(3),
          D      => ddr_dq_in(52),      
          DPRA0  => fifo60_rd_addr_r(0),
          DPRA1  => fifo60_rd_addr_r(1),
          DPRA2  => fifo60_rd_addr_r(2),
          DPRA3  => fifo60_rd_addr_r(3),
          WCLK   =>  dqs6_delayed_col1_n,          
          WE     => fifo_61_wr_en_1 ); 
                      
fifo0_bit53 : RAM16X1D               
port map (DPO    => fifo_60_data_out(5),          
          A0     => fifo_60_wr_addr_1(0),          
          A1     => fifo_60_wr_addr_1(1),
          A2     => fifo_60_wr_addr_1(2),
          A3     => fifo_60_wr_addr_1(3),
          D      => ddr_dq_in(53),      
          DPRA0  => fifo61_rd_addr_r(0),
          DPRA1  => fifo61_rd_addr_r(1),
          DPRA2  => fifo61_rd_addr_r(2),
          DPRA3  => fifo61_rd_addr_r(3),
          WCLK   => dqs6_delayed_col0,          
          WE     => fifo_60_wr_en_1 );                       

fifo1_bit53 : RAM16X1D               
port map (DPO    => fifo_61_data_out(5),          
          A0     => fifo_61_wr_addr(0),          
          A1     => fifo_61_wr_addr(1),
          A2     => fifo_61_wr_addr(2),
          A3     => fifo_61_wr_addr(3),
          D      => ddr_dq_in(53),      
          DPRA0  => fifo61_rd_addr_r(0),
          DPRA1  => fifo61_rd_addr_r(1),
          DPRA2  => fifo61_rd_addr_r(2),
          DPRA3  => fifo61_rd_addr_r(3),
          WCLK   =>  dqs6_delayed_col0_n,          
          WE     => fifo_61_wr_en );                       



fifo0_bit54 : RAM16X1D               
port map (DPO    => fifo_60_data_out(6),          
          A0     => fifo_60_wr_addr(0),          
          A1     => fifo_60_wr_addr(1),
          A2     => fifo_60_wr_addr(2),
          A3     => fifo_60_wr_addr(3),
          D      => ddr_dq_in(54),      
          DPRA0  => fifo60_rd_addr_r(0),
          DPRA1  => fifo60_rd_addr_r(1),
          DPRA2  => fifo60_rd_addr_r(2),
          DPRA3  => fifo60_rd_addr_r(3),
          WCLK   => dqs6_delayed_col1,          
          WE     => fifo_60_wr_en );                       

fifo1_bit54 : RAM16X1D               
port map (DPO    => fifo_61_data_out(6),          
          A0     => fifo_61_wr_addr_1(0),          
          A1     => fifo_61_wr_addr_1(1),
          A2     => fifo_61_wr_addr_1(2),
          A3     => fifo_61_wr_addr_1(3),
          D      => ddr_dq_in(54),      
          DPRA0  => fifo60_rd_addr_r(0),
          DPRA1  => fifo60_rd_addr_r(1),
          DPRA2  => fifo60_rd_addr_r(2),
          DPRA3  => fifo60_rd_addr_r(3),
          WCLK   =>  dqs6_delayed_col1_n,          
          WE     => fifo_61_wr_en_1 ); 
                      
fifo0_bit55 : RAM16X1D               
port map (DPO    => fifo_60_data_out(7),          
          A0     => fifo_60_wr_addr_1(0),          
          A1     => fifo_60_wr_addr_1(1),
          A2     => fifo_60_wr_addr_1(2),
          A3     => fifo_60_wr_addr_1(3),
          D      => ddr_dq_in(55),      
          DPRA0  => fifo61_rd_addr_r(0),
          DPRA1  => fifo61_rd_addr_r(1),
          DPRA2  => fifo61_rd_addr_r(2),
          DPRA3  => fifo61_rd_addr_r(3),
          WCLK   => dqs6_delayed_col0,          
          WE     => fifo_60_wr_en_1 );                       

fifo1_bit55 : RAM16X1D               
port map (DPO    => fifo_61_data_out(7),          
          A0     => fifo_61_wr_addr(0),          
          A1     => fifo_61_wr_addr(1),
          A2     => fifo_61_wr_addr(2),
          A3     => fifo_61_wr_addr(3),
          D      => ddr_dq_in(55),      
          DPRA0  => fifo61_rd_addr_r(0),
          DPRA1  => fifo61_rd_addr_r(1),
          DPRA2  => fifo61_rd_addr_r(2),
          DPRA3  => fifo61_rd_addr_r(3),
          WCLK   =>  dqs6_delayed_col0_n,          
          WE     => fifo_61_wr_en ); 


--Byte7 Fifo Instantiation 

fifo0_bit56 : RAM16X1D               
port map (DPO    => fifo_70_data_out(0),          
          A0     => fifo_70_wr_addr(0),          
          A1     => fifo_70_wr_addr(1),
          A2     => fifo_70_wr_addr(2),
          A3     => fifo_70_wr_addr(3),
          D      => ddr_dq_in(56),      
          DPRA0  => fifo70_rd_addr_r(0),
          DPRA1  => fifo70_rd_addr_r(1),
          DPRA2  => fifo70_rd_addr_r(2),
          DPRA3  => fifo70_rd_addr_r(3),
          WCLK   => dqs7_delayed_col1,          
          WE     => fifo_70_wr_en );                       

fifo1_bit56 : RAM16X1D               
port map (DPO    => fifo_71_data_out(0),          
          A0     => fifo_71_wr_addr_1(0),          
          A1     => fifo_71_wr_addr_1(1),
          A2     => fifo_71_wr_addr_1(2),
          A3     => fifo_71_wr_addr_1(3),
          D      => ddr_dq_in(56),      
          DPRA0  => fifo70_rd_addr_r(0),
          DPRA1  => fifo70_rd_addr_r(1),
          DPRA2  => fifo70_rd_addr_r(2),
          DPRA3  => fifo70_rd_addr_r(3),
          WCLK   =>  dqs7_delayed_col1_n,          
          WE     => fifo_71_wr_en_1 ); 
                      
fifo0_bit57 : RAM16X1D               
port map (DPO    => fifo_70_data_out(1),          
          A0     => fifo_70_wr_addr_1(0),          
          A1     => fifo_70_wr_addr_1(1),
          A2     => fifo_70_wr_addr_1(2),
          A3     => fifo_70_wr_addr_1(3),
          D      => ddr_dq_in(57),      
          DPRA0  => fifo71_rd_addr_r(0),
          DPRA1  => fifo71_rd_addr_r(1),
          DPRA2  => fifo71_rd_addr_r(2),
          DPRA3  => fifo71_rd_addr_r(3),
          WCLK   => dqs7_delayed_col0,          
          WE     => fifo_70_wr_en_1 );                       

fifo1_bit57 : RAM16X1D               
port map (DPO    => fifo_71_data_out(1),          
          A0     => fifo_71_wr_addr(0),          
          A1     => fifo_71_wr_addr(1),
          A2     => fifo_71_wr_addr(2),
          A3     => fifo_71_wr_addr(3),
          D      => ddr_dq_in(57),      
          DPRA0  => fifo71_rd_addr_r(0),
          DPRA1  => fifo71_rd_addr_r(1),
          DPRA2  => fifo71_rd_addr_r(2),
          DPRA3  => fifo71_rd_addr_r(3),
          WCLK   =>  dqs7_delayed_col0_n,          
          WE     => fifo_71_wr_en );                       



fifo0_bit58 : RAM16X1D               
port map (DPO    => fifo_70_data_out(2),          
          A0     => fifo_70_wr_addr(0),          
          A1     => fifo_70_wr_addr(1),
          A2     => fifo_70_wr_addr(2),
          A3     => fifo_70_wr_addr(3),
          D      => ddr_dq_in(58),      
          DPRA0  => fifo70_rd_addr_r(0),
          DPRA1  => fifo70_rd_addr_r(1),
          DPRA2  => fifo70_rd_addr_r(2),
          DPRA3  => fifo70_rd_addr_r(3),
          WCLK   => dqs7_delayed_col1,          
          WE     => fifo_70_wr_en );                       

fifo1_bit58 : RAM16X1D               
port map (DPO    => fifo_71_data_out(2),          
          A0     => fifo_71_wr_addr_1(0),          
          A1     => fifo_71_wr_addr_1(1),
          A2     => fifo_71_wr_addr_1(2),
          A3     => fifo_71_wr_addr_1(3),
          D      => ddr_dq_in(58),      
          DPRA0  => fifo70_rd_addr_r(0),
          DPRA1  => fifo70_rd_addr_r(1),
          DPRA2  => fifo70_rd_addr_r(2),
          DPRA3  => fifo70_rd_addr_r(3),
          WCLK   =>  dqs7_delayed_col1_n,          
          WE     => fifo_71_wr_en_1 ); 
                      
fifo0_bit59 : RAM16X1D               
port map (DPO    => fifo_70_data_out(3),          
          A0     => fifo_70_wr_addr_1(0),          
          A1     => fifo_70_wr_addr_1(1),
          A2     => fifo_70_wr_addr_1(2),
          A3     => fifo_70_wr_addr_1(3),
          D      => ddr_dq_in(59),      
          DPRA0  => fifo71_rd_addr_r(0),
          DPRA1  => fifo71_rd_addr_r(1),
          DPRA2  => fifo71_rd_addr_r(2),
          DPRA3  => fifo71_rd_addr_r(3),
          WCLK   => dqs7_delayed_col0,          
          WE     => fifo_70_wr_en_1 );                       

fifo1_bit59 : RAM16X1D               
port map (DPO    => fifo_71_data_out(3),          
          A0     => fifo_71_wr_addr(0),          
          A1     => fifo_71_wr_addr(1),
          A2     => fifo_71_wr_addr(2),
          A3     => fifo_71_wr_addr(3),
          D      => ddr_dq_in(59),      
          DPRA0  => fifo71_rd_addr_r(0),
          DPRA1  => fifo71_rd_addr_r(1),
          DPRA2  => fifo71_rd_addr_r(2),
          DPRA3  => fifo71_rd_addr_r(3),
          WCLK   =>  dqs7_delayed_col0_n,          
          WE     => fifo_71_wr_en ); 


                      
fifo0_bit60 : RAM16X1D               
port map (DPO    => fifo_70_data_out(4),          
          A0     => fifo_70_wr_addr(0),          
          A1     => fifo_70_wr_addr(1),
          A2     => fifo_70_wr_addr(2),
          A3     => fifo_70_wr_addr(3),
          D      => ddr_dq_in(60),      
          DPRA0  => fifo70_rd_addr_r(0),
          DPRA1  => fifo70_rd_addr_r(1),
          DPRA2  => fifo70_rd_addr_r(2),
          DPRA3  => fifo70_rd_addr_r(3),
          WCLK   => dqs7_delayed_col1,          
          WE     => fifo_70_wr_en );                       

fifo1_bit60 : RAM16X1D               
port map (DPO    => fifo_71_data_out(4),          
          A0     => fifo_71_wr_addr_1(0),          
          A1     => fifo_71_wr_addr_1(1),
          A2     => fifo_71_wr_addr_1(2),
          A3     => fifo_71_wr_addr_1(3),
          D      => ddr_dq_in(60),      
          DPRA0  => fifo70_rd_addr_r(0),
          DPRA1  => fifo70_rd_addr_r(1),
          DPRA2  => fifo70_rd_addr_r(2),
          DPRA3  => fifo70_rd_addr_r(3),
          WCLK   =>  dqs7_delayed_col1_n,          
          WE     => fifo_71_wr_en_1 ); 
                      
fifo0_bit61 : RAM16X1D               
port map (DPO    => fifo_70_data_out(5),          
          A0     => fifo_70_wr_addr_1(0),          
          A1     => fifo_70_wr_addr_1(1),
          A2     => fifo_70_wr_addr_1(2),
          A3     => fifo_70_wr_addr_1(3),
          D      => ddr_dq_in(61),      
          DPRA0  => fifo71_rd_addr_r(0),
          DPRA1  => fifo71_rd_addr_r(1),
          DPRA2  => fifo71_rd_addr_r(2),
          DPRA3  => fifo71_rd_addr_r(3),
          WCLK   => dqs7_delayed_col0,          
          WE     => fifo_70_wr_en_1 );                       

fifo1_bit61 : RAM16X1D               
port map (DPO    => fifo_71_data_out(5),          
          A0     => fifo_71_wr_addr(0),          
          A1     => fifo_71_wr_addr(1),
          A2     => fifo_71_wr_addr(2),
          A3     => fifo_71_wr_addr(3),
          D      => ddr_dq_in(61),      
          DPRA0  => fifo71_rd_addr_r(0),
          DPRA1  => fifo71_rd_addr_r(1),
          DPRA2  => fifo71_rd_addr_r(2),
          DPRA3  => fifo71_rd_addr_r(3),
          WCLK   =>  dqs7_delayed_col0_n,          
          WE     => fifo_71_wr_en );                       



fifo0_bit62 : RAM16X1D               
port map (DPO    => fifo_70_data_out(6),          
          A0     => fifo_70_wr_addr(0),          
          A1     => fifo_70_wr_addr(1),
          A2     => fifo_70_wr_addr(2),
          A3     => fifo_70_wr_addr(3),
          D      => ddr_dq_in(62),      
          DPRA0  => fifo70_rd_addr_r(0),
          DPRA1  => fifo70_rd_addr_r(1),
          DPRA2  => fifo70_rd_addr_r(2),
          DPRA3  => fifo70_rd_addr_r(3),
          WCLK   => dqs7_delayed_col1,          
          WE     => fifo_70_wr_en );                       

fifo1_bit62 : RAM16X1D               
port map (DPO    => fifo_71_data_out(6),          
          A0     => fifo_71_wr_addr_1(0),          
          A1     => fifo_71_wr_addr_1(1),
          A2     => fifo_71_wr_addr_1(2),
          A3     => fifo_71_wr_addr_1(3),
          D      => ddr_dq_in(62),      
          DPRA0  => fifo70_rd_addr_r(0),
          DPRA1  => fifo70_rd_addr_r(1),
          DPRA2  => fifo70_rd_addr_r(2),
          DPRA3  => fifo70_rd_addr_r(3),
          WCLK   =>  dqs7_delayed_col1_n,          
          WE     => fifo_71_wr_en_1 ); 
                      
fifo0_bit63 : RAM16X1D               
port map (DPO    => fifo_70_data_out(7),          
          A0     => fifo_70_wr_addr_1(0),          
          A1     => fifo_70_wr_addr_1(1),
          A2     => fifo_70_wr_addr_1(2),
          A3     => fifo_70_wr_addr_1(3),
          D      => ddr_dq_in(63),      
          DPRA0  => fifo71_rd_addr_r(0),
          DPRA1  => fifo71_rd_addr_r(1),
          DPRA2  => fifo71_rd_addr_r(2),
          DPRA3  => fifo71_rd_addr_r(3),
          WCLK   => dqs7_delayed_col0,          
          WE     => fifo_70_wr_en_1 );                       

fifo1_bit63 : RAM16X1D               
port map (DPO    => fifo_71_data_out(7),          
          A0     => fifo_71_wr_addr(0),          
          A1     => fifo_71_wr_addr(1),
          A2     => fifo_71_wr_addr(2),
          A3     => fifo_71_wr_addr(3),
          D      => ddr_dq_in(63),      
          DPRA0  => fifo71_rd_addr_r(0),
          DPRA1  => fifo71_rd_addr_r(1),
          DPRA2  => fifo71_rd_addr_r(2),
          DPRA3  => fifo71_rd_addr_r(3),
          WCLK   =>  dqs7_delayed_col0_n,          
          WE     => fifo_71_wr_en ); 

--Byte8 Fifo Instantiation 

fifo0_bit64 : RAM16X1D               
port map (DPO    => fifo_80_data_out(0),          
          A0     => fifo_80_wr_addr(0),          
          A1     => fifo_80_wr_addr(1),
          A2     => fifo_80_wr_addr(2),
          A3     => fifo_80_wr_addr(3),
          D      => ddr_dq_in(64),      
          DPRA0  => fifo80_rd_addr_r(0),
          DPRA1  => fifo80_rd_addr_r(1),
          DPRA2  => fifo80_rd_addr_r(2),
          DPRA3  => fifo80_rd_addr_r(3),
          WCLK   => dqs8_delayed_col1,          
          WE     => fifo_80_wr_en );                       

fifo1_bit64 : RAM16X1D               
port map (DPO    => fifo_81_data_out(0),          
          A0     => fifo_81_wr_addr_1(0),          
          A1     => fifo_81_wr_addr_1(1),
          A2     => fifo_81_wr_addr_1(2),
          A3     => fifo_81_wr_addr_1(3),
          D      => ddr_dq_in(64),      
          DPRA0  => fifo80_rd_addr_r(0),
          DPRA1  => fifo80_rd_addr_r(1),
          DPRA2  => fifo80_rd_addr_r(2),
          DPRA3  => fifo80_rd_addr_r(3),
          WCLK   =>  dqs8_delayed_col1_n,          
          WE     => fifo_81_wr_en_1 ); 
                      
fifo0_bit65 : RAM16X1D               
port map (DPO    => fifo_80_data_out(1),          
          A0     => fifo_80_wr_addr_1(0),          
          A1     => fifo_80_wr_addr_1(1),
          A2     => fifo_80_wr_addr_1(2),
          A3     => fifo_80_wr_addr_1(3),
          D      => ddr_dq_in(65),      
          DPRA0  => fifo81_rd_addr_r(0),
          DPRA1  => fifo81_rd_addr_r(1),
          DPRA2  => fifo81_rd_addr_r(2),
          DPRA3  => fifo81_rd_addr_r(3),
          WCLK   => dqs8_delayed_col0,          
          WE     => fifo_80_wr_en_1 );                       

fifo1_bit65 : RAM16X1D               
port map (DPO    => fifo_81_data_out(1),          
          A0     => fifo_81_wr_addr(0),          
          A1     => fifo_81_wr_addr(1),
          A2     => fifo_81_wr_addr(2),
          A3     => fifo_81_wr_addr(3),
          D      => ddr_dq_in(65),      
          DPRA0  => fifo81_rd_addr_r(0),
          DPRA1  => fifo81_rd_addr_r(1),
          DPRA2  => fifo81_rd_addr_r(2),
          DPRA3  => fifo81_rd_addr_r(3),
          WCLK   =>  dqs8_delayed_col0_n,          
          WE     => fifo_81_wr_en );                       



fifo0_bit66 : RAM16X1D               
port map (DPO    => fifo_80_data_out(2),          
          A0     => fifo_80_wr_addr(0),          
          A1     => fifo_80_wr_addr(1),
          A2     => fifo_80_wr_addr(2),
          A3     => fifo_80_wr_addr(3),
          D      => ddr_dq_in(66),      
          DPRA0  => fifo80_rd_addr_r(0),
          DPRA1  => fifo80_rd_addr_r(1),
          DPRA2  => fifo80_rd_addr_r(2),
          DPRA3  => fifo80_rd_addr_r(3),
          WCLK   => dqs8_delayed_col1,          
          WE     => fifo_80_wr_en );                       

fifo1_bit66 : RAM16X1D               
port map (DPO    => fifo_81_data_out(2),          
          A0     => fifo_81_wr_addr_1(0),          
          A1     => fifo_81_wr_addr_1(1),
          A2     => fifo_81_wr_addr_1(2),
          A3     => fifo_81_wr_addr_1(3),
          D      => ddr_dq_in(66),      
          DPRA0  => fifo80_rd_addr_r(0),
          DPRA1  => fifo80_rd_addr_r(1),
          DPRA2  => fifo80_rd_addr_r(2),
          DPRA3  => fifo80_rd_addr_r(3),
          WCLK   =>  dqs8_delayed_col1_n,          
          WE     => fifo_81_wr_en_1 ); 
                      
fifo0_bit67 : RAM16X1D               
port map (DPO    => fifo_80_data_out(3),          
          A0     => fifo_80_wr_addr_1(0),          
          A1     => fifo_80_wr_addr_1(1),
          A2     => fifo_80_wr_addr_1(2),
          A3     => fifo_80_wr_addr_1(3),
          D      => ddr_dq_in(67),      
          DPRA0  => fifo81_rd_addr_r(0),
          DPRA1  => fifo81_rd_addr_r(1),
          DPRA2  => fifo81_rd_addr_r(2),
          DPRA3  => fifo81_rd_addr_r(3),
          WCLK   => dqs8_delayed_col0,          
          WE     => fifo_80_wr_en_1 );                       

fifo1_bit67 : RAM16X1D               
port map (DPO    => fifo_81_data_out(3),          
          A0     => fifo_81_wr_addr(0),          
          A1     => fifo_81_wr_addr(1),
          A2     => fifo_81_wr_addr(2),
          A3     => fifo_81_wr_addr(3),
          D      => ddr_dq_in(67),      
          DPRA0  => fifo81_rd_addr_r(0),
          DPRA1  => fifo81_rd_addr_r(1),
          DPRA2  => fifo81_rd_addr_r(2),
          DPRA3  => fifo81_rd_addr_r(3),
          WCLK   =>  dqs8_delayed_col0_n,          
          WE     => fifo_81_wr_en ); 

                      
fifo0_bit68 : RAM16X1D               
port map (DPO    => fifo_80_data_out(4),          
          A0     => fifo_80_wr_addr(0),          
          A1     => fifo_80_wr_addr(1),
          A2     => fifo_80_wr_addr(2),
          A3     => fifo_80_wr_addr(3),
          D      => ddr_dq_in(68),      
          DPRA0  => fifo80_rd_addr_r(0),
          DPRA1  => fifo80_rd_addr_r(1),
          DPRA2  => fifo80_rd_addr_r(2),
          DPRA3  => fifo80_rd_addr_r(3),
          WCLK   => dqs8_delayed_col1,          
          WE     => fifo_80_wr_en );                       

fifo1_bit68 : RAM16X1D               
port map (DPO    => fifo_81_data_out(4),          
          A0     => fifo_81_wr_addr_1(0),          
          A1     => fifo_81_wr_addr_1(1),
          A2     => fifo_81_wr_addr_1(2),
          A3     => fifo_81_wr_addr_1(3),
          D      => ddr_dq_in(68),      
          DPRA0  => fifo80_rd_addr_r(0),
          DPRA1  => fifo80_rd_addr_r(1),
          DPRA2  => fifo80_rd_addr_r(2),
          DPRA3  => fifo80_rd_addr_r(3),
          WCLK   =>  dqs8_delayed_col1_n,          
          WE     => fifo_81_wr_en_1 ); 
                      
fifo0_bit69 : RAM16X1D               
port map (DPO    => fifo_80_data_out(5),          
          A0     => fifo_80_wr_addr_1(0),          
          A1     => fifo_80_wr_addr_1(1),
          A2     => fifo_80_wr_addr_1(2),
          A3     => fifo_80_wr_addr_1(3),
          D      => ddr_dq_in(69),      
          DPRA0  => fifo81_rd_addr_r(0),
          DPRA1  => fifo81_rd_addr_r(1),
          DPRA2  => fifo81_rd_addr_r(2),
          DPRA3  => fifo81_rd_addr_r(3),
          WCLK   => dqs8_delayed_col0,          
          WE     => fifo_80_wr_en_1 );                       

fifo1_bit69 : RAM16X1D               
port map (DPO    => fifo_81_data_out(5),          
          A0     => fifo_81_wr_addr(0),          
          A1     => fifo_81_wr_addr(1),
          A2     => fifo_81_wr_addr(2),
          A3     => fifo_81_wr_addr(3),
          D      => ddr_dq_in(69),      
          DPRA0  => fifo81_rd_addr_r(0),
          DPRA1  => fifo81_rd_addr_r(1),
          DPRA2  => fifo81_rd_addr_r(2),
          DPRA3  => fifo81_rd_addr_r(3),
          WCLK   =>  dqs8_delayed_col0_n,          
          WE     => fifo_81_wr_en );                       


fifo0_bit70 : RAM16X1D               
port map (DPO    => fifo_80_data_out(6),          
          A0     => fifo_80_wr_addr(0),          
          A1     => fifo_80_wr_addr(1),
          A2     => fifo_80_wr_addr(2),
          A3     => fifo_80_wr_addr(3),
          D      => ddr_dq_in(70),      
          DPRA0  => fifo80_rd_addr_r(0),
          DPRA1  => fifo80_rd_addr_r(1),
          DPRA2  => fifo80_rd_addr_r(2),
          DPRA3  => fifo80_rd_addr_r(3),
          WCLK   => dqs8_delayed_col1,          
          WE     => fifo_80_wr_en );                       

fifo1_bit70 : RAM16X1D               
port map (DPO    => fifo_81_data_out(6),          
          A0     => fifo_81_wr_addr_1(0),          
          A1     => fifo_81_wr_addr_1(1),
          A2     => fifo_81_wr_addr_1(2),
          A3     => fifo_81_wr_addr_1(3),
          D      => ddr_dq_in(70),      
          DPRA0  => fifo80_rd_addr_r(0),
          DPRA1  => fifo80_rd_addr_r(1),
          DPRA2  => fifo80_rd_addr_r(2),
          DPRA3  => fifo80_rd_addr_r(3),
          WCLK   =>  dqs8_delayed_col1_n,          
          WE     => fifo_81_wr_en_1 ); 
                      
fifo0_bit71 : RAM16X1D               
port map (DPO    => fifo_80_data_out(7),          
          A0     => fifo_80_wr_addr_1(0),          
          A1     => fifo_80_wr_addr_1(1),
          A2     => fifo_80_wr_addr_1(2),
          A3     => fifo_80_wr_addr_1(3),
          D      => ddr_dq_in(71),      
          DPRA0  => fifo81_rd_addr_r(0),
          DPRA1  => fifo81_rd_addr_r(1),
          DPRA2  => fifo81_rd_addr_r(2),
          DPRA3  => fifo81_rd_addr_r(3),
          WCLK   => dqs8_delayed_col0,          
          WE     => fifo_80_wr_en_1 );                       

fifo1_bit71 : RAM16X1D               
port map (DPO    => fifo_81_data_out(7),          
          A0     => fifo_81_wr_addr(0),          
          A1     => fifo_81_wr_addr(1),
          A2     => fifo_81_wr_addr(2),
          A3     => fifo_81_wr_addr(3),
          D      => ddr_dq_in(71),      
          DPRA0  => fifo81_rd_addr_r(0),
          DPRA1  => fifo81_rd_addr_r(1),
          DPRA2  => fifo81_rd_addr_r(2),
          DPRA3  => fifo81_rd_addr_r(3),
          WCLK   =>  dqs8_delayed_col0_n,          
          WE     => fifo_81_wr_en ); 


         
end arc_data_read;
