--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--
--*******************************************************************************
--
--  File name :       data_path.vhd
--
--  Description :     This module comprises the write and read data paths for the
--                    DDR1 memory interface. The write data along with write enable 
--                    signals are forwarded to the DDR IOB FFs. The read data is 
--                    captured in CLB FFs and finally input to FIFOs.
-- 
--                    
--  Date - revision : 10/16/2003
--
--  Author :          Maria George
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--library synplify; 
--use synplify.attributes.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--
entity data_path is
port(
     user_input_data    : in std_logic_vector(63 downto 0);
     clk                : in std_logic;
     clk90              : in std_logic;
     reset              : in std_logic;
     reset90            : in std_logic;
     reset180           : in std_logic;
     reset270           : in std_logic;
     write_enable       : in std_logic;
     rst_dqs_div_in     : in std_logic;
     rst_dqs_div_in2    : in std_logic;
     delay_sel          : in std_logic_vector(4 downto 0);   
     dqs_int_delay_in0  : in std_logic;
     dqs_int_delay_in1  : in std_logic;
     dqs_int_delay_in2  : in std_logic;
     dqs_int_delay_in3  : in std_logic;
     dq                 : in std_logic_vector(31 downto 0);       
     u_data_val         : out std_logic;
     user_output_data   : out std_logic_vector(63 downto 0);
     write_en_val       : out std_logic;
     write_en_val1      : out std_logic;
     reset90_r_val      : out std_logic;
     data_mask_f        : out std_logic_vector(3 downto 0);
     data_mask_r        : out std_logic_vector(3 downto 0);
     write_data_falling : out std_logic_vector(31 downto 0);
     write_data_rising  : out std_logic_vector(31 downto 0)
     );
end data_path;

architecture arc_data_path of data_path is

component data_read 
port(

     clk90              : in std_logic;
     reset90_r          : in std_logic;
     ddr_dq_in          : in std_logic_vector(31 downto 0);   
     read_valid_data_1  : in std_logic;  
     fifo_00_wr_en       : in std_logic;
     fifo_10_wr_en       : in std_logic;
     fifo_20_wr_en       : in std_logic;
     fifo_30_wr_en       : in std_logic;               
     fifo_01_wr_en       : in std_logic;
     fifo_11_wr_en       : in std_logic;
     fifo_21_wr_en       : in std_logic;
     fifo_31_wr_en       : in std_logic;               
     fifo_00_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_01_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_10_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_11_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_20_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_21_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_30_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_31_wr_addr    : in std_logic_vector(3 downto 0);
     fifo_00_wr_en_1       : in std_logic;
     fifo_10_wr_en_1       : in std_logic;
     fifo_20_wr_en_1       : in std_logic;
     fifo_30_wr_en_1       : in std_logic;          
     fifo_01_wr_en_1       : in std_logic;
     fifo_11_wr_en_1       : in std_logic;
     fifo_21_wr_en_1       : in std_logic;
     fifo_31_wr_en_1       : in std_logic;               
     fifo_00_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_01_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_10_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_11_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_20_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_21_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_30_wr_addr_1    : in std_logic_vector(3 downto 0);
     fifo_31_wr_addr_1    : in std_logic_vector(3 downto 0);		
     dqs0_delayed_col1  : in std_logic;
     dqs1_delayed_col1  : in std_logic;
     dqs2_delayed_col1  : in std_logic;
     dqs3_delayed_col1  : in std_logic;
     dqs0_delayed_col0  : in std_logic;
     dqs1_delayed_col0  : in std_logic;
     dqs2_delayed_col0  : in std_logic;
     dqs3_delayed_col0  : in std_logic;
     user_output_data   : out std_logic_vector(63 downto 0);
     fifo0_rd_addr_val: out std_logic_vector(3 downto 0);
     fifo1_rd_addr_val: out std_logic_vector(3 downto 0)     

     );
end component;


component data_read_controller 
port(

     clk90              : in std_logic;
     reset_r            : in std_logic;
     reset90_r          : in std_logic;
     rst_dqs_div_in     : in std_logic;
     rst_dqs_div_in2    : in std_logic; 
     delay_sel          : in std_logic_vector(4 downto 0);   
     dqs_int_delay_in0  : in std_logic;
     dqs_int_delay_in1  : in std_logic;
     dqs_int_delay_in2  : in std_logic;
     dqs_int_delay_in3  : in std_logic;
     fifo0_rd_addr      : in std_logic_vector(3 downto 0);
     fifo1_rd_addr      : in std_logic_vector(3 downto 0);
     u_data_val   	: out std_logic;   
     read_valid_data_1_val  : out std_logic;     
     fifo_00_wr_en_val	: out std_logic;	
     fifo_10_wr_en_val			: out std_logic;	
     fifo_20_wr_en_val			: out std_logic;	
     fifo_30_wr_en_val			: out std_logic;
     fifo_01_wr_en_val			: out std_logic;	
     fifo_11_wr_en_val			: out std_logic;	
     fifo_21_wr_en_val			: out std_logic;	
     fifo_31_wr_en_val			: out std_logic;
     fifo_00_wr_addr_val    : out std_logic_vector(3 downto 0);
     fifo_01_wr_addr_val    : out std_logic_vector(3 downto 0);
     fifo_10_wr_addr_val    : out std_logic_vector(3 downto 0);
     fifo_11_wr_addr_val    : out std_logic_vector(3 downto 0);
     fifo_20_wr_addr_val    : out std_logic_vector(3 downto 0);
     fifo_21_wr_addr_val    : out std_logic_vector(3 downto 0);
     fifo_30_wr_addr_val    : out std_logic_vector(3 downto 0);
     fifo_31_wr_addr_val    : out std_logic_vector(3 downto 0);
     fifo_00_wr_en_val_1	             : out std_logic;	
     fifo_10_wr_en_val_1			: out std_logic;	
     fifo_20_wr_en_val_1			: out std_logic;	
     fifo_30_wr_en_val_1			: out std_logic;
     fifo_01_wr_en_val_1			: out std_logic;	
     fifo_11_wr_en_val_1			: out std_logic;	
     fifo_21_wr_en_val_1			: out std_logic;	
     fifo_31_wr_en_val_1			: out std_logic;
     fifo_00_wr_addr_val_1    : out std_logic_vector(3 downto 0);
     fifo_01_wr_addr_val_1    : out std_logic_vector(3 downto 0);
     fifo_10_wr_addr_val_1    : out std_logic_vector(3 downto 0);
     fifo_11_wr_addr_val_1    : out std_logic_vector(3 downto 0);
     fifo_20_wr_addr_val_1    : out std_logic_vector(3 downto 0);
     fifo_21_wr_addr_val_1    : out std_logic_vector(3 downto 0);
     fifo_30_wr_addr_val_1    : out std_logic_vector(3 downto 0);
     fifo_31_wr_addr_val_1    : out std_logic_vector(3 downto 0);
     dqs0_delayed_col0_val  : out std_logic;
     dqs1_delayed_col0_val  : out std_logic;
     dqs2_delayed_col0_val  : out std_logic;
     dqs3_delayed_col0_val  : out std_logic;
     dqs0_delayed_col1_val  : out std_logic;
     dqs1_delayed_col1_val  : out std_logic;
     dqs2_delayed_col1_val  : out std_logic;
     dqs3_delayed_col1_val  : out std_logic


      );

end component;



component data_write
port(
     user_input_data    : in std_logic_vector(63 downto 0);
     clk90              : in std_logic;
     reset90_r          : in std_logic;
     reset270_r         : in std_logic;
     write_enable       : in std_logic;
     write_en_val       : out std_logic;
     write_en_val1      : out std_logic;
     write_data_falling : out std_logic_vector(31 downto 0);
     write_data_rising  : out std_logic_vector(31 downto 0);
     data_mask_f        : out std_logic_vector(3 downto 0);
     data_mask_r        : out std_logic_vector(3 downto 0)
     );
end component;   

component data_path_rst
port(
     clk                : in std_logic;
     clk90              : in std_logic;
     reset              : in std_logic;
     reset90            : in std_logic;
     reset180           : in std_logic;
     reset270           : in std_logic;
     reset_r            : out std_logic;
     reset90_r          : out std_logic;
     reset90_r1         : out std_logic; 
     reset180_r         : out std_logic;
     reset270_r         : out std_logic
    );
end component;

signal reset_r          : std_logic;
signal reset90_r        : std_logic;
signal reset90_r1       : std_logic;
signal reset180_r       : std_logic;
signal reset270_r       : std_logic;                                                                          
 signal fifo0_rd_addr    : std_logic_vector(3 downto 0);
 signal fifo1_rd_addr    : std_logic_vector(3 downto 0);                          
 signal read_valid_data_1  : std_logic; 
 signal fifo_00_wr_addr    : std_logic_vector(3 downto 0);           
 signal fifo_01_wr_addr    : std_logic_vector(3 downto 0);           
 signal fifo_10_wr_addr    : std_logic_vector(3 downto 0);          
 signal fifo_11_wr_addr    : std_logic_vector(3 downto 0);             
 signal fifo_20_wr_addr    : std_logic_vector(3 downto 0);           
 signal fifo_21_wr_addr    : std_logic_vector(3 downto 0);           
 signal fifo_30_wr_addr    : std_logic_vector(3 downto 0);           
 signal fifo_31_wr_addr    : std_logic_vector(3 downto 0);                    
  signal fifo_00_wr_en	                :  std_logic;
 signal fifo_10_wr_en                   :  std_logic;
 signal fifo_20_wr_en			:  std_logic;
 signal fifo_30_wr_en			:  std_logic;
 signal fifo_01_wr_en	  	        :  std_logic;
 signal fifo_11_wr_en                   :  std_logic;
 signal fifo_21_wr_en			:  std_logic;
 signal fifo_31_wr_en			:  std_logic;
 signal fifo_00_wr_addr_1    : std_logic_vector(3 downto 0);           
 signal fifo_01_wr_addr_1    : std_logic_vector(3 downto 0);           
 signal fifo_10_wr_addr_1    : std_logic_vector(3 downto 0);          
 signal fifo_11_wr_addr_1    : std_logic_vector(3 downto 0);             
 signal fifo_20_wr_addr_1    : std_logic_vector(3 downto 0);           
 signal fifo_21_wr_addr_1    : std_logic_vector(3 downto 0);           
 signal fifo_30_wr_addr_1    : std_logic_vector(3 downto 0);           
 signal fifo_31_wr_addr_1    : std_logic_vector(3 downto 0);           
 signal fifo_00_wr_en_1	                :  std_logic;
 signal fifo_10_wr_en_1                   :  std_logic;
 signal fifo_20_wr_en_1			:  std_logic;
 signal fifo_30_wr_en_1			:  std_logic;
 signal fifo_40_wr_en_1			:  std_logic;
 signal fifo_01_wr_en_1	  	        :  std_logic;
 signal fifo_11_wr_en_1                   :  std_logic;
 signal fifo_21_wr_en_1			:  std_logic;
 signal fifo_31_wr_en_1			:  std_logic;
 signal dqs0_delayed_col1  :  std_logic;
 signal dqs1_delayed_col1  :  std_logic;
 signal dqs2_delayed_col1  :  std_logic;
 signal dqs3_delayed_col1  :  std_logic;
 signal dqs0_delayed_col0  :  std_logic;
 signal dqs1_delayed_col0  :  std_logic;
 signal dqs2_delayed_col0  :  std_logic;
 signal dqs3_delayed_col0  :  std_logic;


 begin

reset90_r_val <= reset90_r;

data_read0 : data_read
port map (
           clk90               =>    clk90,
           reset90_r           =>    reset90_r1,
	   ddr_dq_in           =>    dq,
           read_valid_data_1   =>    read_valid_data_1,
	   fifo_00_wr_en		     => fifo_00_wr_en,
	   fifo_10_wr_en		     => fifo_10_wr_en,
	   fifo_20_wr_en		     => fifo_20_wr_en,
	   fifo_30_wr_en		     => fifo_30_wr_en,
	   fifo_01_wr_en		     => fifo_01_wr_en,
	   fifo_11_wr_en		     => fifo_11_wr_en,
	   fifo_21_wr_en		     => fifo_21_wr_en,
	   fifo_31_wr_en		     => fifo_31_wr_en,          
           fifo_00_wr_addr     =>    fifo_00_wr_addr,
           fifo_01_wr_addr     =>    fifo_01_wr_addr,
           fifo_10_wr_addr     =>    fifo_10_wr_addr,
           fifo_11_wr_addr     =>    fifo_11_wr_addr,
           fifo_20_wr_addr     =>    fifo_20_wr_addr,
           fifo_21_wr_addr     =>    fifo_21_wr_addr,
           fifo_30_wr_addr     =>    fifo_30_wr_addr,
           fifo_31_wr_addr     =>    fifo_31_wr_addr,
	   fifo_00_wr_en_1		     => fifo_00_wr_en_1,
	   fifo_10_wr_en_1		     => fifo_10_wr_en_1,
	   fifo_20_wr_en_1		     => fifo_20_wr_en_1,
	   fifo_30_wr_en_1		     => fifo_30_wr_en_1,
	   fifo_01_wr_en_1		     => fifo_01_wr_en_1,
	   fifo_11_wr_en_1		     => fifo_11_wr_en_1,
	   fifo_21_wr_en_1		     => fifo_21_wr_en_1,
	   fifo_31_wr_en_1		     => fifo_31_wr_en_1,
           fifo_00_wr_addr_1     =>    fifo_00_wr_addr_1,
           fifo_01_wr_addr_1     =>    fifo_01_wr_addr_1,
           fifo_10_wr_addr_1     =>    fifo_10_wr_addr_1,
           fifo_11_wr_addr_1     =>    fifo_11_wr_addr_1,
           fifo_20_wr_addr_1     =>    fifo_20_wr_addr_1,
           fifo_21_wr_addr_1     =>    fifo_21_wr_addr_1,
           fifo_30_wr_addr_1     =>    fifo_30_wr_addr_1,
           fifo_31_wr_addr_1     =>    fifo_31_wr_addr_1,
           dqs0_delayed_col1   =>    dqs0_delayed_col1,
	   dqs1_delayed_col1   =>    dqs1_delayed_col1,
           dqs2_delayed_col1   =>    dqs2_delayed_col1,
           dqs3_delayed_col1   =>    dqs3_delayed_col1,
           dqs0_delayed_col0   =>    dqs0_delayed_col0,
           dqs1_delayed_col0   =>    dqs1_delayed_col0,
           dqs2_delayed_col0   =>    dqs2_delayed_col0,
	   dqs3_delayed_col0   =>    dqs3_delayed_col0,          
           user_output_data    =>    user_output_data,
           fifo0_rd_addr_val   =>    fifo0_rd_addr,
           fifo1_rd_addr_val   =>    fifo1_rd_addr
         );



data_read_controller0 : data_read_controller
port map (
            clk90              =>   clk90,
            reset_r            =>   reset_r,
            reset90_r          =>   reset90_r1,
            rst_dqs_div_in     =>   rst_dqs_div_in,
            rst_dqs_div_in2    =>   rst_dqs_div_in2, 
            delay_sel          =>   delay_sel,
            dqs_int_delay_in0  =>   dqs_int_delay_in0,
            dqs_int_delay_in1  =>   dqs_int_delay_in1,
            dqs_int_delay_in2  =>   dqs_int_delay_in2,
            dqs_int_delay_in3  =>   dqs_int_delay_in3,
            fifo0_rd_addr    =>   fifo0_rd_addr,
            fifo1_rd_addr    =>   fifo1_rd_addr,
            u_data_val   						    => u_data_val,
            read_valid_data_1_val  =>   read_valid_data_1,
            fifo_00_wr_en_val				    => fifo_00_wr_en,
            fifo_10_wr_en_val				    => fifo_10_wr_en,            
            fifo_20_wr_en_val				    => fifo_20_wr_en,            
            fifo_30_wr_en_val				    => fifo_30_wr_en,
            fifo_01_wr_en_val				    => fifo_01_wr_en,
            fifo_11_wr_en_val				    => fifo_11_wr_en,            
            fifo_21_wr_en_val				    => fifo_21_wr_en,            
            fifo_31_wr_en_val				    => fifo_31_wr_en,
            fifo_00_wr_addr_val     =>    fifo_00_wr_addr,
            fifo_01_wr_addr_val     =>    fifo_01_wr_addr,
            fifo_10_wr_addr_val     =>    fifo_10_wr_addr,
            fifo_11_wr_addr_val     =>    fifo_11_wr_addr,
            fifo_20_wr_addr_val     =>    fifo_20_wr_addr,
            fifo_21_wr_addr_val     =>    fifo_21_wr_addr,
            fifo_30_wr_addr_val     =>    fifo_30_wr_addr,
            fifo_31_wr_addr_val     =>    fifo_31_wr_addr,
            fifo_00_wr_en_val_1				    => fifo_00_wr_en_1,
            fifo_10_wr_en_val_1				    => fifo_10_wr_en_1,            
            fifo_20_wr_en_val_1				    => fifo_20_wr_en_1,            
            fifo_30_wr_en_val_1				    => fifo_30_wr_en_1,                                 
            fifo_01_wr_en_val_1				    => fifo_01_wr_en_1,
            fifo_11_wr_en_val_1				    => fifo_11_wr_en_1,            
            fifo_21_wr_en_val_1				    => fifo_21_wr_en_1,            
            fifo_31_wr_en_val_1				    => fifo_31_wr_en_1,
            fifo_00_wr_addr_val_1     =>    fifo_00_wr_addr_1,
            fifo_01_wr_addr_val_1     =>    fifo_01_wr_addr_1,
            fifo_10_wr_addr_val_1     =>    fifo_10_wr_addr_1,
            fifo_11_wr_addr_val_1     =>    fifo_11_wr_addr_1,
            fifo_20_wr_addr_val_1     =>    fifo_20_wr_addr_1,
            fifo_21_wr_addr_val_1     =>    fifo_21_wr_addr_1,
            fifo_30_wr_addr_val_1     =>    fifo_30_wr_addr_1,
            fifo_31_wr_addr_val_1     =>    fifo_31_wr_addr_1,
	    dqs0_delayed_col0_val     => dqs0_delayed_col0,
	    dqs1_delayed_col0_val     => dqs1_delayed_col0,
	    dqs2_delayed_col0_val     => dqs2_delayed_col0,
	    dqs3_delayed_col0_val     => dqs3_delayed_col0,
	    dqs0_delayed_col1_val     => dqs0_delayed_col1,
	    dqs1_delayed_col1_val     => dqs1_delayed_col1,
	    dqs2_delayed_col1_val     => dqs2_delayed_col1,
	    dqs3_delayed_col1_val     => dqs3_delayed_col1
        
         );

         
data_write0 : data_write
port map (
          user_input_data    =>   user_input_data,
          clk90              =>   clk90,
          reset90_r          =>   reset90_r1,
          reset270_r         =>   reset270_r,
          write_enable       =>   write_enable,
          write_en_val       =>   write_en_val,
          write_en_val1      =>   write_en_val1,
          write_data_falling =>   write_data_falling,
          write_data_rising  =>   write_data_rising,
          data_mask_f        =>   data_mask_f,
          data_mask_r        =>   data_mask_r
         );

data_path_rst0 : data_path_rst 
port map (
          clk                =>   clk,            
          clk90              =>   clk90,
          reset              =>   reset,
          reset90            =>   reset90,
          reset180           =>   reset180,
          reset270           =>   reset270,
          reset_r            =>   reset_r,
          reset90_r          =>   reset90_r,
          reset90_r1          =>   reset90_r1,
          reset180_r         =>   reset180_r,
          reset270_r         =>   reset270_r
         );
                                                                       
         
end arc_data_path;












