--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 253 - Synthesizable DDR SDRAM Controller
--
--*******************************************************************************
--
--  File name :       controller.vhd
--
--  Description :     
--                    Main DDR SDRAM controller block. This includes the following
--                    features:
--                    - The controller state machine that controls the 
--                    initialization process upon power up, as well as the 
--                    read, write, and refresh commands. 
--                    - Accepts and decodes the user commands.
--                    - Generates the address and Bank address signals
--                    - Generates control signals for other modules, including
--                    the control signals for the dqs_en block.
--
--  Date - revision : 05/01/2002
--
--  Author :          Lakshmi Gopalakrishnan
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--
entity iobs is
port(
     SYS_CLK           : in std_logic;
     SYS_CLKb          : in std_logic;
     clk               : in std_logic;
     clk90             : in std_logic;  
     JP1_header_infra  : in std_logic_vector(7 downto 0);
     JP2_header_infra  : in std_logic_vector(7 downto 0); 
     ddr_rasb_cntrl    : in std_logic;
     ddr_casb_cntrl    : in std_logic;
     ddr_web_cntrl     : in std_logic;
     ddr_cke_cntrl     : in std_logic;
     ddr_csb_cntrl     : in std_logic;
     ddr_address_cntrl : in std_logic_vector(12 downto 0);
     ddr_ba_cntrl      : in std_logic_vector(1 downto 0);
     rst_dqs_div_int   : in std_logic;
     rst_dqs_div_int1  : in std_logic;
     dqs_reset         : in std_logic;
     dqs_enable        : in std_logic;
     ddr_dqs           : inout std_logic_vector(3 downto 0);
     ddr_dq            : inout std_logic_vector(31 downto 0);
     write_data_falling: in std_logic_vector(31 downto 0);
     write_data_rising : in std_logic_vector(31 downto 0);
     write_en_val      : in std_logic;
     write_en_val1      : in std_logic;
     reset90_r        : in std_logic;
     data_mask_f       : in std_logic_vector(3 downto 0);
     data_mask_r       : in std_logic_vector(3 downto 0);
     sys_clk_ibuf      : out std_logic;
     sys_clkb_ibuf     : out std_logic;
     ddr1_clk0         : out std_logic;
     ddr1_clk0b        : out std_logic;
     ddr1_clk1         : out std_logic;
     ddr1_clk1b        : out std_logic;
     ddr1_clk2         : out std_logic;
     ddr1_clk2b        : out std_logic;
     ddr1_clk3         : out std_logic;
     ddr1_clk3b        : out std_logic;
     ddr1_clk4         : out std_logic;
     ddr1_clk4b        : out std_logic;
     dimm_clk0         : out std_logic;
     dimm_clk0b        : out std_logic;
     dimm_clk1         : out std_logic;
     dimm_clk1b        : out std_logic;
     dimm_clk2         : out std_logic;
     dimm_clk2b        : out std_logic; 
     JP1_header        : out std_logic_vector(7 downto 0);
     JP2_header        : out std_logic_vector(7 downto 0);
     ddr_rasb          : out std_logic;
     ddr_casb          : out std_logic;
     ddr_web           : out std_logic;
     ddr_ba            : out std_logic_vector(1 downto 0);
     ddr_address       : out std_logic_vector(12 downto 0);
     ddr_cke           : out std_logic;
     ddr_csb           : out std_logic;
     rst_dqs_div       : out std_logic;
     rst_dqs_div2      : out std_logic; 
     --rst_dqs_div_iob   : inout std_logic;
     rst_dqs_div_in    : in std_logic;--changed by shyam on 13 march
     rst_dqs_div_in2   : in std_logic;
     rst_dqs_div_out   : out std_logic;--changed by shyam on 13 march
     rst_dqs_div_out2   : out std_logic;  
     dqs_int_delay_in0 : out std_logic;
     dqs_int_delay_in1 : out std_logic;
     dqs_int_delay_in2 : out std_logic;
     dqs_int_delay_in3 : out std_logic;

     dq                : out std_logic_vector(31 downto 0);
     ddr_dm            : out std_logic_vector(3 downto 0)              
);
end iobs;


architecture arc_iobs of iobs is

component infrastructure_iobs
port(
     SYS_CLK           : in std_logic;
     SYS_CLKb          : in std_logic;
     clk0              : in std_logic;
     clk90             : in std_logic;
     JP1_header_infra  : in std_logic_vector(7 downto 0);
     JP2_header_infra  : in std_logic_vector(7 downto 0);    
     sys_clk_ibuf      : out std_logic;
     sys_clkb_ibuf     : out std_logic;
     ddr1_clk0         : out std_logic;
     ddr1_clk0b        : out std_logic;
     ddr1_clk1         : out std_logic;
     ddr1_clk1b        : out std_logic;
     ddr1_clk2         : out std_logic;
     ddr1_clk2b        : out std_logic;
     ddr1_clk3         : out std_logic;
     ddr1_clk3b        : out std_logic;
     ddr1_clk4         : out std_logic;
     ddr1_clk4b        : out std_logic;
     dimm_clk0         : out std_logic;
     dimm_clk0b        : out std_logic;
     dimm_clk1         : out std_logic;
     dimm_clk1b        : out std_logic;
     dimm_clk2         : out std_logic;
     dimm_clk2b        : out std_logic; 
     JP1_header        : out std_logic_vector(7 downto 0);
     JP2_header        : out std_logic_vector(7 downto 0)      
     );
end component;


component controller_iobs 
port(
     clk0             : in std_logic;
     ddr_rasb_cntrl   : in std_logic;
     ddr_casb_cntrl   : in std_logic;
     ddr_web_cntrl    : in std_logic;
     ddr_cke_cntrl    : in std_logic;
     ddr_csb_cntrl    : in std_logic;
     ddr_address_cntrl: in std_logic_vector(12 downto 0);
     ddr_ba_cntrl     : in std_logic_vector(1 downto 0);
     rst_dqs_div_int  : in std_logic;
     rst_dqs_div_int1  : in std_logic;
     ddr_rasb         : out std_logic;
     ddr_casb         : out std_logic;
     ddr_web          : out std_logic;
     ddr_ba           : out std_logic_vector(1 downto 0);
     ddr_address      : out std_logic_vector(12 downto 0);
     ddr_cke          : out std_logic;
     ddr_csb          : out std_logic;
     rst_dqs_div      : out std_logic;
     rst_dqs_div2     : out std_logic;
     rst_dqs_div_in   : in std_logic;
     rst_dqs_div_in2  : in std_logic;
     rst_dqs_div_out2 : out std_logic; 
     rst_dqs_div_out  : out std_logic
    );
end component;

component datapath_iobs 
port(
    clk               : in std_logic;
	  clk90             : in std_logic;
    reset90_r         : in std_logic;
	  dqs_reset         : in std_logic;
    dqs_enable        : in std_logic;
    ddr_dqs           : inout std_logic_vector(3 downto 0);
    ddr_dq            : inout std_logic_vector(31 downto 0);
	  write_data_falling: in std_logic_vector(31 downto 0);
    write_data_rising : in std_logic_vector(31 downto 0);
    write_en_val      : in std_logic;
    write_en_val1     : in std_logic;
    data_mask_f       : in std_logic_vector(3 downto 0);
    data_mask_r       : in std_logic_vector(3 downto 0);
    dqs_int_delay_in0 : out std_logic;
    dqs_int_delay_in1 : out std_logic;
    dqs_int_delay_in2 : out std_logic;
    dqs_int_delay_in3 : out std_logic;
    ddr_dq_val         : out std_logic_vector(31 downto 0);
    ddr_dm            : out std_logic_vector(3 downto 0)  
);
end component;






begin

  

infrastructure_iobs0 : infrastructure_iobs port map (
                                                     SYS_CLK          => SYS_CLK,
                                                     SYS_CLKb         => SYS_CLKb,
                                                     clk0             => clk,
                                                     clk90            => clk90,
                                                     JP1_header_infra => JP1_header_infra(7 downto 0),
                                                     JP2_header_infra => JP2_header_infra(7 downto 0),
                                                     sys_clk_ibuf     => sys_clk_ibuf,
                                                     sys_clkb_ibuf    => sys_clkb_ibuf,
                                                     ddr1_clk0        => ddr1_clk0,
                                                     ddr1_clk0b       => ddr1_clk0b,
                                                     ddr1_clk1        => ddr1_clk1,
                                                     ddr1_clk1b       => ddr1_clk1b,
                                                     ddr1_clk2        => ddr1_clk2,
                                                     ddr1_clk2b       => ddr1_clk2b,
                                                     ddr1_clk3        => ddr1_clk3,
                                                     ddr1_clk3b       => ddr1_clk3b,
                                                     ddr1_clk4        => ddr1_clk4,
                                                     ddr1_clk4b       => ddr1_clk4b,
                                                     dimm_clk0        => dimm_clk0,
                                                     dimm_clk0b       => dimm_clk0b,
                                                     dimm_clk1        => dimm_clk1,
                                                     dimm_clk1b       => dimm_clk1b,
                                                     dimm_clk2        => dimm_clk2,
                                                     dimm_clk2b       => dimm_clk2b,
                                                     JP1_header       => JP1_header(7 downto 0),
                                                     JP2_header       => JP2_header(7 downto 0)
                                                    );

controller_iobs0 : controller_iobs port map (
                                             clk0              =>  clk,
                                             ddr_rasb_cntrl    =>  ddr_rasb_cntrl,
                                             ddr_casb_cntrl    =>  ddr_casb_cntrl,
                                             ddr_web_cntrl     =>  ddr_web_cntrl, 
                                             ddr_cke_cntrl     =>  ddr_cke_cntrl,
                                             ddr_csb_cntrl     =>  ddr_csb_cntrl,
                                             ddr_address_cntrl =>  ddr_address_cntrl(12 downto 0),
                                             ddr_ba_cntrl      =>  ddr_ba_cntrl(1 downto 0),
                                             rst_dqs_div_int   =>  rst_dqs_div_int,
                                             rst_dqs_div_int1   =>  rst_dqs_div_int1,
                                             ddr_rasb          =>  ddr_rasb,
                                             ddr_casb          =>  ddr_casb,
                                             ddr_web           =>  ddr_web,
                                             ddr_ba            =>  ddr_ba(1 downto 0),
                                             ddr_address       =>  ddr_address(12 downto 0),
                                             ddr_cke           =>  ddr_cke,
                                             ddr_csb           =>  ddr_csb, 
                                             rst_dqs_div       =>  rst_dqs_div, 
                                             rst_dqs_div2      =>  rst_dqs_div2,
                                             rst_dqs_div_in    => rst_dqs_div_in,
                                             rst_dqs_div_in2   => rst_dqs_div_in2,
                                             rst_dqs_div_out2  => rst_dqs_div_out2,   
                                             rst_dqs_div_out   => rst_dqs_div_out	                                            );

datapath_iobs0 : datapath_iobs port map (
                                         clk                =>   clk,
																	 clk90              =>   clk90,
																	 reset90_r         =>   reset90_r,
                                         dqs_reset          =>   dqs_reset,
                                         dqs_enable         =>   dqs_enable,
                                         ddr_dqs            =>   ddr_dqs(3 downto 0),
                                         ddr_dq             =>   ddr_dq(31 downto 0),
                                         write_data_falling =>   write_data_falling(31 downto 0),
                                         write_data_rising  =>   write_data_rising(31 downto 0),
                                         write_en_val       =>   write_en_val,
                                         write_en_val1      =>   write_en_val1,
                                         data_mask_f        =>   data_mask_f(3 downto 0),
                                         data_mask_r        =>   data_mask_r(3 downto 0),
                                         dqs_int_delay_in0  =>   dqs_int_delay_in0,
                                         dqs_int_delay_in1  =>   dqs_int_delay_in1,
                                         dqs_int_delay_in2  =>   dqs_int_delay_in2,
                                         dqs_int_delay_in3  =>   dqs_int_delay_in3,
                                         ddr_dq_val                 =>   dq(31 downto 0),
                                         ddr_dm             =>   ddr_dm(3 downto 0)
                                        );

   

end arc_iobs;                
         
 




   
