library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on

entity cmp_data is
port(
     clk            : in std_logic;
     data_valid     : in std_logic;
     lfsr_data      : in std_logic_vector(143 downto 0);
     read_data      : in std_logic_vector(143 downto 0);
     rst            : in std_logic;
     pass           : out std_logic;
     led_error_output : out std_logic;
     display         : out std_logic_vector(6 downto 0);
     lfsr_out_data  : out std_logic_vector(143 downto 0);
     read_out_data  : out std_logic_vector(143 downto 0);
     data_valid_out : out std_logic
     );
end cmp_data;

architecture arc_cmp_data of cmp_data is

component OBUF 
	port(
	     O    :	out   STD_LOGIC;
	     I    :	in    STD_LOGIC
	     );
	end component;

type state is (idle, flag_error);
signal led_state     : state;
signal display_buf   : std_logic_vector(6 downto 0);
signal display_n     : std_logic_vector(6 downto 0);
signal valid         : std_logic;
signal valid_1       : std_logic;
signal error         : std_logic;
signal error_r       : std_logic;
signal lfsr_0        : std_logic_vector(71 downto 0);
signal lfsr_1        : std_logic_vector(71 downto 0);
signal lfsr_0_r        : std_logic_vector(71 downto 0);
signal lfsr_1_r        : std_logic_vector(71 downto 0);
signal lfsr_0_r1        : std_logic_vector(71 downto 0);
signal lfsr_1_r1        : std_logic_vector(71 downto 0);
signal data_0        : std_logic_vector(71 downto 0);
signal data_1        : std_logic_vector(71 downto 0);
signal byte_err      : std_logic_vector(1 downto 0);
signal read_data_reg : std_logic_vector(143 downto 0);

begin

process(clk)
begin
 if clk'event and clk = '1' then
  if rst = '1' then
    read_data_reg <= (others => '0');
  else
    read_data_reg <= read_data;
  end if;
 end if;
end process;

--read_data_reg <= read_data;

process (clk)
begin
 if clk'event and clk ='1' then
  if rst = '1' then
      valid_1 <= '0';
      valid <= '0';
      lfsr_0_r <= (others => '0');
      lfsr_1_r <= (others => '0');
      lfsr_0_r1 <= (others => '0');
      lfsr_1_r1 <= (others => '0');
  else
      valid_1 <= valid;
      valid <= data_valid;
      lfsr_0_r <= lfsr_data( 71 downto 0 );
      lfsr_1_r <= lfsr_data(143 downto 72 );
      lfsr_0_r1 <= lfsr_0_r;
      lfsr_1_r1 <= lfsr_1_r;
  end if;
 end if;
end process;       

data_valid_out <= valid_1;

lfsr_out_data <= lfsr_1 & lfsr_0;
read_out_data <= data_1 & data_0;

data_0 <= read_data_reg(71 downto 0);
data_1 <= read_data_reg(143 downto 72);
			
lfsr_0 <= lfsr_0_r;
lfsr_1 <= lfsr_1_r;

byte_err(0) <= '1' when (data_0(71 downto 0) /= lfsr_0(71 downto 0)) else '0';
byte_err(1) <= '1' when (data_1(71 downto 0) /= lfsr_1(71 downto 0)) else '0';

--error <= valid and ( byte_err(0) or byte_err(1) );-- commented by shyam on 10 march
error <= valid and data_valid  and ( byte_err(0) or byte_err(1) );  -- added by shyam on 10 march
--- registered for timing. 
process(clk)
begin
   if clk'event and clk = '1' then
      if rst = '1' then
         error_r <= '0'; 
      else
         error_r <= error;
       end if;
    end if;
end process;



pass  <=  not error_r;		-- pass  <= ~error;

-- LED error output
process(clk)
begin
   if clk'event and clk = '1' then
      if rst = '1' then
          led_state <= idle;
      else
         case led_state is
            when idle =>
                if error_r = '1' then
                     led_state <= flag_error;
                else
                     led_state <= idle;
                end if;
            when flag_error =>
                led_state <= flag_error;
         end case;
      end if;
  end if;
end process;

led_error_output <= '1' when led_state = flag_error else
                    '0';
              
display_buf      <= "1111001" when led_state = flag_error else  --F
                    "1110011" ;  --P
                    
display_n <= not(display_buf);
    
disp_pins : for i in display'range generate
	display_pin : OBUF port map(display(i), display_n(i));
end generate;
	
end arc_cmp_data;
