--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 253 - Synthesizable DDR SDRAM Controller
--
--*******************************************************************************
--
--  File name :       controller.vhd
--
--  Description :     
--                    Main DDR SDRAM controller block. This includes the following
--                    features:
--                    - The controller state machine that controls the 
--                    initialization process upon power up, as well as the 
--                    read, write, and refresh commands. 
--                    - Accepts and decodes the user commands.
--                    - Generates the address and Bank address signals
--                    - Generates control signals for other modules, including
--                    the control signals for the dqs_en block.
--
--  Date - revision : 05/01/2002
--
--  Author :          Lakshmi Gopalakrishnan
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--
entity controller is
port(
    dip1             : in std_logic;
    clk              : in std_logic;
    --clk180           : in std_logic;
    rst0             : in std_logic;
    rst180           : in std_logic;
    address          : in std_logic_vector(21 downto 0);
    bank_address     : in std_logic_vector(1 downto 0);
    config_register  : in std_logic_vector(9 downto 0);
    command_register : in std_logic_vector(2 downto 0);
    burst_done       : in std_logic;
	 
    ddr_rasb_cntrl   : out std_logic;
    ddr_casb_cntrl   : out std_logic;
    ddr_web_cntrl    : out std_logic;
    ddr_BA_cntrl     : out std_logic_vector(1 downto 0);
    ddr_address_cntrl: out std_logic_vector(12 downto 0);
    ddr_cke_cntrl    : out std_logic;
    ddr_csb_cntrl    : out std_logic;
    dqs_enable       : out std_logic;
    dqs_reset        : out std_logic;
    write_enable     : out std_logic;
    rst_calib        : out std_logic;
    rst_dqs_div_int      : out std_logic;
    rst_dqs_div_int1 : out std_logic; 
    cmd_ack          : out std_logic;
    init             : out std_logic;
    ar_done          : out std_logic
    --burst_type		     : out std_logic
);
end controller;


architecture arc_controller of controller is

component FD
   port(
      Q                              :  out   STD_LOGIC;
      D                              :  in    STD_LOGIC;
      C                              :  in    STD_LOGIC);
end component;

component IOBUF_SSTL2_II
port (
       I  : in std_logic;
       T  : in std_logic;
       IO : inout std_logic;
       O  : out std_logic);
end component;



--attribute IOB : string ; 
--attribute IOB of rst_iob_out : label is "FALSE"; 



attribute syn_noprune : boolean;  -- Using Syn_Keep Derictive
attribute syn_preserve : boolean;  -- Using Syn_Keep Derictive
type s_m is (IDLE, PRECHARGE, LOAD_MODE_REG, AUTO_REFRESH, ACTIVE, 
         FIRST_WRITE, WRITE_WAIT, BURST_WRITE, READ_AFTER_WRITE, PRECHARGE_AFTER_WRITE,
         PRECHARGE_AFTER_WRITE_2,READ_WAIT, BURST_READ, BURST_STOP);
signal next_state,current_state: s_m;


signal ack_reg                : std_logic;
signal ack_o                  : std_logic;
signal address_reg            : std_logic_vector(21 downto 0);
signal address_config         : std_logic_vector(12 downto 0);
signal AUTO_REF               : std_logic;
signal auto_ref1              : std_logic;
signal AUTOREF_value          : std_logic;
signal AUTO_REF_detect        : std_logic;
signal AUTO_REF_detect1        : std_logic;
signal AUTO_REF_pulse_end     : std_logic;
signal AUTOREF_COUNT          : std_logic_vector(11 downto 0);
signal AUTOREF_CNT_val        : std_logic_vector(11 downto 0);
signal auto_refresh_cmd       : std_logic;
signal AR_done_p              : std_logic;
signal Burst_terminate        : std_logic;
signal BA_address_active      : std_logic_vector(1 downto 0);
signal BA_address_conflict    : std_logic;
signal BA_address_reg         : std_logic_vector(1 downto 0);
signal burst_length           : std_logic_vector(2 downto 0);
signal burst_length1           : std_logic_vector(2 downto 0);
signal burst_cnt_max          : std_logic_vector(2 downto 0);
--modified by shyam for burst length =4 and =8
signal CAS_COUNT              : std_logic_vector(2 downto 0);
signal cas_count_value        : std_logic_vector(2 downto 0);

signal cas_latency            : std_logic_vector(2 downto 0);
signal COLUMN_ADDRESS_reg     : std_logic_vector(8 downto 0);
signal COLUMN_ADDRESS_reg1     : std_logic_vector(8 downto 0);
signal COLUMN_ADDRESS_reg2     : std_logic_vector(8 downto 0);
signal COLUMN_ADDRESS_reg3     : std_logic_vector(8 downto 0);
signal COLUMN_ADDRESS_reg4     : std_logic_vector(8 downto 0);
signal COLUMN_ADDRESS_reg5     : std_logic_vector(8 downto 0);
signal COLUMN_ADDRESS_reg6     : std_logic_vector(8 downto 0);
signal COLUMN_ADDRESS         : std_logic_vector(8 downto 0);
signal command_reg            : std_logic_vector(2 downto 0);
signal config_reg             : std_logic_vector(9 downto 0);
signal CONFLICT               : std_logic;
signal CONFLICT_value         : std_logic;
--signal cmd                    : std_logic_vector(2 downto 0);
--signal cmd_r1                 : std_logic_vector(2 downto 0);
signal ddr_rasb1              : std_logic;
signal ddr_casb1              : std_logic;
signal ddr_web1               : std_logic;
signal ddr_rasb2              : std_logic;
signal ddr_casb2              : std_logic;
signal ddr_web2               : std_logic;
signal ddr_rasb3              : std_logic;
signal ddr_casb3              : std_logic;
signal ddr_web3               : std_logic;
signal ddr_rasb4              : std_logic;
signal ddr_casb4              : std_logic;
signal ddr_web4               : std_logic;
signal ddr_rst_dqs_rasb4      : std_logic;
signal ddr_rst_dqs_casb4      : std_logic;
signal ddr_rst_dqs_web4       : std_logic;
signal ddr_rasb5              : std_logic;
signal ddr_casb5              : std_logic;
signal ddr_web5               : std_logic;
signal ddr_ba1                : std_logic_vector(1 downto 0);
signal ddr_ba2                : std_logic_vector(1 downto 0);
signal ddr_ba3                : std_logic_vector(1 downto 0);
signal ddr_ba4                : std_logic_vector(1 downto 0);
signal ddr_ba5                : std_logic_vector(1 downto 0);
signal ddr_address1           : std_logic_vector(12 downto 0);
signal ddr_address2           : std_logic_vector(12 downto 0);
signal ddr_address3           : std_logic_vector(12 downto 0);
signal ddr_address4           : std_logic_vector(12 downto 0);
signal ddr_address5           : std_logic_vector(12 downto 0);
signal DQS_enable_out         : std_logic;
signal DQS_reset_out          : std_logic;
signal INIT_COUNT_value       : std_logic_vector(2 downto 0);
signal INIT_COUNT             : std_logic_vector(2 downto 0);
signal INIT_DONE              : std_logic;
signal INIT_DONE_value        : std_logic;
signal INIT_MEMORY            : std_logic;
signal init_mem               : std_logic;
signal initialize_memory      : std_logic;
signal ld_mode                : std_logic;
signal LOAD_RCD_read          : std_logic;
signal LOAD_RCD_write         : std_logic;
signal LOAD_READ              : std_logic;
signal LOAD_WRITE             : std_logic;
signal LOAD_READ_value        : std_logic;
signal LOAD_WRITE_value       : std_logic;
signal MRD_COUNT_value        : std_logic_vector(1 downto 0);
signal max_ref_cnt            : std_logic_vector(11 downto 0);
signal MRD_COUNT              : std_logic_vector(1 downto 0);
signal PRECHARGE_cmd          : std_logic;
signal ras_count_value        : std_logic_vector(3 downto 0);
signal ras_count              : std_logic_vector(3 downto 0);
signal rdburst_chk            : std_logic;
signal read_cmd               : std_logic;
signal read_cmd_issued        : std_logic;
signal read_cmd1              : std_logic;
signal read_cmd2              : std_logic;
signal read_cmd3              : std_logic;
signal read_cmd4              : std_logic;
signal read_cmd5              : std_logic;
signal read_cmd6              : std_logic;
signal read_cmd7              : std_logic;
signal read_cmd8              : std_logic;
signal read_rcd_end           : std_logic;
signal read_cmd_reg           : std_logic;
signal read_write_state       : std_logic;
signal RRD_COUNT              : std_logic_vector(1 downto 0);
signal RCDR_COUNT             : std_logic_vector(2 downto 0);
signal RCDW_COUNT             : std_logic_vector(1 downto 0);
signal RP_cnt_value           : std_logic_vector(2 downto 0);
signal RFC_COUNT_value        : std_logic_vector(3 downto 0);
signal RRD_COUNT_value        : std_logic_vector(1 downto 0);
signal RCDR_COUNT_value       : std_logic_vector(2 downto 0);
signal RCDW_COUNT_value       : std_logic_vector(1 downto 0);
signal RC_COUNT_value         : std_logic_vector(3 downto 0);
signal rdburst_end_cnt_value  : std_logic_vector(2 downto 0);
signal rdburst_end_cnt        : std_logic_vector(2 downto 0);
signal rdburst_end_1            : std_logic;
signal rdburst_end_2            : std_logic;
signal rdburst_end_3            : std_logic;
signal rdburst_end_4            : std_logic;
signal rdburst_end_5            : std_logic;
signal rdburst_end_6            : std_logic;
signal rdburst_end_7            : std_logic;
signal rdburst_end_8            : std_logic;
signal rdburst_end_r            : std_logic;
signal read_enable_out_r      : std_logic;
signal rdburst_end            : std_logic;
signal RP_COUNT               : std_logic_vector(2 downto 0);
signal RC_COUNT               : std_logic_vector(3 downto 0);
signal RFC_COUNT              : std_logic_vector(3 downto 0);
signal Read_enable_out        : std_logic;
signal ROW_ADDRESS            : std_logic_vector(12 downto 0);
signal ROW_ADDRESS_reg        : std_logic_vector(12 downto 0);
signal row_address_active_reg : std_logic_vector(12 downto 0);
signal row_address_conflict   : std_logic;
signal rst_dqs_div_r          : std_logic;
signal rst_dqs_div1_r          : std_logic;
signal rst_dqs_div2_r          : std_logic;
signal rst_dqs_div_r2         : std_logic;
signal rst_dqs_div1_r2        : std_logic;

signal wrburst_end_cnt_value  : std_logic_vector(2 downto 0);
signal wrburst_end_cnt        : std_logic_vector(2 downto 0);
signal wrburst_end            : std_logic;
signal wrburst_end_1          : std_logic;
signal wrburst_end_2          : std_logic;
signal wrburst_end_3          : std_logic;
signal wrburst_end_4          : std_logic;
signal wrburst_end_5          : std_logic;
signal wrburst_end_6          : std_logic;
signal wrburst_end_7          : std_logic;
signal wrburst_end_8          : std_logic;
signal wrburst_end_9          : std_logic;
signal wrburst_chk            : std_logic;
signal wrburst_chk_value      : std_logic;
signal WR_COUNT               : std_logic_vector(1 downto 0);
signal WR_COUNT_value         : std_logic_vector(1 downto 0);
signal Write_enable_out       : std_logic;
signal Write_cmd              : std_logic;
signal write_cmd_in           : std_logic;
signal write_cmd2             : std_logic;
signal write_cmd3             : std_logic;
signal write_cmd4             : std_logic;
signal write_cmd5             : std_logic;
signal write_cmd1             : std_logic;
signal write_cmd6             : std_logic;
signal write_cmd7             : std_logic;
signal write_cmd8             : std_logic;
signal write_cmd_issued       : std_logic;
signal GND                    : std_logic;
--modified by shyam 
signal dqs_div_cascount       : std_logic_vector(2 downto 0);

signal dqs_div_rdburstcount   : std_logic_vector(2 downto 0);

signal DQS_enable1            : std_logic;
signal DQS_enable2            : std_logic;
signal DQS_enable3            : std_logic;
signal DQS_enable4            : std_logic;
signal DQS_reset1_clk0        : std_logic;
signal DQS_reset2_clk0        : std_logic;
signal DQS_reset3_clk0        : std_logic;
signal DQS_reset4_clk0        : std_logic;
signal DQS_enable_int         : std_logic;
signal DQS_reset_int          : std_logic;
signal rst180_r               : std_logic;
signal rst0_r                 : std_logic;
signal rst_dqs_div_in0        : std_logic; 
signal clk180                 : std_logic;
signal rst_cond   : std_logic_vector(2 downto 0);
--signal init_done_1				: std_logic;								
--attribute syn_keep of clk180 : signal is true;

attribute syn_noprune of rst_dqs_div_r : signal is true;
attribute syn_noprune of rst_dqs_div1_r : signal is true;
attribute syn_noprune of rst_dqs_div2_r : signal is true;
attribute syn_preserve of rst_dqs_div_r : signal is true;
attribute syn_preserve of rst_dqs_div1_r : signal is true;
attribute syn_preserve of rst_dqs_div2_r : signal is true;
attribute syn_noprune of rst_dqs_div_r2 : signal is true;
attribute syn_noprune of rst_dqs_div1_r2 : signal is true;

attribute syn_preserve of rst_dqs_div_r2 : signal is true;
attribute syn_preserve of rst_dqs_div1_r2 : signal is true;



begin


clk180 <= not clk;



--  Input : CONFIG REGISTER FORMAT 
-- config_register = {   EMR(Enable/Disable DLL),
--                       BMR (Normal operation/Normal Operation with Reset DLL),
--                       BMR/EMR,
--                       CAS_latency (3),
--                       Burst type ,
--                       Burst_length (3) }
--
-- Input : COMMAND REGISTER FORMAT
--          000  - NOP
--          001  - Precharge 
--          010  - Auto Refresh
--          011  - SElf REfresh
--          100  - Write Request
--          101  - Load Mode Register
--          110  - Read request
--          111  - Burst terminate
--
-- Input : Address format
--   row address = input address(19 downto 8)
--   column addrs = input address( 7 downto 0)
--

ddr_csb_cntrl <= '0'; --dip3;
ddr_cke_cntrl <= dip1;
ROW_ADDRESS <= address_reg(21 downto 9); 
COLUMN_ADDRESS <= address_reg(8 downto 0);
init <= init_done;
GND <= '0';


ddr_rasb_cntrl <= ddr_rasb4;
ddr_casb_cntrl <= ddr_casb4;
ddr_web_cntrl <= ddr_web4;

process(clk180)
 begin
   if clk180'event and clk180 = '1' then 
     rst180_r <= rst180;
   end if;
end process;

process(clk)
 begin
   if clk'event and clk = '1' then 
     rst0_r <= rst0;
   end if;
end process;


--********************************************************************************************
-- register input commands from the user
-- 
--********************************************************************************************

  process(clk180)
  begin
   if clk180'event and clk180 = '1' then
    if rst180_r = '1' then
        config_reg <= "0000000000";
        command_reg <= "000";
        ROW_ADDRESS_reg <= "0000000000000";
        COLUMN_ADDRESS_reg <= "000000000";
        BA_address_reg <= "00";
        address_reg <= "0000000000000000000000";
    else
        config_reg <= config_register;
        command_reg <= command_register;
        ROW_ADDRESS_reg <= ROW_ADDRESS;
        COLUMN_ADDRESS_reg <= COLUMN_ADDRESS;
        BA_address_reg <= Bank_address;
        address_reg <= address;
    end if;
   end if;
  end process;


process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1' then
     burst_length <= "000";
     cas_latency  <= "000";
     
  else
     burst_length <= config_reg(2 downto 0);    
     cas_latency  <= config_reg(6 downto 4);
     
  end if;
 end if;
end process;

process(clk)
begin
 if clk'event and clk = '1' then
  if rst0_r = '1' then
     burst_length1 <= "000";
  else
     burst_length1 <= config_reg(2 downto 0);    
  end if;
 end if;
end process;

	      
process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1' then  
      PRECHARGE_cmd <= '0';
      read_cmd <= '0';
      Burst_terminate <= '0';
      initialize_memory <= '0';
      ld_mode <= '0';
      write_cmd_in <= '0'; 
      auto_refresh_cmd <= '0';
  else
      case command_reg is
         when "001" =>
             PRECHARGE_cmd <= '1';
         when "010" =>
             initialize_memory <= '1';
         when "011" =>
             auto_refresh_cmd <= '1';        
         when "100" =>
              write_cmd_in <= '1';    
         when "101" =>
              ld_mode <= '1';    
         when "110" =>
             read_cmd <= '1';
         when "111" =>
             Burst_terminate <= '1';
               
         when others =>
             PRECHARGE_cmd <= '0';
             read_cmd <= '0';
             Burst_terminate <= '0';
             ld_mode <= '0';
             initialize_memory <= '0';
             write_cmd_in <= '0';
             auto_refresh_cmd <= '0';
      end case;
  end if;
 end if;
end process;


--**************************************************************************
-- write_cmd is used to determine when there is a new write request
--**************************************************************************

-- register write_cmd until WRITE command needs to be issued

  process(clk180)
  begin
   if clk180'event and clk180 = '1' then
     if rst180_r = '1' then
       write_cmd1 <= '0';
       write_cmd2 <= '0';
       write_cmd3 <= '0';
       write_cmd4 <= '0';
       write_cmd5 <= '0';
       write_cmd6  <= '0';
       write_cmd7  <= '0';
       write_cmd8  <= '0';
       write_cmd   <= '0';
     else
       write_cmd1 <= write_cmd_in;
       write_cmd2 <= write_cmd1;
       write_cmd3 <= write_cmd2;
       write_cmd4 <= write_cmd3;
       write_cmd5 <= write_cmd4; 
       write_cmd6 <= write_cmd5; 
       write_cmd7 <= write_cmd6; 
       write_cmd8  <= write_cmd7;   
       write_cmd  <= write_cmd6;               
     end if;
    end if;
  end process; 

--************************************************************************
-- register read cmd until READ command needs to be issued 
--************************************************************************

  process(clk180)
  begin
   if clk180'event and clk180 = '1' then
     if rst180_r = '1' then
       read_cmd1 <= '0';
       read_cmd2 <= '0';
       read_cmd3 <= '0';
       read_cmd4 <= '0';
       read_cmd5 <= '0';
       read_cmd6  <= '0';
       read_cmd7  <= '0';
       read_cmd8  <= '0';
       read_cmd_reg   <= '0';
       read_rcd_end   <= '0';
     else
       read_cmd1 <= read_cmd;
       read_cmd2 <= read_cmd1;
       read_cmd3 <= read_cmd2;
       read_cmd4 <= read_cmd3;
       read_cmd5 <= read_cmd4; 
       read_cmd6 <= read_cmd5;  
       read_cmd7 <= read_cmd6;
       read_cmd_reg    <= read_cmd7; 
       read_cmd8       <= read_cmd7; 
       read_rcd_end    <= read_cmd8;
     end if;
    end if;
  end process; 



--********************************************************************************************                
-- MRD Counter
-- an executable command can be issued only after Tmrd(2 cycles) after a LMR command is issued
--********************************************************************************************
MRD_COUNT_value <= "11" when (next_state = LOAD_MODE_REG) else
                   (MRD_COUNT - "01") when (MRD_COUNT /= "00") else
                    MRD_COUNT;

--********************************************************************************************                
-- RFC Counter
-- an executable command can be issued only after Trfc(72 ns => 72/7.5 = 10 cycles) 
--                                after a AUTOREFRESH command is issued
--********************************************************************************************
RFC_COUNT_value <= "1111" when (next_state = AUTO_REFRESH) else
             (RFC_COUNT - "0001") when (RFC_COUNT /= "0000") else
              RFC_COUNT;

--********************************************************************************************                
-- RP Counter
-- an executable command can be issued only after Trp(20 ns for a -5 device => 4 cycles) 
--                        after a PRECHARGE command is issued
--********************************************************************************************
RP_cnt_value <= "100" when (next_state = PRECHARGE ) else
                (RP_COUNT - "001") when (RP_COUNT /= "000") else
                 RP_COUNT;

--********************************************************************************************                
-- RRD Counter
-- minimum interval between successive ACTIVE commands to different banks - Trrd
-- 2 clock cycles
--********************************************************************************************
RRD_COUNT_value <= "10" when (next_state = ACTIVE) else
                   (RRD_COUNT - 1) when RRD_COUNT /= "00" else
                    "00"; 

--*********************************************************************************************
-- ACTIVE to READ/WRITE counter
-- RCDr counter
-- ACTIVE to READ delay - (-5) device requires 20 ns of delay => 3 clock cycles
--
-- RCDW counter
-- ACTIVE to WRITE delay - (-5) device requires 15 ns of delay => 2 clock cycles
--
--*********************************************************************************************

LOAD_RCD_read <= '1' when command_reg = "110"  else
                 '0';

LOAD_RCD_write <= '1' when command_reg = "100" else
                  '0';


RCDR_COUNT_value <=  "100" when ( next_state = ACTIVE) else
                   (RCDR_COUNT - "001" ) when (RCDR_COUNT /= "000") else
                    RCDR_COUNT;

RCDW_COUNT_value <= "11" when  next_state = ACTIVE else
                   (RCDW_COUNT - "01") when (RCDW_COUNT /= "00") else
                    RCDW_COUNT;

--*********************************************************************************************
-- ACTIVE to PRECHARGE counter
-- RAS counter
-- ACTIVE to PRECHARGE delay -
-- the memory device require 56 ns (6 clock cycles)after issuing an ACTIVE command before issuing a
-- PRECHARGE command
--
--*********************************************************************************************
RAS_count_value <= "1000" when next_state = ACTIVE else
                    RAS_COUNT - "0001" when RAS_COUNT /= "0000" else
                    RAS_COUNT;

--**********************************************************************************************
-- RC counter
-- an ACTIVE command to a different row can be issued only after the previous row has been 
-- precharged & after Trc is met 
-- Trc = 60 ns = 8 clock cycles
--**********************************************************************************************
RC_COUNT_value <= "1100" when next_state = ACTIVE else
                  (RC_COUNT - "0001") when (RC_COUNT /= "0000") else
                   RC_COUNT;


--********************************************************************************************                
-- WR Counter
-- a PRECHARGE command can be applied only after 2 cycles after a WRITE command has finished 
-- executing
--********************************************************************************************
WR_COUNT_value <= "11" when (next_state = PRECHARGE_AFTER_WRITE) else
                  (WR_COUNT - "01") when (WR_COUNT /= "00") else
                  WR_COUNT;






--*********************************************************************************************
-- Auto refresh counter - the design uses AUTO REFRESH
-- 
-- the DDR SDRAM requires AUTO REFRESH cycles at an average interval of 15.6 us
-- the minimum frequency obtained from the system clock = 200 MHz/16 = 12.5 MHz = 80 ns
-- Hence, a counter value of 195 should be used to obtain a 15.6 us clock for Auto Refresh
-- Binary value of 195 = 11000011
--*********************************************************************************************
max_ref_cnt <= "100000100000";    -- AUTOREF_COUNT clocked with 133 MHz (15.6 us) -- 100000100000 (15.6 us)

AUTOREF_value  <= '1' when (AUTOREF_COUNT = max_ref_cnt and next_state = IDLE) else
                  '0';
                  
AUTOREF_CNT_val <= "000000000000" when AUTOREF_COUNT = max_ref_cnt else
                   AUTOREF_COUNT + "000000000001"; 
                   
process(clk180)   -- changed clk_dv16 to clk180
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1' then
     AUTOREF_COUNT <= "000000000000";
  else
     AUTOREF_COUNT <= AUTOREF_CNT_val;
  end if;
 end if;
end process;

process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1'  then
      AUTO_REF_pulse_end <= '0';
      AUTO_REF_detect1   <= '0';
      AUTO_REF_detect    <= '0';
  else
      AUTO_REF_detect1   <= AUTOREF_value;
      AUTO_REF_detect    <= AUTO_REF_detect1;
      AUTO_REF_pulse_end <= AUTO_REF_detect;
  end if;
 end if;
end process;

AUTO_REF1 <= '1' when ((AUTO_REF_detect = '1') and (AUTO_REF_pulse_end /= '1'))  else
           '0';
           
AR_done_p   <= '1' when RFC_COUNT = "0001" else
               '0';     
           
process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1' then
     auto_ref <= '0';
     AR_done  <= '0';
  else
     auto_ref <= auto_ref1;
     AR_done <= AR_done_p;
  end if;
 end if;
end process;
           

--**********************************************************************************************
-- Burst count value counter when there are cosecutive READs or WRITEs
-- While doing consecutive READs or WRITEs, the Burst_count value determines when the next 
-- READ or WRITE command should be issued. The burst length is determined while loading the 
-- Load Mode Register
-- burst_cnt_max shows the number of clock cycles for each butst
-- burst_cnt_max = 1 for a burst length of 2, since it is ddr
--**********************************************************************************************
burst_cnt_max <= "001" when burst_length = "001" else  --burst length==2
                 "010" when burst_length = "010" else		--burst length==4
                 "100" when burst_length = "011" else	 --burst length==8
                 "100" when burst_length = "111" else
                 "000";

--********************************************************************************************
-- CAS latency counter
-- CAS latencies of 2,3,4 can be set using Mode register bits M(6:4)
--
--      M6  M5  M4   CAS latency
--      0    1   0 -    2
--      0    1   1 -    3
--      1    0   0 -    4
--      others     -   reserved
-- This design uses a CAS latency of 3 for a clock rate of 200 MHz
--
--********************************************************************************************
cas_count_value <= "010" when (NEXT_STATE = BURST_READ  and  burst_length = "010" ) else  --100 for burst length =8
                   "100" when (NEXT_STATE = BURST_READ  and  burst_length = "011" ) else 
                    CAS_COUNT - "001" when CAS_COUNT /= "000" else
                    CAS_COUNT;



RDburst_end_cnt_value <= burst_cnt_max when ((CAS_COUNT = "001"  and  burst_length = "010") or
						(CAS_COUNT = "011"  and  burst_length = "011")) else  --011 for 
                      RDBURST_END_CNT - "001" when RDBURST_END_CNT /= "000" else
                     "000" ;



WRburst_end_cnt_value <= burst_cnt_max  when ( (next_state = FIRST_WRITE) or (next_state = BURST_WRITE) ) else
                                                     
                         WRburst_end_cnt - "001" when WRburst_end_cnt /= "000" else
                         "000";


wrburst_chk <= '1' when ( (next_state = BURST_WRITE) or (next_state = WRITE_WAIT)  ) else
                      '0';
                      
rdburst_chk  <= '1' when ( (next_state = BURST_READ) or (next_state = READ_WAIT) ) else
                 '0';


process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1' then
    rdburst_end_1 <= '0';
    rdburst_end_2 <= '0';
    rdburst_end_3 <= '0';
    rdburst_end_4 <= '0';
    rdburst_end_5 <= '0';
  else
    if ( ( CAS_COUNT(1) = '1' or CAS_COUNT(0) = '1')  and burst_done = '1') then 
        rdburst_end_1 <= '1';
    else
        rdburst_end_1 <= '0';
    end if;
    rdburst_end_2 <= rdburst_end_1;
    rdburst_end_3 <= rdburst_end_2;
    rdburst_end_4 <= rdburst_end_3;
    rdburst_end_5 <= rdburst_end_4;
    rdburst_end_6 <= rdburst_end_5;
    rdburst_end_7 <= rdburst_end_6;
    rdburst_end_8 <= rdburst_end_7;
  end if;
 end if;
end process;

rdburst_end_r <= rdburst_end_3 or rdburst_end_4 or rdburst_end_5 or rdburst_end_6 or rdburst_end_7 or rdburst_end_8;
rdburst_end <= rdburst_end_1 or rdburst_end_2 ;

process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1' then
     wrburst_end_1 <= '0';
     wrburst_end_2 <= '0';
     wrburst_end_3 <= '0';
     wrburst_end_4 <= '0';
     wrburst_end_5 <= '0';
     wrburst_end_6 <= '0';
     wrburst_end_7 <= '0';
     wrburst_end_8 <= '0';
     wrburst_end_9 <= '0';
  else
     if (  ( (WRburst_end_cnt(1) or WRburst_end_cnt(0)) = '1') and burst_done = '1' ) then
         wrburst_end_1 <= '1';
     else 
         wrburst_end_1 <= '0';
     end if;
     wrburst_end_2 <= wrburst_end_1;
     wrburst_end_3   <= wrburst_end_2; 
     wrburst_end_4  <= wrburst_end_3;
     wrburst_end_5  <= wrburst_end_4;
     wrburst_end_6  <= wrburst_end_5;
     wrburst_end_7  <= wrburst_end_6;
     wrburst_end_8  <= wrburst_end_7;
  end if;
 end if;  
end process;

wrburst_end <= wrburst_end_1 or wrburst_end_2 or wrburst_end_3;
--**********************************************************************************************
-- to generate the Data Strobe enable and reset signal
-- The DQS signal needs to be generated center aligned with the data. 
-- The controller generates the DQS enable signal when the state machine is in the FIRST_WRITE
-- state,to take care of the write preamble
--**********************************************************************************************
DQS_enable_out <= '1' when ( (next_state = FIRST_WRITE) or (next_state = BURST_WRITE)  or (WRburst_end_cnt /= "000")  ) else
                  '0';

DQS_reset_out  <= '1' when next_state = FIRST_WRITE else
                 '0';
                 
DQS_enable <= DQS_enable4;
dqs_reset  <= DQS_reset4_clk0;

process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1' then
    DQS_enable_int <= '0';
    DQS_reset_int  <= '0';
  else
    DQS_enable_int <= DQS_enable_out;
    DQS_reset_int  <= DQS_reset_out;
  end if;        
 end if;
end process;

process(clk)
begin
if clk'event and clk = '1' then
 if rst0_r = '1' then
   DQS_enable1    <= '0';
   DQS_enable2    <= '0';
   DQS_enable3    <= '0';
   DQS_enable4    <= '0'; 
   DQS_reset1_clk0 <= '0';
   DQS_reset2_clk0 <= '0';
   DQS_reset3_clk0 <= '0';
   DQS_reset4_clk0 <= '0';
 else                 
   DQS_enable1    <= DQS_enable_int;
   DQS_enable2    <= DQS_enable1;
   DQS_enable3    <= DQS_enable2;
   DQS_enable4    <= DQS_enable3;
   DQS_reset1_clk0 <= DQS_reset_int;
   DQS_reset2_clk0 <= DQS_reset1_clk0;
   DQS_reset3_clk0 <= DQS_reset2_clk0;
   DQS_reset4_clk0 <= DQS_reset3_clk0;
 end if;               
end if;                
end process;           
                       
--****************************************************************************************************
--Generating WRITE and READ enable signals
--*****************************************************************************************************

  write_enable_out <= '1' when ( (wrburst_chk = '1') or (WRburst_end_cnt /= "000"))  else
                     '0'; 

read_enable_out <= '1' when ( (cas_count /= "00" ) or (rdburst_chk = '1')  )else
                    '0'; 
                    
read_enable_out_r <= read_enable_out or rdburst_end_r;

-- WRITE or READ command has already been issued

write_cmd_issued <= write_cmd1 or write_cmd2 or write_cmd3 or write_cmd4 or write_cmd5 or write_cmd or
                         write_enable_out;

read_cmd_issued  <= read_cmd1 or read_cmd2 or read_cmd3 or read_cmd4 or read_cmd5 or read_cmd6 or read_cmd7
                        or read_cmd_reg or read_enable_out;
                        
process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1' then
   write_enable <= '0';
  else
   write_enable <= write_enable_out;
  end if;
 end if;
end process;

cmd_ack <= ack_reg;

ACK_REG_INST1 : FD port map (  Q => ack_reg,
                         D => ack_o,                             
                         C => clk180);                            

ack_o <= '1' when ((write_cmd6 = '1') or (read_cmd7 = '1')) else
       '0';

read_write_state <= '1' when (next_state = FIRST_WRITE or next_state = BURST_WRITE or 
                                 next_state = BURST_READ) else
                    '0';

--*********************************************************************************************
--  to initialize memory
--*********************************************************************************************



process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1' then
     init_memory <= '0';
     init_done   <= '0';
  else
     init_memory <= init_mem;
     init_done   <= init_done_value;
  end if;
 end if;
end process;

init_mem <=  '1' when initialize_memory = '1' else
             '0' when (  (init_count = "111") and (MRD_COUNT = "00") ) else
             init_memory;

-- counter for Memory Initialization sequence

INIT_COUNT_value <= (INIT_COUNT + "001") when (((next_state = PRECHARGE) or (next_state = LOAD_MODE_REG)
                                  or (next_state = AUTO_REFRESH)) and INIT_MEMORY = '1') else
                    INIT_COUNT ;
                    
INIT_DONE_value <= '1' when (  (INIT_COUNT = "111") and (MRD_COUNT = "01")  ) else
                   '0';                    

-- LOAD MODE REGISTER when a WRITE command / READ command has been issued

LOAD_WRITE_value <= '1' when (write_cmd_in = '1' and RP_COUNT = "000" and write_cmd_issued = '0') else
               '0';

LOAD_READ_value <= '1' when (read_cmd = '1' and RP_COUNT = "000" and read_cmd_issued = '0') else
                   '0';

-- To check if there is a bank conflict after an ACTIVE command has been issued for a particular bank

CONFLICT_value  <= '1' when (RRD_COUNT = "01" and BA_address_conflict = '1') else
                   '0';

process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1' then
   LOAD_WRITE <= '0';
   LOAD_READ  <= '0';
   CONFLICT   <= '0';
  else
   LOAD_WRITE <= LOAD_WRITE_value;
   LOAD_READ  <= LOAD_READ_value;
   CONFLICT   <= CONFLICT_value;
  end if;
 end if;
end process;

--**********************************************************************************************
-- Register counter values
--**********************************************************************************************
process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1'  then
     INIT_COUNT     <= "000";
     RP_COUNT       <= "000";
     MRD_COUNT      <= "00";
     RFC_COUNT      <= "0000";
     RAS_COUNT      <= "0000";
     CAS_COUNT      <= "000";
     RRD_COUNT      <= "00";
     RCDR_COUNT     <= "000";
     RCDW_COUNT     <= "00";
     RC_COUNT       <= "0000";
     RDBURST_END_CNT <= "000";
     WRburst_end_cnt <= "000";  
     WR_COUNT        <= "00";
  else
     INIT_COUNT     <= INIT_COUNT_value;
     RP_COUNT       <= RP_CNT_value;
     MRD_COUNT      <= MRD_COUNT_value after 1 ns;
     RFC_COUNT      <= RFC_COUNT_value;
     RAS_COUNT      <= RAS_COUNT_value;
     CAS_COUNT      <= CAS_COUNT_value;
     RRD_COUNT      <= RRD_COUNT_value;
     RCDR_COUNT     <= RCDR_COUNT_value;
     RCDW_COUNT     <= RCDW_COUNT_value;
     RC_COUNT       <= RC_COUNT_value;
     WRBURST_END_CNT <= WRBURST_END_CNT_value;
     RDBURST_END_CNT <= RDBURST_END_CNT_value;
     WR_COUNT        <= WR_COUNT_value;
  end if;
 end if;
end process;

--*********************************************************************************************
-- to check current state for the address bus
--*********************************************************************************************
process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1'  then
   current_state <= IDLE;
  else
   current_state <= next_state;
  end if;
 end if;
end process;

--*********************************************************************************************
-- main state machine
--*********************************************************************************************
process(clk180, rst180_r, RP_COUNT, INIT_COUNT, MRD_COUNT, RFC_COUNT, CAS_COUNT)
begin
if rst180_r = '1' then
    next_state <= IDLE;
elsif clk180'event and clk180 = '1' then
    case next_state is
      
         when IDLE =>
               if INIT_MEMORY = '1' then
                  case INIT_COUNT is 

                   -- this state is for NOP/Deselect
                   when "000" =>
                           next_state <= PRECHARGE;
                        
                   when "001" =>
                         if RP_COUNT = "001" then   
                              next_state <= LOAD_MODE_REG;
                         else
                               next_state <= IDLE;
                         end if;
                   when "010" =>
                         if MRD_COUNT = "01" then
                            next_state <= LOAD_MODE_REG;  -- for reseting DLL in Base Mode register
                         else
                            next_state <= IDLE;
                         end if;
                   when "011" =>
                         if MRD_COUNT = "01" then
                            next_state <= PRECHARGE;
                         else
                            next_state <= IDLE;
                         end if;
                   when "100" => 
                         if RP_COUNT = "001" then -- wait for 4 clock cycles (Trp)
                               next_state <= AUTO_REFRESH;
                         else
                               next_state <= IDLE;
                         end if;
                   when "101" =>
                         if RFC_COUNT = "0001" then
                           next_state <= AUTO_REFRESH;
                         else 
                           next_state <= IDLE;
                         end if;
                   when "110" =>
                         if RFC_count = "0001" then
                            next_state <= LOAD_MODE_REG;  -- to deactivate the rst DLL bit in the LMR
                         else
                            next_state <= IDLE;
                         end if;
                   when "111" =>
                         if MRD_COUNT /= "00" then
                           next_state <= IDLE;
                         end if;
                   when others =>
                        next_state <= IDLE;
                   end case; 

            elsif PRECHARGE_CMD = '1' then
                  next_state <= PRECHARGE;

            elsif LOAD_WRITE = '1' or LOAD_READ = '1' then 
                 next_state <= LOAD_MODE_REG;
                             
            elsif (auto_refresh_cmd = '1' and RP_COUNT = "000") then
                  next_state <= AUTO_REFRESH; -- normal Refresh in the IDLE state
              
           elsif MRD_COUNT = "01" or CONFLICT = '1' then
                  next_state <= ACTIVE; 
           
            elsif RCDW_COUNT = "01" and write_cmd = '1'then
                  next_state <= FIRST_WRITE;

            elsif RCDR_COUNT = "001" and read_rcd_end = '1' then
                  next_state <= BURST_READ;

            else
                  next_state <= IDLE;
            end if;              


         when PRECHARGE =>
              next_state <= IDLE;

         when LOAD_MODE_REG =>
              next_state <= IDLE;
      
         when AUTO_REFRESH =>
              next_state <= IDLE; 
         
         when ACTIVE =>
              next_state <= IDLE;
         
         when FIRST_WRITE =>
              -- to meet the write preamble   
              if burst_length = "001" then
                  next_state <= BURST_WRITE;
              else
                  next_state <= WRITE_WAIT;
              end if;
    
         when WRITE_WAIT =>
                     case wrburst_end is
                       when '1' =>
                         next_state <= PRECHARGE_AFTER_WRITE;  
                     
                     when '0' =>
                            if WRburst_end_cnt = "010"  then-- changed from 010
                                 next_state <= BURST_WRITE;
                            else
                                 next_state <= WRITE_WAIT;
                            end if;
                     when others =>
                            next_state <= WRITE_WAIT;
                         
                     end case;

          when BURST_WRITE =>
                     if (burst_length = "001" ) then
                          next_state <= BURST_WRITE;
                     else
                         next_state <= WRITE_WAIT;
                     end if; 
         
          when READ_after_WRITE =>
                     next_state <= BURST_READ;
          
          when PRECHARGE_AFTER_WRITE =>
                        next_state <= PRECHARGE_AFTER_WRITE_2;
                     
          when PRECHARGE_AFTER_WRITE_2 =>   
                        next_state <= PRECHARGE;
 
          when READ_WAIT =>
                     case rdburst_end is
                     
                       
                      when '1' =>
                         
                      next_state <= PRECHARGE_AFTER_WRITE;
                            
                    when '0' =>
                         if CAS_COUNT = "10" then
                             next_state <= BURST_READ;
                         else
                             next_state <= READ_WAIT;
                         end if;

                    when others =>
                         next_state <= READ_WAIT;
                    end case;
          when BURST_READ =>
                     if burst_length = "001"  then
                          next_state <= BURST_READ;
                     else
                         next_state <= READ_WAIT;
                     end if; 
                    
          when BURST_STOP =>
                    next_state <= READ_WAIT;
          when others =>
                next_state <= IDLE;
        end case;
end if;
end process;    

--************************************************************************************************
-- address generation logic
--************************************************************************************************

address_config(12 downto 7) <= "000000" when INIT_COUNT = "010" else 
                               "000010" when INIT_COUNT = "011" else
                               "001000" when current_state = PRECHARGE else --and BA_address_reg ="00" else-- PRECHARGE all
                               "000000";

address_config(6 downto 4) <=  "000" when INIT_COUNT = "010" else
                               cas_latency when current_state = LOAD_MODE_REG else
                                "000";
address_config(3)          <= '0' ; -- design uses sequential burst
address_config(2 downto 0) <=  "000" when INIT_COUNT = "010" else
                               burst_length when current_state = LOAD_MODE_REG else
                              "000";


ddr_address1 <=    address_config when current_state = LOAD_MODE_REG or current_state = PRECHARGE else
                   ROW_ADDRESS_reg when current_state = ACTIVE else
                   ("0000" & COLUMN_ADDRESS_reg) when current_state = BURST_WRITE or current_state = FIRST_WRITE or
                                                                current_state = BURST_READ else
                                                
                   "0000000000000";

    
    
     DDR_BA1 <=     "01" when  (current_state = LOAD_MODE_REG and INIT_COUNT = "010")  else
                     BA_ADDRESS_reg when ( current_state = ACTIVE or current_state = FIRST_WRITE or
                                          current_state = BURST_WRITE or current_state = BURST_READ)else
                    "00";

--********************************************************************************************************
--  register row address
--********************************************************************************************************
process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1' then
   row_address_active_reg <= "0000000000000";
   row_address_conflict <= '0';
  else
   if next_state = ACTIVE then
       row_address_active_reg <= row_address_reg;
   else
       row_address_active_reg <= row_address_active_reg;
   end if;
   if row_address_reg /= row_address_active_reg then
       row_address_conflict <= '1';
   else
       row_address_conflict <= '0';
   end if;
  end if;   
 end if;
end process;

--********************************************************************************************************
--  register bank address
--********************************************************************************************************

process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1' then
   BA_address_active <= "00";
   BA_address_conflict <= '0';
  else
   if next_state = ACTIVE then
       BA_address_active <= BA_address_reg;
   else
       BA_address_active <= BA_address_active;
   end if;
   if BA_address_reg /= BA_address_active then
       BA_address_conflict <= '1';
   else
       BA_address_conflict <= '0';
   end if;
  end if;   
 end if;
end process;

--********************************************************************************************************
--  register column address
--********************************************************************************************************
process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1' then
     column_address_reg1 <= "000000000";
     column_address_reg2 <= "000000000";
     column_address_reg3 <= "000000000";
     column_address_reg4 <= "000000000";
     column_address_reg5 <= "000000000";
     column_address_reg6 <= "000000000";
  else
     column_address_reg1 <= column_address_reg;
     column_address_reg2 <= column_address_reg1;
     column_address_reg3 <= column_address_reg2;
     column_address_reg4 <= column_address_reg3;
     column_address_reg5 <= column_address_reg4;
     column_address_reg6 <= column_address_reg5;
  end if;
 end if;
end process;



--**************************************************************************************************
--Pipeline stages for ddr_address and ddr_ba
--**************************************************************************************************

process(clk180)
begin
if clk180'event and clk180 = '1' then 
if rst180_r = '1' then
   ddr_address2  <= "0000000000000";
   ddr_address3  <= "0000000000000";
   ddr_ba2       <= "00";
   ddr_ba3       <= "00";
else
    ddr_address2 <= ddr_address1;
    ddr_address3 <= ddr_address2;
    ddr_ba2      <= ddr_ba1;
    ddr_ba3      <= ddr_ba2;
end if;
end if;
end process;

process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1' then
   ddr_address4  <= "0000000000000";
   ddr_address5  <= "0000000000000";
   ddr_address_cntrl   <= "0000000000000";
  else
   ddr_address4  <= ddr_address3;
   ddr_address5  <= ddr_address4;
   ddr_address_cntrl   <= ddr_address4;
  end if;
 end if;
end process;

process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1' then
     ddr_ba4 <= "00";
     ddr_ba5 <= "00";
     ddr_ba_cntrl <= "ZZ";

  else
     ddr_ba4 <= ddr_ba3;--ba3
     ddr_ba5 <= ddr_ba4;
     ddr_ba_cntrl  <= ddr_ba4;

  end if;
 end if;
end process;

--************************************************************************************************
-- control signals to the Memory
--************************************************************************************************

ddr_rasb1 <= '0' when ( current_state = ACTIVE or current_state = PRECHARGE or current_state = AUTO_REFRESH 
                    or current_state = LOAD_MODE_REG) else
             '1';

ddr_casb1 <= '0' when ( current_state = BURST_READ or current_state = BURST_WRITE or current_state = FIRST_WRITE
                      or current_state = AUTO_REFRESH or current_state = LOAD_MODE_REG) else
             '1';

ddr_web1  <= '0' when (current_state = BURST_WRITE or current_state = FIRST_WRITE 
                         or current_state = BURST_STOP 
                        or current_state = PRECHARGE or current_state = LOAD_MODE_REG) else
             '1';

--*************************************************************************************************
-- register CONTROL SIGNALS outputs
--**************************************************************************************************
process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1'  then
   ddr_rasb3 <= '1';
   ddr_casb3 <= '1';
   ddr_web3  <= '1';
   ddr_rasb2 <= '1';
   ddr_casb2 <= '1';
   ddr_web2  <= '1';
  else
   ddr_rasb2    <= ddr_rasb1;
   ddr_casb2    <= ddr_casb1;
   ddr_web2     <= ddr_web1;
   ddr_rasb3     <= ddr_rasb2;
   ddr_casb3    <= ddr_casb2;
   ddr_web3     <= ddr_web2;
  end if;
 end if;
end process;

process(clk180)
begin
 if clk180'event and clk180 = '1' then
  if rst180_r = '1' then
   ddr_rasb4  <= '1';
   ddr_casb4  <= '1';
   ddr_web4   <= '1'; 
   ddr_rst_dqs_rasb4  <= '1';
   ddr_rst_dqs_casb4  <= '1';
   ddr_rst_dqs_web4   <= '1'; 
   ddr_rasb5 <= '1';
   ddr_casb5 <= '1';
   ddr_web5  <= '1';
  else
   ddr_rasb4  <= ddr_rasb3;
   ddr_casb4  <= ddr_casb3;
   ddr_web4   <= ddr_web3;
   ddr_rasb5   <= ddr_rasb4;
   ddr_casb5   <= ddr_casb4;
   ddr_web5    <= ddr_web4;

	if(cas_latency = "010") then  --CL2
		ddr_rst_dqs_rasb4  <= ddr_rasb2;
   		ddr_rst_dqs_casb4  <= ddr_casb2;
   		ddr_rst_dqs_web4   <= ddr_web2; 
	elsif(cas_latency = "011") then --CL3
		ddr_rst_dqs_rasb4  <= ddr_rasb3;
   		ddr_rst_dqs_casb4  <= ddr_casb3;
   		ddr_rst_dqs_web4   <= ddr_web3; 
	else
	  ddr_rst_dqs_rasb4  <= ddr_rst_dqs_rasb4;
   	ddr_rst_dqs_casb4  <= ddr_rst_dqs_casb4;
   	ddr_rst_dqs_web4   <= ddr_rst_dqs_web4; 
	end if;

  end if;  
 end if; 
end process;

process(clk)
begin
 if clk'event and clk = '1' then
  if rst0_r = '1' then
      rst_cond <= "111";  
  else
     if(cas_latency = "010") then
        rst_cond <= (ddr_rasb2 & ddr_casb2 & ddr_web2);
     elsif(cas_latency = "011") then --CL3
        rst_cond <= (ddr_rasb3 & ddr_casb3 & ddr_web3);
     else
        rst_cond <= rst_cond;
     end if; 

  end if;
 end if;
end process;
  
process(clk)
begin
  if clk'event and clk = '1' then
    if rst0_r = '1' then
        dqs_div_cascount <= "000";
        dqs_div_rdburstcount <= "000";
        rst_dqs_div_r <= '0';
        rst_dqs_div1_r <= '0';
        rst_dqs_div2_r <= '0';
    else
        if (rst_cond = "101"   and   burst_length1 = "010") then
             dqs_div_cascount <= "010";  --100
        elsif(rst_cond = "101" and   burst_length1 = "011") then
             dqs_div_cascount <= "100";  --100
             
         else if dqs_div_cascount /= "000" then
             dqs_div_cascount <= dqs_div_cascount - "001";
          else
                 dqs_div_cascount <= dqs_div_cascount;
             end if;
        end if;
        
        if (dqs_div_cascount = "001"  and  burst_length1 = "010") then  --011
            dqs_div_rdburstcount <= "010"; --100
        elsif (dqs_div_cascount = "011"  and  burst_length1 = "011") then  --011
            dqs_div_rdburstcount <= "100"; --100
        else 
            if dqs_div_rdburstcount /= "000" then
               dqs_div_rdburstcount <= dqs_div_rdburstcount - "001";
            else
               dqs_div_rdburstcount <= dqs_div_rdburstcount;
            end if;
        end if;
        

        if (dqs_div_cascount = "001"   and  burst_length1 = "010")then --011
            rst_dqs_div_r <= '1';
            rst_dqs_div1_r <= '1';
            rst_dqs_div2_r <= '1'; 
           
         elsif (dqs_div_cascount = "011"   and  burst_length1 = "011")then --011
            rst_dqs_div_r <= '1';
            rst_dqs_div1_r <= '1';
            rst_dqs_div2_r <= '1';
       
        elsif (dqs_div_rdburstcount = "001" and dqs_div_cascount = "000") then
            rst_dqs_div_r <= '0';
            rst_dqs_div1_r<= '0';
            rst_dqs_div2_r<= '0';
            
           
        else
            rst_dqs_div_r <= rst_dqs_div_r;
            rst_dqs_div1_r <= rst_dqs_div1_r;
            rst_dqs_div2_r <= rst_dqs_div2_r;
            
        end if;

      
        

  end if;
  end if;
end process;    


rst_dqs_div_int <= rst_dqs_div_r2;
rst_dqs_div_int1 <= rst_dqs_div1_r2;

process(clk)
begin
 if clk'event and clk = '1' then
  if rst0 = '1'  then
   rst_dqs_div_r2 <= '0';
   rst_dqs_div1_r2 <= '0';
  else
          rst_dqs_div_r2 <= rst_dqs_div_r;
        rst_dqs_div1_r2 <= rst_dqs_div1_r; 
  end if;
 end if;
end process;


process(clk)
begin
 if clk'event and clk = '1' then
  if rst0 = '1'  then
   rst_calib <= '0';
  else
   rst_calib <= rst_dqs_div2_r;
  end if;
 end if;
end process;

--rst_calib0 : FD port map (
--                         Q    => rst_calib,     
--                         D    => rst_dqs_div_r,                         
--                         C    => clk180);         


--rst_iob_out : FD port map (
--                         Q    => rst_dqs_div_int,     
--                         D    => rst_dqs_div_r,                         
--                         C    => clk180);

--process(clk180)
--begin
--   if clk180'event and clk180 = '0' then
 --     rst_dqs_div_int <= rst_dqs_div_r;
 --   end if;
--end process; 

--process(clk180)
--begin
-- if clk180'event and clk180 = '1' then
--     rst_calib <= rst_dqs_div_r;
--  end if;
--end process;                                                                              
                            
end arc_controller;                
         
 




   