--*********************************************************************
-- DDR 72 Bit Controller DATA PATH for LEFT RIGHT Pins

-- In the current DATA PATH logic DATA CAPTURE part was modified.
-- The below changes were made to reduce the resources in 
-- the data capture

-- in the current architecture data ( dq ) from ddr memory 
-- directly stored into the FIFO's.

-- Architectural changes :

-- Used only TWO FIFOs ( instead of FOUR FIFOs ) 
-- Used Single col ( col0 ) dqs_delayed_col signals
-- Used Gray Counters for write and read pointers of the FIFOs 

-- fbit stage is removed from ddr1_dqbit module ( in the data capture )
-- dq_clk stage was removed 
-- dqs_clk_div logic was removed
-- ddr1_transfer_done logic was removed 
-- data valid signals registering in clk90 domain was removed

-- fifo_0 and fifo_1 wr_addr was double registered in clk90 domain
-- only fifo_0 and fifo_1 empty signals were used for read_data_valid_1 logic  
-- write enable for the FIFOs derived from rst_dqs_div signal


-- Code revised by 	: Narayana Murty.
-- Date 			      : Nov 18, 2003. 

--*********************************************************************


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--library synplify; 
--use synplify.attributes.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--

entity datapath_iobs is
port(
    clk               : in std_logic;
	  clk90             : in std_logic;
    reset90_r        : in std_logic;
	  dqs_reset         : in std_logic;
    dqs_enable        : in std_logic;
    ddr_dqs           : inout std_logic_vector(8 downto 0);
    ddr_dq            : inout std_logic_vector(71 downto 0);
	  write_data_falling: in std_logic_vector(71 downto 0);
    write_data_rising : in std_logic_vector(71 downto 0);
    write_en_val      : in std_logic;
    write_en_val1     : in std_logic;
    data_mask_f       : in std_logic_vector(8 downto 0);
    data_mask_r       : in std_logic_vector(8 downto 0);
    dqs_int_delay_in0 : out std_logic;
    dqs_int_delay_in1 : out std_logic;
    dqs_int_delay_in2 : out std_logic;
    dqs_int_delay_in3 : out std_logic;
    dqs_int_delay_in4 : out std_logic;
    dqs_int_delay_in5 : out std_logic;
    dqs_int_delay_in6 : out std_logic;
    dqs_int_delay_in7 : out std_logic;
    dqs_int_delay_in8 : out std_logic;
    ddr_dq_val         : out std_logic_vector(71 downto 0);
    ddr_dm            : out std_logic_vector(8 downto 0)  
);
end datapath_iobs;


architecture arc_datapathiobs of datapath_iobs is


attribute syn_keep : boolean;  -- Using Syn_Keep Derictive


component s3_dqs_iob
port(
     clk            : in std_logic;
     clk180         : in std_logic;
     ddr_dqs_reset  : in std_logic;
     ddr_dqs_enable : in std_logic;
     ddr_dqs        : inout std_logic;
     dqs            : out std_logic
     );
end component;

component s3_ddr_iob  
port (
      ddr_dq_inout       : inout std_logic; --Bi-directional SDRAM data bus
      write_data_falling : in std_logic;    --Transmit data, output on falling edge
      write_data_rising  : in std_logic;    --Transmit data, output on rising edge
      read_data_in       : out std_logic;   -- Received data
      clk90              : in std_logic;    --Clock 90
      clk270             : in std_logic;
      write_en_val       : in std_logic;    --Transmit enable
      reset              : in std_logic); 

end component;

component ddr1_dm                                     
port (                                                
      ddr_dm       : out std_logic_vector(8 downto 0);
      mask_falling : in std_logic_vector(8 downto 0); 
      mask_rising  : in std_logic_vector(8 downto 0); 
      clk90        : in std_logic;    
      clk270       : in std_logic                     
      );   

                                           
end component;                                        

signal clk270       : std_logic;
signal clk180       : std_logic;
signal ddr_dq_in    : std_logic_vector( 71 downto 0);
--attribute syn_keep of clk180 : signal is true; 
--attribute syn_keep of clk270 : signal is true; 

signal write_en_val_r : std_logic;
signal write_en_val1_r : std_logic;


begin

clk270  <=  not clk90;
clk180  <=  not clk;


ddr_dq_val <= ddr_dq_in;

ddr1_dm0 : ddr1_dm port map (
                             ddr_dm       => ddr_dm(8 downto 0),
                             mask_falling => data_mask_f(8 downto 0),
                             mask_rising  => data_mask_r(8 downto 0),
                             clk90        => clk90,
                             clk270       => clk270
                            );

process(clk90)
begin
 if clk90'event and clk90 = '0' then
  if reset90_r = '1' then
  
    write_en_val_r    <= '0';
    write_en_val1_r   <= '0';
  
  else

     write_en_val_r  <= write_en_val;
     write_en_val1_r  <= write_en_val1; 
   
  end if;
 end if;
end process;

--***********************************************************************
--    Read Data Capture Module Instantiations
--***********************************************************************
-- DQS IOB instantiations
--***********************************************************************

 s3_dqs_iob0 : s3_dqs_iob port map (
                              clk            => clk,
                              clk180         => clk180,
                              ddr_dqs_reset  => dqs_reset,
                              ddr_dqs_enable => dqs_enable,
                              ddr_dqs        => ddr_dqs(0),
                              dqs            => dqs_int_delay_in0
					);
                             
 s3_dqs_iob1 : s3_dqs_iob port map (
                              clk            => clk,
                              clk180         => clk180,
                              ddr_dqs_reset  => dqs_reset,
                              ddr_dqs_enable => dqs_enable,
                              ddr_dqs        => ddr_dqs(1),
                              dqs            => dqs_int_delay_in1
                             );
                             
 s3_dqs_iob2 : s3_dqs_iob port map (
                              clk            => clk,
                              clk180         => clk180,
                              ddr_dqs_reset  => dqs_reset,
                              ddr_dqs_enable => dqs_enable,
                              ddr_dqs        => ddr_dqs(2),
                              dqs            => dqs_int_delay_in2
                             );
                             
 s3_dqs_iob3 : s3_dqs_iob port map (
                              clk            => clk,
                              clk180         => clk180,
                              ddr_dqs_reset  => dqs_reset,
                              ddr_dqs_enable => dqs_enable,
                              ddr_dqs        => ddr_dqs(3),
                              dqs            => dqs_int_delay_in3
                             );
                             
 s3_dqs_iob4 : s3_dqs_iob port map (
                              clk            => clk,
                              clk180         => clk180,
                              ddr_dqs_reset  => dqs_reset,
                              ddr_dqs_enable => dqs_enable,
                              ddr_dqs        => ddr_dqs(4),
                              dqs            => dqs_int_delay_in4
                             );
                             
 s3_dqs_iob5 : s3_dqs_iob port map (
                              clk            => clk,
                              clk180         => clk180,
                              ddr_dqs_reset  => dqs_reset,
                              ddr_dqs_enable => dqs_enable,
                              ddr_dqs        => ddr_dqs(5),
                              dqs            => dqs_int_delay_in5
                             );
                             
 s3_dqs_iob6 : s3_dqs_iob port map (
                              clk            => clk,
                              clk180         => clk180,
                              ddr_dqs_reset  => dqs_reset,
                              ddr_dqs_enable => dqs_enable,
                              ddr_dqs        => ddr_dqs(6),
                              dqs            => dqs_int_delay_in6
                             );
                             
 s3_dqs_iob7 : s3_dqs_iob port map (
                              clk            => clk,
                              clk180         => clk180,
                              ddr_dqs_reset  => dqs_reset,
                              ddr_dqs_enable => dqs_enable,
                              ddr_dqs        => ddr_dqs(7),
                              dqs            => dqs_int_delay_in7
                             );
                             
 s3_dqs_iob8 : s3_dqs_iob port map (
                              clk            => clk,
                              clk180         => clk180,
                              ddr_dqs_reset  => dqs_reset,
                              ddr_dqs_enable => dqs_enable,
                              ddr_dqs        => ddr_dqs(8),
                              dqs            => dqs_int_delay_in8
                             );
                             


--******************************************************************************************************************************
-- DDR Data bit instantiations (72-bits)
--******************************************************************************************************************************            
   
s3_ddr_iob0 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(0),
      							write_data_falling => write_data_falling(0),
      							write_data_rising  => write_data_rising(0),
      							read_data_in       => ddr_dq_in(0),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);


s3_ddr_iob1 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(1),
      							write_data_falling => write_data_falling(1),
      							write_data_rising  => write_data_rising(1),
      							read_data_in       => ddr_dq_in(1),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob2 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(2),
      							write_data_falling => write_data_falling(2),
      							write_data_rising  => write_data_rising(2),
      							read_data_in       => ddr_dq_in(2),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob3 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(3),
      							write_data_falling => write_data_falling(3),
      							write_data_rising  => write_data_rising(3),
      							read_data_in       => ddr_dq_in(3),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob4 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(4),
      							write_data_falling => write_data_falling(4),
      							write_data_rising  => write_data_rising(4),
      							read_data_in       => ddr_dq_in(4),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob5 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(5),
      							write_data_falling => write_data_falling(5),
      							write_data_rising  => write_data_rising(5),
      							read_data_in       => ddr_dq_in(5),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob6 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(6),
      							write_data_falling => write_data_falling(6),
      							write_data_rising  => write_data_rising(6),
      							read_data_in       => ddr_dq_in(6),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob7 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(7),
      							write_data_falling => write_data_falling(7),
      							write_data_rising  => write_data_rising(7),
      							read_data_in       => ddr_dq_in(7),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob8  : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(8),
      							write_data_falling => write_data_falling(8),
      							write_data_rising  => write_data_rising(8),
      							read_data_in       => ddr_dq_in(8),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob9 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(9),
      							write_data_falling => write_data_falling(9),
      							write_data_rising  => write_data_rising(9),
      							read_data_in       => ddr_dq_in(9),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);


s3_ddr_iob10  : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(10),
      							write_data_falling => write_data_falling(10),
      							write_data_rising  => write_data_rising(10),
      							read_data_in       => ddr_dq_in(10),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);


s3_ddr_iob11 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(11),
      							write_data_falling => write_data_falling(11),
      							write_data_rising  => write_data_rising(11),
      							read_data_in       => ddr_dq_in(11),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob12  : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(12),
      							write_data_falling => write_data_falling(12),
      							write_data_rising  => write_data_rising(12),
      							read_data_in       => ddr_dq_in(12),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob13  : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(13),
      							write_data_falling => write_data_falling(13),
      							write_data_rising  => write_data_rising(13),
      							read_data_in       => ddr_dq_in(13),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob14 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(14),
      							write_data_falling => write_data_falling(14),
      							write_data_rising  => write_data_rising(14),
      							read_data_in       => ddr_dq_in(14),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob15 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(15),
      							write_data_falling => write_data_falling(15),
      							write_data_rising  => write_data_rising(15),
      							read_data_in       => ddr_dq_in(15),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob16 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(16),
      							write_data_falling => write_data_falling(16),
      							write_data_rising  => write_data_rising(16),
      							read_data_in       => ddr_dq_in(16),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob17 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(17),
      							write_data_falling => write_data_falling(17),
      							write_data_rising  => write_data_rising(17),
      							read_data_in       => ddr_dq_in(17),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob18 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(18),
      							write_data_falling => write_data_falling(18),
      							write_data_rising  => write_data_rising(18),
      							read_data_in       => ddr_dq_in(18),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob19  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(19),
      							write_data_falling => write_data_falling(19),
      							write_data_rising  => write_data_rising(19),
      							read_data_in       => ddr_dq_in(19),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);



s3_ddr_iob20  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(20),
      							write_data_falling => write_data_falling(20),
      							write_data_rising  => write_data_rising(20),
      							read_data_in       => ddr_dq_in(20),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);


s3_ddr_iob21  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(21),
      							write_data_falling => write_data_falling(21),
      							write_data_rising  => write_data_rising(21),
      							read_data_in       => ddr_dq_in(21),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob22  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(22),
      							write_data_falling => write_data_falling(22),
      							write_data_rising  => write_data_rising(22),
      							read_data_in       => ddr_dq_in(22),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob23  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(23),
      							write_data_falling => write_data_falling(23),
      							write_data_rising  => write_data_rising(23),
      							read_data_in       => ddr_dq_in(23),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob24  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(24),
      							write_data_falling => write_data_falling(24),
      							write_data_rising  => write_data_rising(24),
      							read_data_in       => ddr_dq_in(24),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob25  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(25),
      							write_data_falling => write_data_falling(25),
      							write_data_rising  => write_data_rising(25),
      							read_data_in       => ddr_dq_in(25),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob26  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(26),
      							write_data_falling => write_data_falling(26),
      							write_data_rising  => write_data_rising(26),
      							read_data_in       => ddr_dq_in(26),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob27  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(27),
      							write_data_falling => write_data_falling(27),
      							write_data_rising  => write_data_rising(27),
      							read_data_in       => ddr_dq_in(27),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob28  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(28),
      							write_data_falling => write_data_falling(28),
      							write_data_rising  => write_data_rising(28),
      							read_data_in       => ddr_dq_in(28),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob29  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(29),
      							write_data_falling => write_data_falling(29),
      							write_data_rising  => write_data_rising(29),
      							read_data_in       => ddr_dq_in(29),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);


s3_ddr_iob30  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(30),
      							write_data_falling => write_data_falling(30),
      							write_data_rising  => write_data_rising(30),
      							read_data_in       => ddr_dq_in(30),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);


s3_ddr_iob31  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(31),
      							write_data_falling => write_data_falling(31),
      							write_data_rising  => write_data_rising(31),
      							read_data_in       => ddr_dq_in(31),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob32  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(32),
      							write_data_falling => write_data_falling(32),
      							write_data_rising  => write_data_rising(32),
      							read_data_in       => ddr_dq_in(32),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob33  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(33),
      							write_data_falling => write_data_falling(33),
      							write_data_rising  => write_data_rising(33),
      							read_data_in       => ddr_dq_in(33),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob34  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(34),
      							write_data_falling => write_data_falling(34),
      							write_data_rising  => write_data_rising(34),
      							read_data_in       => ddr_dq_in(34),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob35  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(35),
      							write_data_falling => write_data_falling(35),
      							write_data_rising  => write_data_rising(35),
      							read_data_in       => ddr_dq_in(35),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob36  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(36),
      							write_data_falling => write_data_falling(36),
      							write_data_rising  => write_data_rising(36),
      							read_data_in       => ddr_dq_in(36),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob37  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(37),
      							write_data_falling => write_data_falling(37),
      							write_data_rising  => write_data_rising(37),
      							read_data_in       => ddr_dq_in(37),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob38  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(38),
      							write_data_falling => write_data_falling(38),
      							write_data_rising  => write_data_rising(38),
      							read_data_in       => ddr_dq_in(38),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob39  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(39),
      							write_data_falling => write_data_falling(39),
      							write_data_rising  => write_data_rising(39),
      							read_data_in       => ddr_dq_in(39),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val_r,
      							reset              => reset90_r
						    	);


s3_ddr_iob40  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(40),
      							write_data_falling => write_data_falling(40),
      							write_data_rising  => write_data_rising(40),
      							read_data_in       => ddr_dq_in(40),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);


s3_ddr_iob41  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(41),
      							write_data_falling => write_data_falling(41),
      							write_data_rising  => write_data_rising(41),
      							read_data_in       => ddr_dq_in(41),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob42  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(42),
      							write_data_falling => write_data_falling(42),
      							write_data_rising  => write_data_rising(42),
      							read_data_in       => ddr_dq_in(42),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob43  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(43),
      							write_data_falling => write_data_falling(43),
      							write_data_rising  => write_data_rising(43),
      							read_data_in       => ddr_dq_in(43),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob44  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(44),
      							write_data_falling => write_data_falling(44),
      							write_data_rising  => write_data_rising(44),
      							read_data_in       => ddr_dq_in(44),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob45  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(45),
      							write_data_falling => write_data_falling(45),
      							write_data_rising  => write_data_rising(45),
      							read_data_in       => ddr_dq_in(45),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob46  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(46),
      							write_data_falling => write_data_falling(46),
      							write_data_rising  => write_data_rising(46),
      							read_data_in       => ddr_dq_in(46),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob47  : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(47),
      							write_data_falling => write_data_falling(47),
      							write_data_rising  => write_data_rising(47),
      							read_data_in       => ddr_dq_in(47),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

  s3_ddr_iob48 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(48),
      							write_data_falling => write_data_falling(48),
      							write_data_rising  => write_data_rising(48),
      							read_data_in       => ddr_dq_in(48),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob49  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(49),
      							write_data_falling => write_data_falling(49),
      							write_data_rising  => write_data_rising(49),
      							read_data_in       => ddr_dq_in(49),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);



s3_ddr_iob50  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(50),
      							write_data_falling => write_data_falling(50),
      							write_data_rising  => write_data_rising(50),
      							read_data_in       => ddr_dq_in(50),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);


s3_ddr_iob51  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(51),
      							write_data_falling => write_data_falling(51),
      							write_data_rising  => write_data_rising(51),
      							read_data_in       => ddr_dq_in(51),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob52  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(52),
      							write_data_falling => write_data_falling(52),
      							write_data_rising  => write_data_rising(52),
      							read_data_in       => ddr_dq_in(52),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob53  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(53),
      							write_data_falling => write_data_falling(53),
      							write_data_rising  => write_data_rising(53),
      							read_data_in       => ddr_dq_in(53),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob54  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(54),
      							write_data_falling => write_data_falling(54),
      							write_data_rising  => write_data_rising(54),
      							read_data_in       => ddr_dq_in(54),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob55  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(55),
      							write_data_falling => write_data_falling(55),
      							write_data_rising  => write_data_rising(55),
      							read_data_in       => ddr_dq_in(55),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob56  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(56),
      							write_data_falling => write_data_falling(56),
      							write_data_rising  => write_data_rising(56),
      							read_data_in       => ddr_dq_in(56),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob57  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(57),
      							write_data_falling => write_data_falling(57),
      							write_data_rising  => write_data_rising(57),
      							read_data_in       => ddr_dq_in(57),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob58  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(58),
      							write_data_falling => write_data_falling(58),
      							write_data_rising  => write_data_rising(58),
      							read_data_in       => ddr_dq_in(58),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob59  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(59),
      							write_data_falling => write_data_falling(59),
      							write_data_rising  => write_data_rising(59),
      							read_data_in       => ddr_dq_in(59),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);


s3_ddr_iob60  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(60),
      							write_data_falling => write_data_falling(60),
      							write_data_rising  => write_data_rising(60),
      							read_data_in       => ddr_dq_in(60),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);


s3_ddr_iob61  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(61),
      							write_data_falling => write_data_falling(61),
      							write_data_rising  => write_data_rising(61),
      							read_data_in       => ddr_dq_in(61),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob62  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(62),
      							write_data_falling => write_data_falling(62),
      							write_data_rising  => write_data_rising(62),
      							read_data_in       => ddr_dq_in(62),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob63  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(63),
      							write_data_falling => write_data_falling(63),
      							write_data_rising  => write_data_rising(63),
      							read_data_in       => ddr_dq_in(63),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob64 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(64),
      							write_data_falling => write_data_falling(64),
      							write_data_rising  => write_data_rising(64),
      							read_data_in       => ddr_dq_in(64),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

  s3_ddr_iob65 : s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(65),
      							write_data_falling => write_data_falling(65),
      							write_data_rising  => write_data_rising(65),
      							read_data_in       => ddr_dq_in(65),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

  s3_ddr_iob66  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(66),
      							write_data_falling => write_data_falling(66),
      							write_data_rising  => write_data_rising(66),
      							read_data_in       => ddr_dq_in(66),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob67  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(67),
      							write_data_falling => write_data_falling(67),
      							write_data_rising  => write_data_rising(67),
      							read_data_in       => ddr_dq_in(67),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob68  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(68),
      							write_data_falling => write_data_falling(68),
      							write_data_rising  => write_data_rising(68),
      							read_data_in       => ddr_dq_in(68),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob69  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(69),
      							write_data_falling => write_data_falling(69),
      							write_data_rising  => write_data_rising(69),
      							read_data_in       => ddr_dq_in(69),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob70  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(70),
      							write_data_falling => write_data_falling(70),
      							write_data_rising  => write_data_rising(70),
      							read_data_in       => ddr_dq_in(70),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);

s3_ddr_iob71  :  s3_ddr_iob  port map(
      							ddr_dq_inout       => ddr_dq(71),
      							write_data_falling => write_data_falling(71),
      							write_data_rising  => write_data_rising(71),
      							read_data_in       => ddr_dq_in(71),
      							clk90              => clk90,
      							clk270             => clk270,
      							write_en_val       => write_en_val1_r,
      							reset              => reset90_r
						    	);


end arc_datapathiobs;
