--#############################################################################//
--         Internal dqs delay structure for ddr sdram controller               //                          
--#############################################################################//

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;


-- pragma translate_off     
library UNISIM;             
use UNISIM.VCOMPONENTS.ALL; 
-- pragma translate_on      

entity tap_dly is 
              port (
		    clk    : in std_logic;
		    reset  : in std_logic; 	
		    tapIn : in std_logic;
		    flop2  : out std_logic_vector(31 downto 0)
		  );
end tap_dly;

architecture arc_tap_dly of tap_dly is

-------------------------------------------------------------------------------------------------------------------------------
attribute syn_keep : boolean;
-------------------------------------------------------------------------------------------------------------------------------

component LUT4
   generic(
      INIT                           :  bit_vector(15 downto 0) := x"0000" );
   port(
      O                              :	out   STD_ULOGIC;
      I0                             :	in    STD_ULOGIC;
      I1                             :	in    STD_ULOGIC;
      I2                             :	in    STD_ULOGIC;
      I3                             :	in    STD_ULOGIC
      );
end component;	

component FDR
    port(
      Q                              : out STD_LOGIC;
      C                              : in STD_LOGIC;
      D                              : in STD_LOGIC;
	R					 : in STD_LOGIC 	
      );
  end component;


  
signal tap        : std_logic_vector(31 downto 0);
signal flop1      : std_logic_vector(31 downto 0);



signal flop2n_0 :  std_logic;
signal flop2n_2 :  std_logic;
signal flop2n_4 :  std_logic;
signal flop2n_6 :  std_logic;
signal flop2n_8 :  std_logic;
signal flop2n_10:  std_logic;
signal flop2n_12:  std_logic;
signal flop2n_14:  std_logic;
signal flop2n_16:  std_logic;
signal flop2n_18:  std_logic;
signal flop2n_20:  std_logic;
signal flop2n_22:  std_logic;
signal flop2n_24:  std_logic;
signal flop2n_26:  std_logic;
signal flop2n_28:  std_logic;
signal flop2n_30:  std_logic;

signal high       : std_logic;
signal low        : std_logic;

----------------------------------------------------------------------------------------------------------------------
attribute syn_keep of flop1       :  signal is true;
attribute syn_keep of flop2n_0  :  signal is true;
attribute syn_keep of flop2n_2  :  signal is true;
attribute syn_keep of flop2n_4  :  signal is true;
attribute syn_keep of flop2n_6  :  signal is true;
attribute syn_keep of flop2n_8  :  signal is true;
attribute syn_keep of flop2n_10 :  signal is true;
attribute syn_keep of flop2n_12 :  signal is true;
attribute syn_keep of flop2n_14 :  signal is true;
attribute syn_keep of flop2n_16 :  signal is true;
attribute syn_keep of flop2n_18 :  signal is true;
attribute syn_keep of flop2n_20 :  signal is true;
attribute syn_keep of flop2n_22 :  signal is true;
attribute syn_keep of flop2n_24 :  signal is true;
attribute syn_keep of flop2n_26 :  signal is true;
attribute syn_keep of flop2n_28 :  signal is true;
attribute syn_keep of flop2n_30 :  signal is true;

attribute syn_keep of tap      : signal is true;
----------------------------------------------------------------------------------------------------------------------

begin

flop2(0) <= not flop2n_0;
flop2(2) <= not flop2n_2;
flop2(4) <= not flop2n_4;
flop2(6) <= not flop2n_6;
flop2(8) <= not flop2n_8;
flop2(10) <= not flop2n_10;
flop2(12) <= not flop2n_12;
flop2(14) <= not flop2n_14;
flop2(16) <= not flop2n_16;
flop2(18) <= not flop2n_18;
flop2(20) <= not flop2n_20;
flop2(22) <= not flop2n_22;
flop2(24) <= not flop2n_24;
flop2(26) <= not flop2n_26;
flop2(28) <= not flop2n_28;
flop2(30) <= not flop2n_30;

high <= '1';   
low  <= '0';

l0 :  LUT4  generic map (INIT => x"e2e2")  
port map   ( I0 => high, 
             I1 => tapIn, 
             I2 => low, 
             I3 => high, 
             O  => tap(0)
            );

l1 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(0), 
              I2 => low, 
              I3 => high, 
              O  => tap(1) 
             );
l2 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(1), 
              I2 => low, 
              I3 => high, 
              O  => tap(2) 
             );
l3 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(2), 
              I2 => low, 
              I3 => high, 
              O  => tap(3) 
             );
l4 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(3), 
              I2 => low, 
              I3 => high, 
              O  => tap(4) 
             );
l5 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(4), 
              I2 => low, 
              I3 => high, 
              O  => tap(5) 
             );
l6 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(5), 
              I2 => low, 
              I3 => high, 
              O  => tap(6) 
             );
l7 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(6), 
              I2 => low, 
              I3 => high, 
              O  => tap(7) 
             );
l8 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(7), 
              I2 => low, 
              I3 => high, 
              O  => tap(8) 
             );
l9 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(8), 
              I2 => low, 
              I3 => high, 
              O  => tap(9) 
             );
l10 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(9), 
              I2 => low, 
              I3 => high, 
              O  => tap(10) 
             );
l11 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(10), 
              I2 => low, 
              I3 => high, 
              O  => tap(11) 
             );
l12 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(11), 
              I2 => low, 
              I3 => high, 
              O  => tap(12) 
             );
l13 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(12), 
              I2 => low, 
              I3 => high, 
              O  => tap(13) 
             );
l14 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(13), 
              I2 => low, 
              I3 => high, 
              O  => tap(14) 
             );
l15 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(14), 
              I2 => low, 
              I3 => high, 
              O  => tap(15) 
             );
l16 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(15), 
              I2 => low, 
              I3 => high, 
              O  => tap(16) 
             );
l17 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(16), 
              I2 => low, 
              I3 => high, 
              O  => tap(17) 
             );
l18 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(17), 
              I2 => low, 
              I3 => high, 
              O  => tap(18) 
             );
l19 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(18), 
              I2 => low, 
              I3 => high, 
              O  => tap(19) 
             );
l20 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(19), 
              I2 => low, 
              I3 => high, 
              O  => tap(20) 
             );
l21 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(20), 
              I2 => low, 
              I3 => high, 
              O  => tap(21) 
             );
l22 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(21), 
              I2 => low, 
              I3 => high, 
              O  => tap(22) 
             );
l23 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(22), 
              I2 => low, 
              I3 => high, 
              O  => tap(23) 
             );
l24 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(23), 
              I2 => low, 
              I3 => high, 
              O  => tap(24) 
             );
l25 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(24), 
              I2 => low, 
              I3 => high, 
              O  => tap(25) 
             );
l26 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(25), 
              I2 => low, 
              I3 => high, 
              O  => tap(26) 
             );
l27 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(26), 
              I2 => low, 
              I3 => high, 
              O  => tap(27) 
             );
l28 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(27), 
              I2 => low, 
              I3 => high, 
              O  => tap(28) 
             );
l29 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(28), 
              I2 => low, 
              I3 => high, 
              O  => tap(29) 
             );
l30 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(29), 
              I2 => low, 
              I3 => high, 
              O  => tap(30) 
             );
l31 :  LUT4  generic map (INIT => x"e2e2")    
port map     ( 
              I0 => high, 
              I1 => tap(30), 
              I2 => low, 
              I3 => high, 
              O  => tap(31) 
             );
  

r0  : FDR port map  (
			   Q => flop1(0),
			   C => clk,
			   D => tap(0),
			   R => reset 	
			  );

r1  : FDR port map  (
			   Q => flop1(1),
			   C => clk,
			   D => tap(1),
			   R => reset 	
			  );

r2  : FDR port map  (
			   Q => flop1(2),
			   C => clk,
			   D => tap(2),
			   R => reset 	
			  );

r3  : FDR port map  (
			   Q => flop1(3),
			   C => clk,
			   D => tap(3),
			   R => reset 	
			  );

r4  : FDR port map  (
			   Q => flop1(4),
			   C => clk,
			   D => tap(4),
			   R => reset 	
			  );

r5  : FDR port map  (
			   Q => flop1(5),
			   C => clk,
			   D => tap(5),
			   R => reset 	
			  );

r6  : FDR port map  (
			   Q => flop1(6),
			   C => clk,
			   D => tap(6),
			   R => reset 	
			  );

r7  : FDR port map  (
			   Q => flop1(7),
			   C => clk,
			   D => tap(7),
			   R => reset 	
			  );

r8  : FDR port map  (
			   Q => flop1(8),
			   C => clk,
			   D => tap(8),
			   R => reset 	
			  );

r9  : FDR port map  (
			   Q => flop1(9),
			   C => clk,
			   D => tap(9),
			   R => reset 	
			  );

r10  : FDR port map  (
			   Q => flop1(10),
			   C => clk,
			   D => tap(10),
			   R => reset 	
			  );

r11  : FDR port map  (
			   Q => flop1(11),
			   C => clk,
			   D => tap(11),
			   R => reset 	
			  );

r12  : FDR port map  (
			   Q => flop1(12),
			   C => clk,
			   D => tap(12),
			   R => reset 	
			  );

r13  : FDR port map  (
			   Q => flop1(13),
			   C => clk,
			   D => tap(13),
			   R => reset 	
			  );

r14  : FDR port map  (
			   Q => flop1(14),
			   C => clk,
			   D => tap(14),
			   R => reset 	
			  );

r15  : FDR port map  (
			   Q => flop1(15),
			   C => clk,
			   D => tap(15),
			   R => reset 	
			  );

r16  : FDR port map  (
			   Q => flop1(16),
			   C => clk,
			   D => tap(16),
			   R => reset 	
			  );

r17  : FDR port map  (
			   Q => flop1(17),
			   C => clk,
			   D => tap(17),
			   R => reset 	
			  );

r18  : FDR port map  (
			   Q => flop1(18),
			   C => clk,
			   D => tap(18),
			   R => reset 	
			  );

r19  : FDR port map  (
			   Q => flop1(19),
			   C => clk,
			   D => tap(19),
			   R => reset 	
			  );

r20  : FDR port map  (
			   Q => flop1(20),
			   C => clk,
			   D => tap(20),
			   R => reset 	
			  );

r21  : FDR port map  (
			   Q => flop1(21),
			   C => clk,
			   D => tap(21),
			   R => reset 	
			  );

r22  : FDR port map  (
			   Q => flop1(22),
			   C => clk,
			   D => tap(22),
			   R => reset 	
			  );

r23  : FDR port map  (
			   Q => flop1(23),
			   C => clk,
			   D => tap(23),
			   R => reset 	
			  );

r24  : FDR port map  (
			   Q => flop1(24),
			   C => clk,
			   D => tap(24),
			   R => reset 	
			  );

r25  : FDR port map  (
			   Q => flop1(25),
			   C => clk,
			   D => tap(25),
			   R => reset 	
			  );

r26  : FDR port map  (
			   Q => flop1(26),
			   C => clk,
			   D => tap(26),
			   R => reset 	
			  );

r27  : FDR port map  (
			   Q => flop1(27),
			   C => clk,
			   D => tap(27),
			   R => reset 	
			  );

r28  : FDR port map  (
			   Q => flop1(28),
			   C => clk,
			   D => tap(28),
			   R => reset 	
			  );

r29  : FDR port map  (
			   Q => flop1(29),
			   C => clk,
			   D => tap(29),
			   R => reset 	
			  );

r30  : FDR port map  (
			   Q => flop1(30),
			   C => clk,
			   D => tap(30),
			   R => reset 	
			  );

r31  : FDR port map  (
			   Q => flop1(31),
			   C => clk,
			   D => tap(31),
			   R => reset 	
			  );

u0  : FDR port map  (
			   Q => flop2n_0,
			   C => clk,
			   D => flop1(0),
			   R => reset 	
			  );

u1  : FDR port map  (
			   Q => flop2(1),
			   C => clk,
			   D => flop1(1),
			   R => reset 	
			  );

u2  : FDR port map  (
			   Q => flop2n_2,
			   C => clk,
			   D => flop1(2),
			   R => reset 	
			  );

u3  : FDR port map  (
			   Q => flop2(3),
			   C => clk,
			   D => flop1(3),
			   R => reset 	
			  );

u4  : FDR port map  (
			   Q => flop2n_4,
			   C => clk,
			   D => flop1(4),
			   R => reset 	
			  );

u5  : FDR port map  (
			   Q => flop2(5),
			   C => clk,
			   D => flop1(5),
			   R => reset 	
			  );

u6  : FDR port map  (
			   Q => flop2n_6,
			   C => clk,
			   D => flop1(6),
			   R => reset 	
			  );

u7  : FDR port map  (
			   Q => flop2(7),
			   C => clk,
			   D => flop1(7),
			   R => reset 	
			  );

u8  : FDR port map  (
			   Q => flop2n_8,
			   C => clk,
			   D => flop1(8),
			   R => reset 	
			  );

u9  : FDR port map  (
			   Q => flop2(9),
			   C => clk,
			   D => flop1(9),
			   R => reset 	
			  );

u10  : FDR port map  (
			   Q => flop2n_10,
			   C => clk,
			   D => flop1(10),
			   R => reset 	
			  );

u11  : FDR port map  (
			   Q => flop2(11),
			   C => clk,
			   D => flop1(11),
			   R => reset 	
			  );

u12  : FDR port map  (
			   Q => flop2n_12,
			   C => clk,
			   D => flop1(12),
			   R => reset 	
			  );

u13  : FDR port map  (
			   Q => flop2(13),
			   C => clk,
			   D => flop1(13),
			   R => reset 	
			  );

u14  : FDR port map  (
			   Q => flop2n_14,
			   C => clk,
			   D => flop1(14),
			   R => reset 	
			  );

u15  : FDR port map  (
			   Q => flop2(15),
			   C => clk,
			   D => flop1(15),
			   R => reset 	
			  );

u16  : FDR port map  (
			   Q => flop2n_16,
			   C => clk,
			   D => flop1(16),
			   R => reset 	
			  );

u17  : FDR port map  (
			   Q => flop2(17),
			   C => clk,
			   D => flop1(17),
			   R => reset 	
			  );

u18  : FDR port map  (
			   Q => flop2n_18,
			   C => clk,
			   D => flop1(18),
			   R => reset 	
			  );

u19  : FDR port map  (
			   Q => flop2(19),
			   C => clk,
			   D => flop1(19),
			   R => reset 	
			  );

u20  : FDR port map  (
			   Q => flop2n_20,
			   C => clk,
			   D => flop1(20),
			   R => reset 	
			  );

u21  : FDR port map  (
			   Q => flop2(21),
			   C => clk,
			   D => flop1(21),
			   R => reset 	
			  );

u22  : FDR port map  (
			   Q => flop2n_22,
			   C => clk,
			   D => flop1(22),
			   R => reset 	
			  );

u23  : FDR port map  (
			   Q => flop2(23),
			   C => clk,
			   D => flop1(23),
			   R => reset 	
			  );

u24  : FDR port map  (
			   Q => flop2n_24,
			   C => clk,
			   D => flop1(24),
			   R => reset 	
			  );

u25  : FDR port map  (
			   Q => flop2(25),
			   C => clk,
			   D => flop1(25),
			   R => reset 	
			  );

u26  : FDR port map  (
			   Q => flop2n_26,
			   C => clk,
			   D => flop1(26),
			   R => reset 	
			  );

u27  : FDR port map  (
			   Q => flop2(27),
			   C => clk,
			   D => flop1(27),
			   R => reset 	
			  );

u28  : FDR port map  (
			   Q => flop2n_28,
			   C => clk,
			   D => flop1(28),
			   R => reset 	
			  );

u29  : FDR port map  (
			   Q => flop2(29),
			   C => clk,
			   D => flop1(29),
			   R => reset 	
			  );

u30  : FDR port map  (
			   Q => flop2n_30,
			   C => clk,
			   D => flop1(30),
			   R => reset 	
			  );

u31  : FDR port map  (
			   Q => flop2(31),
			   C => clk,
			   D => flop1(31),
			   R => reset 	
			  );			  
   

end arc_tap_dly;